@@ -4,8 +4,6 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: align.cpp 1471 2009-01-28 01:45:08Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -20,15 +18,18 @@
 
 
 static chunk_t *align_var_def_brace(chunk_t *pc, int span, int *nl_count);
-chunk_t *align_trailing_comments(chunk_t *start);
+static chunk_t *align_trailing_comments(chunk_t *start);
 static void align_init_brace(chunk_t *start);
 static void align_func_params();
 static void align_same_func_call_params();
 static void align_func_proto(int span);
 static void align_oc_msg_spec(int span);
-
-
 static void align_typedefs(int span);
+static void align_left_shift(void);
+static void align_oc_msg_colons();
+static void align_oc_msg_colon(chunk_t *so);
+static void align_oc_decl_colon(void);
+
 
 /*
  *   Here are the items aligned:
@@ -135,24 +136,17 @@ static void align_stack(ChunkStack& cs, int col, bool align_single, log_sev_t se
       LOG_FMT(sev, "%s: max_col=%d\n", __func__, col);
       while ((pc = cs.Pop()) != NULL)
       {
-         indent_to_column(pc, col);
+         align_to_column(pc, col);
          pc->flags |= PCF_WAS_ALIGNED;
 
-         if (pc->type == CT_NL_CONT)
-         {
-            LOG_FMT(sev, "%s: indented [NL_CONT] on line %d to %d\n",
-                    __func__, pc->orig_line, pc->column);
-         }
-         else
-         {
-            LOG_FMT(sev, "%s: indented [%.*s] on line %d to %d\n",
-                    __func__, pc->len, pc->str, pc->orig_line, pc->column);
-         }
+         LOG_FMT(sev, "%s: indented [%.*s] on line %d to %d\n",
+                 __func__, pc->len, pc->str, pc->orig_line, pc->column);
       }
    }
    cs.Reset();
 }
 
+
 /**
  * Adds an item to the align stack and adjust the nl_count and max_col.
  * Adjust max_col as needed
@@ -190,8 +184,9 @@ static void align_add(ChunkStack& cs, chunk_t *pc, int& max_col, int min_pad, bo
             min_col = pc->column;
          }
       }
-      LOG_FMT(LALADD, "%s: pc->col=%d max_col=%d min_pad=%d min_col=%d prev->col=%d\n",
-              __func__, pc->column, max_col, min_pad, min_col, prev->column);
+      LOG_FMT(LALADD, "%s: pc->col=%d max_col=%d min_pad=%d min_col=%d multi:%s prev->col=%d prev->len=%d %s\n",
+              __func__, pc->column, max_col, min_pad, min_col, (prev->type == CT_COMMENT_MULTI) ? "Y" : "N",
+              (prev->type == CT_COMMENT_MULTI) ? prev->orig_col_end : prev->column, prev->len, get_token_name(prev->type));
    }
 
    if (cs.Empty())
@@ -206,10 +201,11 @@ static void align_add(ChunkStack& cs, chunk_t *pc, int& max_col, int min_pad, bo
    }
 }
 
+
 void quick_align_again(void)
 {
-   chunk_t *pc;
-   chunk_t *tmp;
+   chunk_t    *pc;
+   chunk_t    *tmp;
    AlignStack as;
 
    LOG_FMT(LALAGAIN, "%s:\n", __func__);
@@ -225,8 +221,10 @@ void quick_align_again(void)
 
          LOG_FMT(LALAGAIN, "   [%.*s:%d]", pc->len, pc->str, pc->orig_line);
          as.Add(pc->align.start);
+         pc->flags |= PCF_WAS_ALIGNED;
          for (tmp = pc->align.next; tmp != NULL; tmp = tmp->align.next)
          {
+            tmp->flags |= PCF_WAS_ALIGNED;
             as.Add(tmp->align.start);
             LOG_FMT(LALAGAIN, " => [%.*s:%d]", tmp->len, tmp->str, tmp->orig_line);
          }
@@ -236,6 +234,7 @@ void quick_align_again(void)
    }
 }
 
+
 void align_all(void)
 {
    if (cpd.settings[UO_align_typedef_span].n > 0)
@@ -243,6 +242,21 @@ void align_all(void)
       align_typedefs(cpd.settings[UO_align_typedef_span].n);
    }
 
+   if (cpd.settings[UO_align_left_shift].b)
+   {
+      align_left_shift();
+   }
+
+   if (cpd.settings[UO_align_oc_msg_colon_span].n > 0)
+   {
+      align_oc_msg_colons();
+   }
+
+   if (cpd.settings[UO_align_oc_decl_colon].b)
+   {
+      align_oc_decl_colon();
+   }
+
    /* Align variable definitions */
    if ((cpd.settings[UO_align_var_def_span].n > 0) ||
        (cpd.settings[UO_align_var_struct_span].n > 0))
@@ -255,7 +269,7 @@ void align_all(void)
                 cpd.settings[UO_align_assign_span].n,
                 cpd.settings[UO_align_assign_thresh].n);
 
-   /* Align structure initiailizers */
+   /* Align structure initializers */
    if (cpd.settings[UO_align_struct_init_span].n > 0)
    {
       align_struct_initializers();
@@ -288,6 +302,7 @@ void align_all(void)
    quick_align_again();
 }
 
+
 /**
  * Aligns all function prototypes in the file.
  */
@@ -313,6 +328,7 @@ static void align_oc_msg_spec(int span)
    as.End();
 }
 
+
 /**
  * Aligns all backslash-newline combos in the file.
  * This should be done LAST.
@@ -333,6 +349,7 @@ void align_backslash_newline(void)
    }
 }
 
+
 void align_right_comments(void)
 {
    chunk_t *pc;
@@ -341,16 +358,19 @@ void align_right_comments(void)
 
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
    {
-      if ((pc->type == CT_COMMENT) || (pc->type == CT_COMMENT_CPP))
+      if ((pc->type == CT_COMMENT) ||
+          (pc->type == CT_COMMENT_CPP) ||
+          (pc->type == CT_COMMENT_MULTI))
       {
          skip = false;
          if (pc->parent_type == CT_COMMENT_END)
          {
             prev = chunk_get_prev(pc);
-            if (pc->orig_col <= (prev->orig_col_end + cpd.settings[UO_align_right_cmt_gap].n))
+            if (pc->orig_col < (prev->orig_col_end + cpd.settings[UO_align_right_cmt_gap].n))
             {
-               LOG_FMT(LALTC, "NOT changing END comment on line %d\n",
-                       pc->orig_line);
+               LOG_FMT(LALTC, "NOT changing END comment on line %d (%d <= %d + %d)\n",
+                       pc->orig_line,
+                       pc->orig_col, prev->orig_col_end, cpd.settings[UO_align_right_cmt_gap].n);
                skip = true;
             }
             if (!skip)
@@ -367,7 +387,7 @@ void align_right_comments(void)
             int tmp_col = 1 + (pc->brace_level * cpd.settings[UO_indent_columns].n);
 
             /* If the comment is further right than the brace level... */
-            if (pc->column > (tmp_col + cpd.settings[UO_align_right_cmt_gap].n))
+            if (pc->column >= (tmp_col + cpd.settings[UO_input_tab_size].n))
             {
                LOG_FMT(LALTC, "Changing WHOLE comment on line %d into a RIGHT-comment\n",
                        pc->orig_line);
@@ -392,6 +412,7 @@ void align_right_comments(void)
    }
 }
 
+
 /**
  * Aligns stuff inside a multi-line "= { ... }" sequence.
  */
@@ -414,6 +435,7 @@ void align_struct_initializers(void)
    }
 }
 
+
 /**
  * Scans the whole file for #defines. Aligns all within X lines of each other
  */
@@ -492,6 +514,7 @@ void align_preprocessor(void)
    asf.End();
 }
 
+
 /**
  * Aligns all assignment operators on the same level as first, starting with
  * first.
@@ -631,6 +654,7 @@ chunk_t *align_assign(chunk_t *first, int span, int thresh)
    return(pc);
 }
 
+
 /**
  * Counts how many '*' pointers are in a row, going backwards
  *
@@ -649,6 +673,7 @@ int count_prev_ptr_type(chunk_t *pc)
    return(count);
 }
 
+
 static chunk_t *align_func_param(chunk_t *start)
 {
    AlignStack as;
@@ -661,13 +686,16 @@ static chunk_t *align_func_param(chunk_t *start)
 
    bool did_this_line = false;
    int  comma_count   = 0;
+   int  chunk_count   = 0;
 
    while ((pc = chunk_get_next(pc)) != NULL)
    {
+      chunk_count++;
       if (chunk_is_newline(pc))
       {
          did_this_line = false;
          comma_count   = 0;
+         chunk_count   = 0;
       }
       else if (pc->level <= start->level)
       {
@@ -675,7 +703,10 @@ static chunk_t *align_func_param(chunk_t *start)
       }
       else if (!did_this_line && (pc->flags & PCF_VAR_DEF))
       {
-         as.Add(pc);
+         if (chunk_count > 1)
+         {
+            as.Add(pc);
+         }
          did_this_line = true;
       }
       else if (comma_count > 0)
@@ -700,6 +731,7 @@ static chunk_t *align_func_param(chunk_t *start)
    return(pc);
 }
 
+
 static void align_func_params()
 {
    chunk_t *pc;
@@ -721,6 +753,7 @@ static void align_func_params()
    }
 }
 
+
 static int align_params(chunk_t *start, chunk_t *chunks[], int num_chunks)
 {
    int     count     = 0;
@@ -744,7 +777,7 @@ static int align_params(chunk_t *start, chunk_t *chunks[], int num_chunks)
          if (hit_comma)
          {
             chunks[count++] = pc;
-            hit_comma   = false;
+            hit_comma       = false;
          }
          else if (pc->type == CT_COMMA)
          {
@@ -755,12 +788,13 @@ static int align_params(chunk_t *start, chunk_t *chunks[], int num_chunks)
    return(count);
 }
 
+
 static void align_same_func_call_params()
 {
-   chunk_t *pc;
-   chunk_t *align_root = NULL;
-   chunk_t *align_cur  = NULL;
-   int     align_len   = 0;
+   chunk_t    *pc;
+   chunk_t    *align_root = NULL;
+   chunk_t    *align_cur  = NULL;
+   int        align_len   = 0;
    chunk_t    *chunks[16];
    AlignStack as[16];
    AlignStack fcn_as;
@@ -786,6 +820,12 @@ static void align_same_func_call_params()
          continue;
       }
 
+      /* Only align function calls that are right after a newline */
+      if (!chunk_is_newline(chunk_get_prev(pc)))
+      {
+         continue;
+      }
+
       fcn_as.Add(pc);
       add_str = NULL;
 
@@ -817,7 +857,7 @@ static void align_same_func_call_params()
          align_root = pc;
          align_cur  = pc;
          align_len  = 1;
-         add_str = "Start";
+         add_str    = "Start";
       }
 
       if (add_str != NULL)
@@ -857,6 +897,7 @@ static void align_same_func_call_params()
    }
 }
 
+
 /**
  * Aligns all function prototypes in the file.
  */
@@ -910,6 +951,7 @@ static void align_func_proto(int span)
    as_br.End();
 }
 
+
 /**
  * Scan everything at the current level until the close brace and find the
  * variable def align column.  Also aligns bit-colons, but that assumes that
@@ -920,12 +962,13 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span, int *p_nl_count)
    chunk_t    *pc;
    chunk_t    *next;
    chunk_t    *prev;
-   int        align_mask = PCF_IN_FCN_DEF | PCF_VAR_1ST;
+   UINT64     align_mask = PCF_IN_FCN_DEF | PCF_VAR_1ST;
    int        myspan     = span;
    int        mythresh   = 0;
    int        mygap      = 0;
    AlignStack as;    /* var/proto/def */
    AlignStack as_bc; /* bit-colon */
+   AlignStack as_at; /* attribute */
    AlignStack as_br; /* one-liner brace open */
    bool       fp_active   = cpd.settings[UO_align_mix_var_proto].b;
    bool       fp_look_bro = false;
@@ -979,6 +1022,8 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span, int *p_nl_count)
    as_bc.Start(myspan, 0);
    as_bc.m_gap = cpd.settings[UO_align_var_def_colon_gap].n;
 
+   as_at.Start(myspan, 0);
+
    /* Set up the brace open aligner */
    as_br.Start(myspan, mythresh);
    as_br.m_gap = cpd.settings[UO_align_single_line_brace_gap].n;
@@ -993,6 +1038,7 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span, int *p_nl_count)
          {
             as.NewLines(pc->nl_count);
             as_bc.NewLines(pc->nl_count);
+            as_at.NewLines(pc->nl_count);
             as_br.NewLines(pc->nl_count);
          }
          pc = chunk_get_next(pc);
@@ -1033,6 +1079,7 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span, int *p_nl_count)
             did_this_line = false;
             as.NewLines(sub_nl_count);
             as_bc.NewLines(sub_nl_count);
+            as_at.NewLines(sub_nl_count);
             as_br.NewLines(sub_nl_count);
             if (p_nl_count != NULL)
             {
@@ -1055,6 +1102,7 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span, int *p_nl_count)
          did_this_line = false;
          as.NewLines(pc->nl_count);
          as_bc.NewLines(pc->nl_count);
+         as_at.NewLines(pc->nl_count);
          as_br.NewLines(pc->nl_count);
          if (p_nl_count != NULL)
          {
@@ -1090,6 +1138,22 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span, int *p_nl_count)
                   as_bc.Add(next);
                }
             }
+            if (cpd.settings[UO_align_var_def_attribute].b)
+            {
+               next = pc;
+               while ((next = chunk_get_next_nc(next)) != NULL)
+               {
+                  if (next->type == CT_ATTRIBUTE)
+                  {
+                     as_at.Add(next);
+                     break;
+                  }
+                  if ((next->type == CT_SEMICOLON) || chunk_is_newline(next))
+                  {
+                     break;
+                  }
+               }
+            }
          }
          did_this_line = true;
       }
@@ -1106,11 +1170,13 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span, int *p_nl_count)
 
    as.End();
    as_bc.End();
+   as_at.End();
    as_br.End();
 
    return(pc);
 }
 
+
 /**
  * For a series of lines ending in backslash-newline, align them.
  * The series ends when a newline or multi-line C comment is encountered.
@@ -1140,18 +1206,17 @@ chunk_t *align_nl_cont(chunk_t *start)
       pc = chunk_get_next(pc);
    }
 
-   /* shift out to the next tabstop */
-   max_col = align_tab_column(max_col);
-
    /* NL_CONT is always the last thing on a line */
    while ((tmp = cs.Pop()) != NULL)
    {
+      tmp->flags |= PCF_WAS_ALIGNED;
       tmp->column = max_col;
    }
 
    return(pc);
 }
 
+
 enum CmtAlignType
 {
    CAT_REGULAR,
@@ -1169,6 +1234,7 @@ static CmtAlignType get_comment_align_type(chunk_t *cmt)
    {
       if ((prev->type == CT_PP_ENDIF) ||
           (prev->type == CT_PP_ELSE) ||
+          (prev->type == CT_ELSE) ||
           (prev->type == CT_BRACE_CLOSE))
       {
          /* REVISIT: someone may want this configurable */
@@ -1181,43 +1247,59 @@ static CmtAlignType get_comment_align_type(chunk_t *cmt)
    return(cmt_type);
 }
 
+
 /**
  * For a series of lines ending in a comment, align them.
  * The series ends when more than align_right_cmt_span newlines are found.
  *
+ * Interesting info:
+ *  - least pysically allowed column
+ *  - intended column
+ *  - least original cmt column
+ *
+ * min_col is the minimum allowed column (based on prev token col/size)
+ * cmt_col less than
+ *
  * @param start   Start point
  * @return        pointer the last item looked at
  */
 chunk_t *align_trailing_comments(chunk_t *start)
 {
-   int          max_col  = 0;
+   int          min_col  = 0;
+   int          min_orig = -1;
    chunk_t      *pc      = start;
    int          nl_count = 0;
    ChunkStack   cs;
    CmtAlignType cmt_type_start, cmt_type_cur;
-   int          last_col, col;
+   int          col;
+   int          intended_col = cpd.settings[UO_align_right_cmt_at_col].n;
 
    cmt_type_start = get_comment_align_type(pc);
 
-   last_col = pc->column;
+   LOG_FMT(LALADD, "%s: start on line=%d\n", __func__, pc->orig_line);
 
    /* Find the max column */
    while ((pc != NULL) && (nl_count < cpd.settings[UO_align_right_cmt_span].n))
    {
-      if ((pc->flags & PCF_RIGHT_COMMENT) != 0)
+      if (((pc->flags & PCF_RIGHT_COMMENT) != 0) && (pc->column > 1))
       {
          cmt_type_cur = get_comment_align_type(pc);
 
          if (cmt_type_cur == cmt_type_start)
          {
-            last_col = pc->column;
-
             col = 1 + (pc->brace_level * cpd.settings[UO_indent_columns].n);
+            LOG_FMT(LALADD, "%s: line=%d col=%d max_col=%d pc->col=%d pc->len=%d %s\n",
+                    __func__, pc->orig_line, col, min_col, pc->column, pc->len,
+                    get_token_name(pc->type));
+            if ((min_orig < 0) || (min_orig > pc->column))
+            {
+               min_orig = pc->column;
+            }
             if (pc->column < col)
             {
                pc->column = col;
             }
-            align_add(cs, pc, max_col, 1, false);
+            align_add(cs, pc, min_col, 1, true);//(intended_col < col));
             nl_count = 0;
          }
       }
@@ -1228,11 +1310,31 @@ chunk_t *align_trailing_comments(chunk_t *start)
       pc = chunk_get_next(pc);
    }
 
-   align_stack(cs, max_col, false, LALTC);
+   /* Start with the minimum original column */
+   col = min_orig;
+   /* fall back to the intended column */
+   if ((intended_col > 0) && (col > intended_col))
+   {
+      col = intended_col;
+   }
+   /* if less than allowed, bump it out */
+   if (col < min_col)
+   {
+      col = min_col;
+   }
+   /* bump out to the intended column */
+   if (col < intended_col)
+   {
+      col = intended_col;
+   }
+   LOG_FMT(LALADD, "%s:  -- min_orig=%d intend=%d min_allowed=%d ==> col=%d\n", __func__,
+           min_orig, intended_col, min_col, col);
+   align_stack(cs, col, (intended_col != 0), LALTC);
 
    return(chunk_get_next(pc));
 }
 
+
 /**
  * Shifts out all columns by a certain amount.
  *
@@ -1248,6 +1350,7 @@ void ib_shift_out(int idx, int num)
    }
 }
 
+
 /**
  * Scans a line for stuff to align on.
  *
@@ -1260,7 +1363,7 @@ static chunk_t *scan_ib_line(chunk_t *start, bool first_pass)
    chunk_t *next;
    chunk_t *prev_match = NULL;
    int     token_width;
-   int     idx              = 0;
+   int     idx = 0;
    bool    last_was_comment = false;
 
    /* Skip past C99 "[xx] =" stuff */
@@ -1360,6 +1463,7 @@ static chunk_t *scan_ib_line(chunk_t *start, bool first_pass)
    return(pc);
 }
 
+
 static void align_log_al(log_sev_t sev, int line)
 {
    int idx;
@@ -1376,6 +1480,7 @@ static void align_log_al(log_sev_t sev, int line)
    }
 }
 
+
 /**
  * Generically aligns on '=', '{', '(' and item after ','
  * It scans the first line and picks up the location of those tags.
@@ -1567,6 +1672,7 @@ static void align_init_brace(chunk_t *start)
    } while ((pc != NULL) && (pc->level > start->level));
 }
 
+
 /**
  * Aligns simple typedefs that are contained on a single line each.
  * This should be called after the typedef target is marked as a type.
@@ -1621,3 +1727,221 @@ static void align_typedefs(int span)
 
    as.End();
 }
+
+
+/**
+ * Align '<<' (CT_ARITH?)
+ */
+static void align_left_shift(void)
+{
+   chunk_t    *pc;
+   chunk_t    *start = NULL;
+   AlignStack as;
+
+   as.Start(2);
+
+   pc = chunk_get_head();
+   while (pc != NULL)
+   {
+      if (chunk_is_newline(pc))
+      {
+         as.NewLines(pc->nl_count);
+      }
+      else if ((start != NULL) && (pc->level < start->level))
+      {
+         /* A drop in level restarts the aligning */
+         as.Flush();
+         start = NULL;
+      }
+      else if ((start != NULL) && (pc->level > start->level))
+      {
+         /* Ignore any deeper levels when aligning */
+      }
+      else if (pc->type == CT_SEMICOLON)
+      {
+         /* A semicolon at the same level flushes */
+         as.Flush();
+         start = NULL;
+      }
+      else if (chunk_is_str(pc, "<<", 2))
+      {
+         if (as.m_aligned.Empty())
+         {
+            /* first one can be anywhere */
+            as.Add(pc);
+            start = pc;
+         }
+         else if (chunk_is_newline(chunk_get_prev(pc)))
+         {
+            /* subsequent ones must be after a newline */
+            as.Add(pc);
+         }
+      }
+
+      pc = chunk_get_next(pc);
+   }
+   as.End();
+}
+
+
+/**
+ * Aligns an OC message
+ *
+ * @param so   the square open of the message
+ * @param span the span value
+ */
+static void align_oc_msg_colon(chunk_t *so)
+{
+   int        span = cpd.settings[UO_align_oc_msg_colon_span].n;
+   chunk_t    *pc;
+   chunk_t    *tmp;
+   AlignStack cas;   /* for the colons */
+   AlignStack nas;   /* for the parameter tag */
+   int        level;
+   bool       did_line;
+   int        lcnt;  /* line count with no colon for span */
+   bool       has_colon;
+
+
+   nas.Reset();
+   nas.m_right_align = true;
+
+   cas.Start(span);
+
+   level = so->level;
+   pc    = chunk_get_next_ncnl(so, CNAV_PREPROC);
+
+   did_line  = false;
+   has_colon = false;
+   lcnt      = 0;
+
+   while ((pc != NULL) && (pc->level > level))
+   {
+      if (pc->level > (level + 1))
+      {
+         /* do nothing */
+      }
+      else if (chunk_is_newline(pc))
+      {
+         if (!has_colon)
+         {
+            ++lcnt;
+         }
+         did_line  = false;
+         has_colon = !has_colon;
+      }
+      else if (!did_line && (lcnt - 1 < span) && (pc->type == CT_OC_COLON))
+      {
+         has_colon = true;
+         cas.Add(pc);
+         tmp = chunk_get_prev(pc);
+         if ((tmp != NULL) &&
+             ((tmp->type == CT_OC_MSG_FUNC) ||
+              (tmp->type == CT_OC_MSG_NAME)))
+         {
+            nas.Add(tmp);
+         }
+         did_line = true;
+      }
+      pc = chunk_get_next(pc, CNAV_PREPROC);
+   }
+   nas.End();
+   cas.End();
+}
+
+
+/**
+ * Aligns OC messages
+ */
+static void align_oc_msg_colons()
+{
+   chunk_t *pc;
+
+   for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
+   {
+      if ((pc->type == CT_SQUARE_OPEN) && (pc->parent_type == CT_OC_MSG))
+      {
+         align_oc_msg_colon(pc);
+      }
+   }
+}
+
+
+/**
+ * Aligns OC declarations on the colon
+ * -(void) doSomething: (NSString*) param1
+ *                with: (NSString*) param2
+ */
+static void align_oc_decl_colon(void)
+{
+   chunk_t    *pc = chunk_get_head();
+   chunk_t    *tmp;
+   chunk_t    *tmp2;
+   AlignStack cas;   /* for the colons */
+   AlignStack nas;   /* for the parameter label */
+   int        level;
+   bool       did_line;
+
+   cas.Start(4);
+   nas.Start(4);
+   nas.m_right_align = true;
+
+   while (pc != NULL)
+   {
+      if (pc->type != CT_OC_SCOPE)
+      {
+         pc = chunk_get_next(pc);
+         continue;
+      }
+
+      nas.Reset();
+      cas.Reset();
+
+      level = pc->level;
+      pc    = chunk_get_next_ncnl(pc, CNAV_PREPROC);
+
+      did_line = false;
+
+      while ((pc != NULL) && (pc->level >= level))
+      {
+         /* The decl ends with an open brace or semicolon */
+         if ((pc->type == CT_BRACE_OPEN) || chunk_is_semicolon(pc))
+         {
+            break;
+         }
+
+         if (chunk_is_newline(pc))
+         {
+            nas.NewLines(pc->nl_count);
+            cas.NewLines(pc->nl_count);
+            did_line  = false;
+         }
+         else if (!did_line && (pc->type == CT_OC_COLON))
+         {
+            cas.Add(pc);
+
+            tmp = chunk_get_prev(pc, CNAV_PREPROC);
+            tmp2 = chunk_get_prev_ncnl(tmp, CNAV_PREPROC);
+
+            /* Check for an un-labeled parameter */
+            if ((tmp != NULL) &&
+                (tmp2 != NULL)
+                &&
+                ((tmp->type == CT_WORD) ||
+                 (tmp->type == CT_TYPE) ||
+                 (tmp->type == CT_OC_MSG_DECL))
+                &&
+                ((tmp2->type == CT_WORD) ||
+                 (tmp2->type == CT_TYPE) ||
+                 (tmp2->type == CT_PAREN_CLOSE)))
+            {
+               nas.Add(tmp);
+            }
+            did_line = true;
+         }
+         pc = chunk_get_next(pc, CNAV_PREPROC);
+      }
+      nas.End();
+      cas.End();
+   }
+}
