@@ -6,10 +6,7 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: align_stack.cpp 1329 2008-08-16 17:32:13Z bengardner $
  */
-
 #include "align_stack.h"
 #include "prototypes.h"
 #include "chunk_list.h"
@@ -38,6 +35,7 @@ void AlignStack::Start(int span, int thresh)
    m_amp_style   = SS_IGNORE;
 }
 
+
 /**
  * Calls Add on all the skipped items
  */
@@ -64,6 +62,7 @@ void AlignStack::ReAddSkipped()
    }
 }
 
+
 /**
  * Adds an entry to the appropriate stack.
  *
@@ -155,8 +154,15 @@ void AlignStack::Add(chunk_t *start, int seqnum)
        * In the above examples, that is 'void', 'char', 'foomatic', 'A', or 'B'.
        *
        * The '*' and '&' can float between the two.
+       *
+       * If align_on_tabstop=true, then SS_DANGLE is changed to SS_INCLUDE.
        */
 
+      if (cpd.settings[UO_align_on_tabstop].b && (m_star_style == SS_DANGLE))
+      {
+         m_star_style = SS_INCLUDE;
+      }
+
       /* Find ref. Back up to the real item that is aligned. */
       prev = start;
       while (((prev = chunk_get_prev(prev)) != NULL) &&
@@ -298,6 +304,7 @@ void AlignStack::Add(chunk_t *start, int seqnum)
    }
 }
 
+
 /**
  * Adds some newline and calls Flush() if needed
  */
@@ -318,6 +325,7 @@ void AlignStack::NewLines(int cnt)
    }
 }
 
+
 /**
  * Aligns all the stuff in m_aligned.
  * Re-adds 'newer' items in m_skipped.
@@ -328,7 +336,7 @@ void AlignStack::Flush()
    int idx;
    int tmp_col;
    const ChunkStack::Entry *ce = NULL;
-   chunk_t *pc;
+   chunk_t                 *pc;
 
    LOG_FMT(LAS, "Flush (min=%d, max=%d)\n", m_min_col, m_max_col);
 
@@ -387,6 +395,16 @@ void AlignStack::Flush()
       }
    }
 
+   if (cpd.settings[UO_align_on_tabstop].b && (m_aligned.Len() > 1))
+   {
+      int rem = (m_max_col - 1) % cpd.settings[UO_output_tab_size].n;
+      if (rem != 0)
+      {
+         LOG_FMT(LAS, "%s: align_on_tabstop col=%d rem=%d", __func__, m_max_col, rem);
+         m_max_col += cpd.settings[UO_output_tab_size].n - rem;
+      }
+   }
+
    for (idx = 0; idx < m_aligned.Len(); idx++)
    {
       ce = m_aligned.Get(idx);
@@ -439,6 +457,17 @@ void AlignStack::Flush()
    }
 }
 
+
+/**
+ * Resets the stack, discarding anything that was previously added
+ */
+void AlignStack::Reset()
+{
+   m_aligned.Reset();
+   m_skipped.Reset();
+}
+
+
 /**
  * Aligns everything else and resets the lists.
  */
