@@ -6,10 +6,7 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: brace_cleanup.cpp 1313 2008-08-01 18:42:10Z bengardner $
  */
-
 #include "uncrustify_types.h"
 #include "char_table.h"
 #include "prototypes.h"
@@ -24,8 +21,8 @@
 static chunk_t *insert_vbrace(chunk_t *pc, bool after,
                               struct parse_frame *frm);
 
-#define insert_vbrace_close_after(pc, frm)     insert_vbrace(pc, true, frm)
-#define insert_vbrace_open_before(pc, frm)     insert_vbrace(pc, false, frm)
+#define insert_vbrace_close_after(pc, frm)    insert_vbrace(pc, true, frm)
+#define insert_vbrace_open_before(pc, frm)    insert_vbrace(pc, false, frm)
 
 static void parse_cleanup(struct parse_frame *frm, chunk_t *pc);
 
@@ -72,6 +69,7 @@ static int preproc_start(struct parse_frame *frm, chunk_t *pc)
    return(pp_level);
 }
 
+
 static void print_stack(int logsev, const char *str,
                         struct parse_frame *frm, chunk_t *pc)
 {
@@ -97,6 +95,7 @@ static void print_stack(int logsev, const char *str,
    }
 }
 
+
 /**
  * Scans through the whole list and does stuff.
  * It has to do some tricks to parse preprocessors.
@@ -108,7 +107,7 @@ void brace_cleanup(void)
    chunk_t            *pc;
    chunk_t            vs_chunk;
    struct parse_frame frm;
-   int pp_level;
+   int                pp_level;
 
    memset(&frm, 0, sizeof(frm));
    memset(&vs_chunk, 0, sizeof(vs_chunk));
@@ -170,6 +169,7 @@ void brace_cleanup(void)
    }
 }
 
+
 /**
  * pc is a CT_WHILE.
  * Scan backwards to see if we find a brace/vbrace with the parent set to CT_DO
@@ -200,10 +200,11 @@ static bool maybe_while_of_do(chunk_t *pc)
    return(false);
 }
 
+
 static void push_fmr_pse(struct parse_frame *frm, chunk_t *pc,
                          brstage_e stage, const char *logtext)
 {
-   if (frm->pse_tos < (int)ARRAY_SIZE(frm->pse))
+   if (frm->pse_tos < ((int)ARRAY_SIZE(frm->pse) - 1))
    {
       frm->pse_tos++;
       frm->pse[frm->pse_tos].type  = pc->type;
@@ -220,6 +221,7 @@ static void push_fmr_pse(struct parse_frame *frm, chunk_t *pc,
    }
 }
 
+
 /**
  * At the heart of this algorithm are two stacks.
  * There is the Paren Stack (PS) and the Frame stack.
@@ -227,7 +229,7 @@ static void push_fmr_pse(struct parse_frame *frm, chunk_t *pc,
  * The PS (pse in the code) keeps track of braces, parens,
  * if/else/switch/do/while/etc items -- anything that is nestable.
  * Complex statements go through stages.
- * Take this simple if statment as an example:
+ * Take this simple if statement as an example:
  *   if ( x ) { x--; }
  *
  * The stack would change like so: 'token' stack afterwards
@@ -256,7 +258,7 @@ static void push_fmr_pse(struct parse_frame *frm, chunk_t *pc,
  * 'x'    [ELSE - 0] [VBRACE OPEN] <- lack of '{' -> VBrace
  * '++'   [ELSE - 0] [VBRACE OPEN]
  * ';'    [ELSE - 0]               <- VBrace close inserted after semicolon
- *                                 <- ELSE removed after statment close
+ *                                 <- ELSE removed after statement close
  *
  * The pse stack is kept on a frame stack.
  * The frame stack is need for languages that support preprocessors (C, C++, C#)
@@ -538,7 +540,7 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
 
    pattern_class patcls = get_token_pattern_class(pc->type);
 
-   /** Create a stack entry for complex statments IF/DO/FOR/WHILE/SWITCH */
+   /** Create a stack entry for complex statements IF/DO/FOR/WHILE/SWITCH */
    if (patcls == PATCLS_BRACED)
    {
       push_fmr_pse(frm, pc,
@@ -589,6 +591,7 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
    /* Mark expression starts */
    if ((pc->type == CT_ARITH) ||
        (pc->type == CT_ASSIGN) ||
+       (pc->type == CT_CASE) ||
        (pc->type == CT_COMPARE) ||
        (pc->type == CT_STAR) ||
        (pc->type == CT_BOOL) ||
@@ -615,6 +618,7 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
    }
 }
 
+
 /**
  * Checks the progression of complex statements.
  * - checks for else after if
@@ -725,7 +729,7 @@ static bool check_complex_statements(struct parse_frame *frm, chunk_t *pc)
    {
       parent = frm->pse[frm->pse_tos].type;
 
-      vbrace              = insert_vbrace_open_before(pc, frm);
+      vbrace = insert_vbrace_open_before(pc, frm);
       vbrace->parent_type = parent;
 
       frm->level++;
@@ -765,6 +769,7 @@ static bool check_complex_statements(struct parse_frame *frm, chunk_t *pc)
    return(false);
 }
 
+
 /**
  * Handles a close paren or brace - just progress the stage, if the end
  * of the statement is hit, call close_statement()
@@ -868,6 +873,7 @@ static bool handle_complex_close(struct parse_frame *frm, chunk_t *pc)
    return(false);
 }
 
+
 static chunk_t *insert_vbrace(chunk_t *pc, bool after,
                               struct parse_frame *frm)
 {
@@ -928,6 +934,7 @@ static chunk_t *insert_vbrace(chunk_t *pc, bool after,
    return(rv);
 }
 
+
 /**
  * Called when a statement was just closed and the pse_tos was just
  * decremented.
@@ -968,8 +975,8 @@ bool close_statement(struct parse_frame *frm, chunk_t *pc)
       else
       {
          /* otherwise, add before it and consume the vbrace */
-         vbc              = chunk_get_prev_ncnl(pc);
-         vbc              = insert_vbrace_close_after(vbc, frm);
+         vbc = chunk_get_prev_ncnl(pc);
+         vbc = insert_vbrace_close_after(vbc, frm);
          vbc->parent_type = frm->pse[frm->pse_tos].parent;
 
          frm->level--;
