@@ -4,10 +4,7 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: chunk_list.cpp 1313 2008-08-01 18:42:10Z bengardner $
  */
-
 #include "chunk_list.h"
 #include <cstring>
 #include <cstdlib>
@@ -19,30 +16,77 @@ typedef ListManager<chunk_t>   ChunkList;
 
 ChunkList g_cl;
 
-static chunk_t *chunk_dup(const chunk_t *pc_in);
-
-
 chunk_t *chunk_get_head(void)
 {
    return(g_cl.GetHead());
 }
 
+
 chunk_t *chunk_get_tail(void)
 {
    return(g_cl.GetTail());
 }
 
-chunk_t *chunk_get_next(chunk_t *cur)
+
+chunk_t *chunk_get_next(chunk_t *cur, chunk_nav_t nav)
 {
-   return(g_cl.GetNext(cur));
+   if (cur == NULL)
+   {
+      return(NULL);
+   }
+   chunk_t *pc = g_cl.GetNext(cur);
+   if ((pc == NULL) || (nav == CNAV_ALL))
+   {
+      return(pc);
+   }
+   if (cur->flags & PCF_IN_PREPROC)
+   {
+      /* If in a preproc, return NULL if trying to leave */
+      if ((pc->flags & PCF_IN_PREPROC) == 0)
+      {
+         return(NULL);
+      }
+      return(pc);
+   }
+   /* Not in a preproc, skip any proproc */
+   while ((pc != NULL) && (pc->flags & PCF_IN_PREPROC))
+   {
+      pc = g_cl.GetNext(pc);
+   }
+   return(pc);
 }
 
-chunk_t *chunk_get_prev(chunk_t *cur)
+
+chunk_t *chunk_get_prev(chunk_t *cur, chunk_nav_t nav)
 {
-   return(g_cl.GetPrev(cur));
+   if (cur == NULL)
+   {
+      return(NULL);
+   }
+   chunk_t *pc = g_cl.GetPrev(cur);
+   if ((pc == NULL) || (nav == CNAV_ALL))
+   {
+      return(pc);
+   }
+   if (cur->flags & PCF_IN_PREPROC)
+   {
+      /* If in a preproc, return NULL if trying to leave */
+      if ((pc->flags & PCF_IN_PREPROC) == 0)
+      {
+         return(NULL);
+      }
+      return(pc);
+   }
+   /* Not in a preproc, skip any proproc */
+   while ((pc != NULL) && (pc->flags & PCF_IN_PREPROC))
+   {
+      pc = g_cl.GetPrev(pc);
+   }
+   return(pc);
 }
 
-static chunk_t *chunk_dup(const chunk_t *pc_in)
+
+chunk_t *chunk_dup(const chunk_t *pc_in)
 {
    chunk_t *pc;
 
@@ -60,6 +104,7 @@ static chunk_t *chunk_dup(const chunk_t *pc_in)
    return(pc);
 }
 
+
 /**
  * Add to the tail of the list
  */
@@ -74,6 +119,7 @@ chunk_t *chunk_add(const chunk_t *pc_in)
    return(pc);
 }
 
+
 /**
  * Add a copy after the given chunk.
  * If ref is NULL, add at the head.
@@ -96,6 +142,7 @@ chunk_t *chunk_add_after(const chunk_t *pc_in, chunk_t *ref)
    return(pc);
 }
 
+
 /**
  * Add a copy before the given chunk.
  * If ref is NULL, add at the head.
@@ -118,6 +165,7 @@ chunk_t *chunk_add_before(const chunk_t *pc_in, chunk_t *ref)
    return(pc);
 }
 
+
 void chunk_del(chunk_t *pc)
 {
    g_cl.Pop(pc);
@@ -129,6 +177,7 @@ void chunk_del(chunk_t *pc)
    delete pc;
 }
 
+
 void chunk_move_after(chunk_t *pc_in, chunk_t *ref)
 {
    g_cl.Pop(pc_in);
@@ -140,80 +189,86 @@ void chunk_move_after(chunk_t *pc_in, chunk_t *ref)
    pc_in->orig_col_end = pc_in->orig_col + pc_in->len;
 }
 
+
 /**
  * Gets the next NEWLINE chunk
  */
-chunk_t *chunk_get_next_nl(chunk_t *cur)
+chunk_t *chunk_get_next_nl(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_next(pc);
+      pc = chunk_get_next(pc, nav);
    } while ((pc != NULL) && !chunk_is_newline(pc));
    return(pc);
 }
 
+
 /**
  * Gets the prev NEWLINE chunk
  */
-chunk_t *chunk_get_prev_nl(chunk_t *cur)
+chunk_t *chunk_get_prev_nl(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_prev(pc);
+      pc = chunk_get_prev(pc, nav);
    } while ((pc != NULL) && !chunk_is_newline(pc));
    return(pc);
 }
 
+
 /**
  * Gets the next non-NEWLINE chunk
  */
-chunk_t *chunk_get_next_nnl(chunk_t *cur)
+chunk_t *chunk_get_next_nnl(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_next(pc);
+      pc = chunk_get_next(pc, nav);
    } while (chunk_is_newline(pc));
    return(pc);
 }
 
+
 /**
  * Gets the prev non-NEWLINE chunk
  */
-chunk_t *chunk_get_prev_nnl(chunk_t *cur)
+chunk_t *chunk_get_prev_nnl(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_prev(pc);
+      pc = chunk_get_prev(pc, nav);
    } while ((pc != NULL) && chunk_is_newline(pc));
    return(pc);
 }
 
+
 /**
  * Gets the next non-NEWLINE and non-comment chunk
  */
-chunk_t *chunk_get_next_ncnl(chunk_t *cur)
+chunk_t *chunk_get_next_ncnl(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_next(pc);
+      pc = chunk_get_next(pc, nav);
    } while ((pc != NULL) && (chunk_is_comment(pc) || chunk_is_newline(pc)));
    return(pc);
 }
 
+
 /**
  * Gets the next non-NEWLINE and non-comment chunk, non-preprocessor chunk
  */
-chunk_t *chunk_get_next_ncnlnp(chunk_t *cur)
+chunk_t *chunk_get_next_ncnlnp(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
@@ -221,7 +276,7 @@ chunk_t *chunk_get_next_ncnlnp(chunk_t *cur)
    {
       do
       {
-         pc = chunk_get_next(pc);
+         pc = chunk_get_next(pc, nav);
       } while ((pc != NULL) && chunk_is_preproc(pc) &&
                (chunk_is_comment(pc) || chunk_is_newline(pc)));
    }
@@ -229,7 +284,7 @@ chunk_t *chunk_get_next_ncnlnp(chunk_t *cur)
    {
       do
       {
-         pc = chunk_get_next(pc);
+         pc = chunk_get_next(pc, nav);
       } while ((pc != NULL) && (chunk_is_comment(pc) ||
                                 chunk_is_newline(pc) ||
                                 chunk_is_preproc(pc)));
@@ -237,10 +292,11 @@ chunk_t *chunk_get_next_ncnlnp(chunk_t *cur)
    return(pc);
 }
 
+
 /**
  * Gets the prev non-NEWLINE and non-comment chunk, non-preprocessor chunk
  */
-chunk_t *chunk_get_prev_ncnlnp(chunk_t *cur)
+chunk_t *chunk_get_prev_ncnlnp(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
@@ -248,7 +304,7 @@ chunk_t *chunk_get_prev_ncnlnp(chunk_t *cur)
    {
       do
       {
-         pc = chunk_get_prev(pc);
+         pc = chunk_get_prev(pc, nav);
       } while ((pc != NULL) && chunk_is_preproc(pc) &&
                (chunk_is_comment(pc) || chunk_is_newline(pc)));
    }
@@ -256,7 +312,7 @@ chunk_t *chunk_get_prev_ncnlnp(chunk_t *cur)
    {
       do
       {
-         pc = chunk_get_prev(pc);
+         pc = chunk_get_prev(pc, nav);
       } while ((pc != NULL) && (chunk_is_comment(pc) ||
                                 chunk_is_newline(pc) ||
                                 chunk_is_preproc(pc)));
@@ -264,79 +320,85 @@ chunk_t *chunk_get_prev_ncnlnp(chunk_t *cur)
    return(pc);
 }
 
+
 /**
  * Gets the next non-blank chunk
  */
-chunk_t *chunk_get_next_nblank(chunk_t *cur)
+chunk_t *chunk_get_next_nblank(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_next(pc);
+      pc = chunk_get_next(pc, nav);
    } while ((pc != NULL) && (chunk_is_comment(pc) ||
                              chunk_is_newline(pc) ||
                              chunk_is_blank(pc)));
    return(pc);
 }
 
+
 /**
  * Gets the prev non-blank chunk
  */
-chunk_t *chunk_get_prev_nblank(chunk_t *cur)
+chunk_t *chunk_get_prev_nblank(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_prev(pc);
+      pc = chunk_get_prev(pc, nav);
    } while ((pc != NULL) && (chunk_is_comment(pc) || chunk_is_newline(pc) ||
                              chunk_is_blank(pc)));
    return(pc);
 }
 
+
 /**
  * Gets the next non-comment chunk
  */
-chunk_t *chunk_get_next_nc(chunk_t *cur)
+chunk_t *chunk_get_next_nc(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_next(pc);
+      pc = chunk_get_next(pc, nav);
    } while ((pc != NULL) && chunk_is_comment(pc));
    return(pc);
 }
 
+
 /**
  * Gets the prev non-NEWLINE and non-comment chunk
  */
-chunk_t *chunk_get_prev_ncnl(chunk_t *cur)
+chunk_t *chunk_get_prev_ncnl(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_prev(pc);
+      pc = chunk_get_prev(pc, nav);
    } while ((pc != NULL) && (chunk_is_comment(pc) || chunk_is_newline(pc)));
    return(pc);
 }
 
+
 /**
  * Gets the prev non-comment chunk
  */
-chunk_t *chunk_get_prev_nc(chunk_t *cur)
+chunk_t *chunk_get_prev_nc(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_prev(pc);
+      pc = chunk_get_prev(pc, nav);
    } while ((pc != NULL) && chunk_is_comment(pc));
    return(pc);
 }
 
+
 /**
  * Grabs the next chunk of the given type at the level.
  *
@@ -346,13 +408,13 @@ chunk_t *chunk_get_prev_nc(chunk_t *cur)
  * @return        NULL or the match
  */
 chunk_t *chunk_get_next_type(chunk_t *cur, c_token_t type,
-                             int level)
+                             int level, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_next(pc);
+      pc = chunk_get_next(pc, nav);
       if ((pc == NULL) ||
           ((pc->type == type) && ((pc->level == level) || (level < 0))))
       {
@@ -362,13 +424,15 @@ chunk_t *chunk_get_next_type(chunk_t *cur, c_token_t type,
    return(pc);
 }
 
-chunk_t *chunk_get_next_str(chunk_t *cur, const char *str, int len, int level)
+
+chunk_t *chunk_get_next_str(chunk_t *cur, const char *str, int len, int level,
+                            chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_next(pc);
+      pc = chunk_get_next(pc, nav);
       if ((pc == NULL) ||
           ((pc->len == len) && (memcmp(str, pc->str, len) == 0) &&
            ((pc->level == level) || (level < 0))))
@@ -379,6 +443,7 @@ chunk_t *chunk_get_next_str(chunk_t *cur, const char *str, int len, int level)
    return(pc);
 }
 
+
 /**
  * Grabs the prev chunk of the given type at the level.
  *
@@ -388,13 +453,13 @@ chunk_t *chunk_get_next_str(chunk_t *cur, const char *str, int len, int level)
  * @return        NULL or the match
  */
 chunk_t *chunk_get_prev_type(chunk_t *cur, c_token_t type,
-                             int level)
+                             int level, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_prev(pc);
+      pc = chunk_get_prev(pc, nav);
       if ((pc == NULL) ||
           ((pc->type == type) && ((pc->level == level) || (level < 0))))
       {
@@ -404,13 +469,15 @@ chunk_t *chunk_get_prev_type(chunk_t *cur, c_token_t type,
    return(pc);
 }
 
-chunk_t *chunk_get_prev_str(chunk_t *cur, const char *str, int len, int level)
+
+chunk_t *chunk_get_prev_str(chunk_t *cur, const char *str, int len, int level,
+                            chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_prev(pc);
+      pc = chunk_get_prev(pc, nav);
       if ((pc == NULL) ||
           ((pc->len == len) && (memcmp(str, pc->str, len) == 0) &&
            ((pc->level == level) || (level < 0))))
@@ -421,6 +488,7 @@ chunk_t *chunk_get_prev_str(chunk_t *cur, const char *str, int len, int level)
    return(pc);
 }
 
+
 /**
  * Check to see if there is a newline bewteen the two chunks
  */
@@ -438,6 +506,7 @@ bool chunk_is_newline_between(chunk_t *start, chunk_t *end)
    return(false);
 }
 
+
 /**
  * Swaps the two chunks.
  *
@@ -449,6 +518,7 @@ void chunk_swap(chunk_t *pc1, chunk_t *pc2)
    g_cl.Swap(pc1, pc2);
 }
 
+
 /**
  * Finds the first chunk on the line that pc is on.
  * This just backs up until a newline or NULL is hit.
@@ -469,6 +539,7 @@ chunk_t *chunk_first_on_line(chunk_t *pc)
    return(first);
 }
 
+
 /**
  * Swaps two lines that are started with the specified chunks.
  *
@@ -547,30 +618,32 @@ void chunk_swap_lines(chunk_t *pc1, chunk_t *pc2)
    }
 }
 
+
 /**
  * Gets the next non-vbrace chunk
  */
-chunk_t *chunk_get_next_nvb(chunk_t *cur)
+chunk_t *chunk_get_next_nvb(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_next(pc);
+      pc = chunk_get_next(pc, nav);
    } while (chunk_is_vbrace(pc));
    return(pc);
 }
 
+
 /**
  * Gets the prev non-vbrace chunk
  */
-chunk_t *chunk_get_prev_nvb(chunk_t *cur)
+chunk_t *chunk_get_prev_nvb(chunk_t *cur, chunk_nav_t nav)
 {
    chunk_t *pc = cur;
 
    do
    {
-      pc = chunk_get_prev(pc);
+      pc = chunk_get_prev(pc, nav);
    } while (chunk_is_vbrace(pc));
    return(pc);
 }
