@@ -4,8 +4,6 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: indent.cpp 1481 2009-02-21 18:44:54Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -130,6 +128,14 @@ void indent_to_column(chunk_t *pc, int column)
    reindent_line(pc, column);
 }
 
+
+enum align_mode
+{
+   ALMODE_SHIFT,     /* shift relative to the current column */
+   ALMODE_KEEP_ABS,  /* try to keep the original absolute column */
+   ALMODE_KEEP_REL,  /* try to keep the original gap */
+};
+
 /* Same as indent_to_column, except we can move both ways */
 void align_to_column(chunk_t *pc, int column)
 {
@@ -138,54 +144,73 @@ void align_to_column(chunk_t *pc, int column)
       return;
    }
 
-   int col_delta;
-   int min_col;
-
    LOG_FMT(LINDLINE, "%s: %d] col %d on %.*s [%s] => %d\n",
            __func__, pc->orig_line, pc->column, pc->len, pc->str,
            get_token_name(pc->type), column);
 
-   col_delta  = column - pc->column;
-   pc->column = column;
-   min_col    = column;
+   int col_delta = column - pc->column;
+   int min_col   = column;
+   int min_delta;
 
+   pc->column = column;
    do
    {
-      chunk_t *next = chunk_get_next(pc);
+      chunk_t    *next = chunk_get_next(pc);
+      chunk_t    *prev;
+      align_mode almod = ALMODE_SHIFT;
 
       if (next == NULL)
       {
          break;
       }
-      min_col += space_col_align(pc, next);
-      pc = next;
-      bool is_comment = chunk_is_comment(pc);
-      bool keep = is_comment && chunk_is_single_line_comment(pc) &&
-                  cpd.settings[UO_indent_relative_single_line_comments].b;
+      min_delta = space_col_align(pc, next);
+      min_col  += min_delta;
+      prev      = pc;
+      pc        = next;
 
-      if (is_comment && (pc->parent_type != CT_COMMENT_EMBED) && !keep)
+      if (chunk_is_comment(pc) && (pc->parent_type != CT_COMMENT_EMBED))
       {
+         almod = (chunk_is_single_line_comment(pc) &&
+                  cpd.settings[UO_indent_relative_single_line_comments].b) ?
+                 ALMODE_KEEP_REL : ALMODE_KEEP_ABS;
+      }
+
+      if (almod == ALMODE_KEEP_ABS)
+      {
+         /* Keep same absolute column */
          pc->column = pc->orig_col;
          if (pc->column < min_col)
          {
-            pc->column = min_col;// + 1;
+            pc->column = min_col;
          }
-         LOG_FMT(LINDLINE, "%s: set comment on line %d to col %d (orig %d)\n",
-                 __func__, pc->orig_line, pc->column, pc->orig_col);
       }
-      else
+      else if (almod == ALMODE_KEEP_REL)
       {
+         /* Keep same relative column */
+         int orig_delta = pc->orig_col - prev->orig_col;
+         if (orig_delta < min_delta)
+         {
+            orig_delta = min_delta;
+         }
+         pc->column = prev->column + orig_delta;
+      }
+      else /* ALMODE_SHIFT */
+      {
+         /* Shift by the same amount */
          pc->column += col_delta;
          if (pc->column < min_col)
          {
             pc->column = min_col;
          }
-         LOG_FMT(LINDLINED, "   set column of '%.*s' to %d (orig %d)\n",
-                 pc->len, pc->str, pc->column, pc->orig_col);
       }
+      LOG_FMT(LINDLINED, "   %s set column of %s on line %d to col %d (orig %d)\n",
+              (almod == ALMODE_KEEP_ABS) ? "abs" :
+              (almod == ALMODE_KEEP_REL) ? "rel" : "sft",
+              get_token_name(pc->type), pc->orig_line, pc->column, pc->orig_col);
    } while ((pc != NULL) && (pc->nl_count == 0));
 }
 
+
 /**
  * Changes the initial indent for a line to the given column
  *
@@ -194,9 +219,6 @@ void align_to_column(chunk_t *pc, int column)
  */
 void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno)
 {
-   int col_delta;
-   int min_col;
-
    LOG_FMT(LINDLINE, "%s: %d] col %d on %.*s [%s] => %d <called from '%s' line %d\n",
            __func__, pc->orig_line, pc->column, pc->len, pc->str,
            get_token_name(pc->type), column, fcn_name, lineno);
@@ -205,10 +227,11 @@ void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno)
    {
       return;
    }
-   col_delta  = column - pc->column;
-   pc->column = column;
-   min_col    = column;
 
+   int col_delta = column - pc->column;
+   int min_col   = column;
+
+   pc->column = column;
    do
    {
       chunk_t *next = chunk_get_next(pc);
@@ -218,18 +241,18 @@ void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno)
          break;
       }
       min_col += space_col_align(pc, next);
-      pc = next;
+      pc       = next;
 
       bool is_comment = chunk_is_comment(pc);
-      bool keep = is_comment && chunk_is_single_line_comment(pc) &&
-                  cpd.settings[UO_indent_relative_single_line_comments].b;
+      bool keep       = is_comment && chunk_is_single_line_comment(pc) &&
+                        cpd.settings[UO_indent_relative_single_line_comments].b;
 
       if (is_comment && (pc->parent_type != CT_COMMENT_EMBED) && !keep)
       {
          pc->column = pc->orig_col;
          if (pc->column < min_col)
          {
-            pc->column = min_col;// + 1;
+            pc->column = min_col; // + 1;
          }
          LOG_FMT(LINDLINE, "%s: set comment on line %d to col %d (orig %d)\n",
                  __func__, pc->orig_line, pc->column, pc->orig_col);
@@ -247,6 +270,7 @@ void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno)
    } while ((pc != NULL) && (pc->nl_count == 0));
 }
 
+
 /**
  * Starts a new entry
  *
@@ -258,7 +282,7 @@ static void indent_pse_push(struct parse_frame& frm, chunk_t *pc)
    static int ref = 0;
 
    /* check the stack depth */
-   if (frm.pse_tos < (int)ARRAY_SIZE(frm.pse))
+   if (frm.pse_tos < ((int)ARRAY_SIZE(frm.pse) - 1))
    {
       /* Bump up the index and initialize it */
       frm.pse_tos++;
@@ -278,6 +302,7 @@ static void indent_pse_push(struct parse_frame& frm, chunk_t *pc)
    }
 }
 
+
 /**
  * Removes the top entry
  *
@@ -313,6 +338,7 @@ static void indent_pse_pop(struct parse_frame& frm, chunk_t *pc)
    }
 }
 
+
 static int token_indent(c_token_t type)
 {
    switch (type)
@@ -326,6 +352,7 @@ static int token_indent(c_token_t type)
       return(4);
 
    case CT_WHILE:
+   case CT_USING_STMT:
       return(6);
 
    case CT_SWITCH:
@@ -339,13 +366,26 @@ static int token_indent(c_token_t type)
    }
 }
 
-#define indent_column_set(X) \
-   do { \
-      indent_column = (X); \
-       LOG_FMT(LINDENT2, "[line %d] indent_column = %d\n", \
-               __LINE__, indent_column); \
+
+#define indent_column_set(X)                              \
+   do {                                                   \
+      indent_column = (X);                                \
+      LOG_FMT(LINDENT2, "[line %d] indent_column = %d\n", \
+              __LINE__, indent_column);                   \
    } while (0)
 
+
+/**
+ * Determines the continuation indent if indent_continue is non-zero.
+ */
+static int continue_indent(chunk_t *pc)
+{
+   return(1 + (pc->level * cpd.settings[UO_indent_columns].n) +
+          cpd.settings[UO_indent_continue].n);
+
+}
+
+
 /**
  * Change the top-level indentation only by changing the column member in
  * the chunk structures.
@@ -364,8 +404,6 @@ void indent_text(void)
    struct parse_frame frm;
    bool               in_preproc = false, was_preproc = false;
    int                indent_column;
-   int                cout_col            = 0; // for aligning << stuff
-   int                cout_level          = 0; // for aligning << stuff
    int                parent_token_indent = 0;
    int                xml_indent          = 0;
    bool               token_used;
@@ -510,14 +548,6 @@ void indent_text(void)
          }
       }
 
-      if ((cout_col > 0) &&
-          (chunk_is_semicolon(pc) ||
-           (pc->level < cout_level)))
-      {
-         cout_col   = 0;
-         cout_level = 0;
-      }
-
       /* Check for close XML tags "</..." */
       if (cpd.settings[UO_indent_xml_string].n > 0)
       {
@@ -606,6 +636,7 @@ void indent_text(void)
             if ((frm.pse[frm.pse_tos].type == CT_CLASS_COLON) &&
                 ((pc->type == CT_BRACE_OPEN) ||
                  (pc->type == CT_OC_END) ||
+                 (pc->type == CT_OC_SCOPE) ||
                  chunk_is_semicolon(pc)))
             {
                indent_pse_pop(frm, pc);
@@ -635,6 +666,14 @@ void indent_text(void)
                indent_pse_pop(frm, pc);
             }
 
+            /* an OC SCOPE ('-' or '+') ends with a semicolon or brace open */
+            if ((frm.pse[frm.pse_tos].type == CT_OC_SCOPE) &&
+                (chunk_is_semicolon(pc) ||
+                 (pc->type == CT_BRACE_OPEN)))
+            {
+               indent_pse_pop(frm, pc);
+            }
+
             /* an SQL EXEC is ended with a semicolon */
             if ((frm.pse[frm.pse_tos].type == CT_SQL_EXEC) &&
                 chunk_is_semicolon(pc))
@@ -690,7 +729,14 @@ void indent_text(void)
       {
          brace_indent = (cpd.settings[UO_indent_braces].b &&
                          (!cpd.settings[UO_indent_braces_no_func].b ||
-                          (pc->parent_type != CT_FUNC_DEF)));
+                          (pc->parent_type != CT_FUNC_DEF)) &&
+                         (!cpd.settings[UO_indent_braces_no_func].b ||
+                          (pc->parent_type != CT_FUNC_CLASS)) &&
+                         (!cpd.settings[UO_indent_braces_no_class].b ||
+                          (pc->parent_type != CT_CLASS)) &&
+                         (!cpd.settings[UO_indent_braces_no_struct].b ||
+                          (pc->parent_type != CT_STRUCT)));
+
       }
 
       if (pc->type == CT_BRACE_CLOSE)
@@ -739,6 +785,7 @@ void indent_text(void)
                 (pc->parent_type == CT_CATCH) ||
                 (pc->parent_type == CT_DO) ||
                 (pc->parent_type == CT_WHILE) ||
+                (pc->parent_type == CT_USING_STMT) ||
                 (pc->parent_type == CT_SWITCH) ||
                 (pc->parent_type == CT_FOR))
             {
@@ -770,9 +817,23 @@ void indent_text(void)
             {
                frm.pse[frm.pse_tos].indent -= indent_size;
             }
-            else if ((pc->parent_type == CT_NAMESPACE) && !cpd.settings[UO_indent_namespace].b)
+            else if (pc->parent_type == CT_NAMESPACE)
             {
-               frm.pse[frm.pse_tos].indent -= indent_size;
+               if ((pc->flags & PCF_LONG_BLOCK) ||
+                   !cpd.settings[UO_indent_namespace].b)
+               {
+                  /* don't indent long blocks */
+                  frm.pse[frm.pse_tos].indent -= indent_size;
+               }
+               else /* indenting 'short' namespace */
+               {
+                  if (cpd.settings[UO_indent_namespace_level].n > 0)
+                  {
+                     frm.pse[frm.pse_tos].indent -= indent_size;
+                     frm.pse[frm.pse_tos].indent +=
+                        cpd.settings[UO_indent_namespace_level].n;
+                  }
+               }
             }
             else if ((pc->parent_type == CT_EXTERN) && !cpd.settings[UO_indent_extern].b)
             {
@@ -1046,7 +1107,12 @@ void indent_text(void)
              chunk_is_newline(chunk_get_prev(pc)) &&
              !chunk_is_newline(chunk_get_next(pc)))
          {
-            frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos].indent + indent_size;
+            frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent + indent_size;
+            indent_column_set(frm.pse[frm.pse_tos].indent);
+         }
+         if (cpd.settings[UO_indent_continue].n != 0)
+         {
+            frm.pse[frm.pse_tos].indent = continue_indent(pc);
             indent_column_set(frm.pse[frm.pse_tos].indent);
          }
          frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
@@ -1074,7 +1140,11 @@ void indent_text(void)
          if (next != NULL)
          {
             indent_pse_push(frm, pc);
-            if (chunk_is_newline(next) || !cpd.settings[UO_indent_align_assign].b)
+            if (cpd.settings[UO_indent_continue].n)
+            {
+               frm.pse[frm.pse_tos].indent = continue_indent(pc);
+            }
+            else if (chunk_is_newline(next) || !cpd.settings[UO_indent_align_assign].b)
             {
                frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent_tmp + indent_size;
                if (pc->type == CT_ASSIGN)
@@ -1106,15 +1176,6 @@ void indent_text(void)
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos - 1].indent;
          }
       }
-      else if (chunk_is_str(pc, "<<", 2) &&
-               cpd.settings[UO_align_left_shift].b)
-      {
-         if (cout_col == 0)
-         {
-            cout_col   = pc->column;
-            cout_level = pc->level;
-         }
-      }
       else if (pc->type == CT_THROW)
       {
          // Pick up what was just before this.
@@ -1149,11 +1210,52 @@ void indent_text(void)
             }
          }
       }
+      else if (pc->type == CT_OC_SCOPE)
+      {
+         indent_pse_push(frm, pc);
+         if (cpd.settings[UO_indent_continue].n != 0)
+         {
+            frm.pse[frm.pse_tos].indent += cpd.settings[UO_indent_continue].n;
+         }
+         else
+         {
+            frm.pse[frm.pse_tos].indent += indent_size;
+         }
+      }
       else
       {
          /* anything else? */
       }
 
+      /**
+       * Handle variable definition continuation indenting
+       */
+      if ((vardefcol == 0) &&
+          (pc->type == CT_WORD) &&
+          ((pc->flags & PCF_IN_FCN_DEF) == 0) &&
+          ((pc->flags & PCF_VAR_1ST_DEF) == PCF_VAR_1ST_DEF))
+      {
+         if (cpd.settings[UO_indent_continue].n != 0)
+         {
+            vardefcol = frm.pse[frm.pse_tos].indent +
+                        cpd.settings[UO_indent_continue].n;
+         }
+         else if (cpd.settings[UO_indent_var_def_cont].b ||
+                  chunk_is_newline(chunk_get_prev(pc)))
+         {
+            vardefcol = frm.pse[frm.pse_tos].indent + indent_size;
+         }
+         else
+         {
+            vardefcol = pc->column;
+         }
+      }
+      if (chunk_is_semicolon(pc) ||
+          ((pc->type == CT_BRACE_OPEN) && (pc->parent_type == CT_FUNCTION)))
+      {
+         vardefcol = 0;
+      }
+
       /**
        * Indent the line if needed
        */
@@ -1195,16 +1297,14 @@ void indent_text(void)
                     __func__, pc->orig_line, tmp);
             reindent_line(pc, tmp);
          }
-         else if (chunk_is_str(pc, "<<", 2) && (cout_col > 0))
-         {
-            LOG_FMT(LINDENT, "%s: %d] cout_col => %d\n",
-                    __func__, pc->orig_line, cout_col);
-            reindent_line(pc, cout_col);
-         }
          else if ((vardefcol > 0) &&
+                  (pc->level == pc->brace_level) &&
                   (pc->type == CT_WORD) &&
-                  ((pc->flags & PCF_VAR_DEF) != 0) &&
-                  (prev != NULL) && (prev->type == CT_COMMA))
+                  (prev != NULL) &&
+                  ((prev->type == CT_COMMA) ||
+                   (prev->type == CT_TYPE) ||
+                   (prev->type == CT_WORD)) &&
+                  ((pc->flags & PCF_VAR_DEF) != 0))
          {
             LOG_FMT(LINDENT, "%s: %d] Vardefcol => %d\n",
                     __func__, pc->orig_line, vardefcol);
@@ -1297,6 +1397,11 @@ void indent_text(void)
                 chunk_is_paren_open(frm.pse[frm.pse_tos].pc))
             {
                indent_column_set(frm.pse[frm.pse_tos].pc->column);
+               if (cpd.settings[UO_indent_first_bool_expr].b)
+               {
+                  reindent_line(chunk_get_next(frm.pse[frm.pse_tos].pc),
+                                indent_column + pc->len + 1);
+               }
             }
             LOG_FMT(LINDENT, "%s: %d] bool => %d [%.*s]\n",
                     __func__, pc->orig_line, indent_column, pc->len, pc->str);
@@ -1350,21 +1455,6 @@ void indent_text(void)
          }
       }
 
-      /**
-       * Handle variable definition continuation indenting
-       */
-      if ((pc->type == CT_WORD) &&
-          ((pc->flags & PCF_IN_FCN_DEF) == 0) &&
-          ((pc->flags & PCF_VAR_1ST_DEF) == PCF_VAR_1ST_DEF))
-      {
-         vardefcol = pc->column;
-      }
-      if (chunk_is_semicolon(pc) ||
-          ((pc->type == CT_BRACE_OPEN) && (pc->parent_type == CT_FUNCTION)))
-      {
-         vardefcol = 0;
-      }
-
       /* if we hit a newline, reset indent_tmp */
       if (chunk_is_newline(pc) ||
           (pc->type == CT_COMMENT_MULTI) ||
@@ -1429,6 +1519,7 @@ void indent_text(void)
    quick_align_again();
 }
 
+
 /**
  * returns true if forward scan reveals only single newlines or comments
  * stops when hits code
@@ -1478,6 +1569,7 @@ static bool single_line_comment_indent_rule_applies(chunk_t *start)
    return(false);
 }
 
+
 /**
  * REVISIT: This needs to be re-checked, maybe cleaned up
  *
@@ -1564,6 +1656,7 @@ static void indent_comment(chunk_t *pc, int col)
    reindent_line(pc, col);
 }
 
+
 /**
  * Scan to see if the whole file is covered by one #ifdef
  */
@@ -1625,6 +1718,7 @@ static bool ifdef_over_whole_file()
    return(stage == 2);
 }
 
+
 /**
  * Indent the preprocessor stuff from column 1.
  * FIXME: This is broken if there is a comment or escaped newline
