@@ -4,8 +4,6 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: combine.cpp 391 2006-08-04 02:02:37Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -48,6 +46,7 @@ chunk_t *pawn_add_vsemi_after(chunk_t *pc)
    return(chunk_add_after(&chunk, pc));
 }
 
+
 /**
  * Turns certain virtual semicolons invisible.
  *  - after a close brace with a parent of switch, case, else, if
@@ -83,6 +82,7 @@ void pawn_scrub_vsemi(void)
    }
 }
 
+
 /**
  * Checks to see if a token continues a statement to the next line.
  * We need to check for 'open' braces/paren/etc because the level doesn't
@@ -118,7 +118,7 @@ static bool pawn_continued(chunk_t *pc, int br_level)
        (pc->parent_type == CT_WHILE) ||
        (pc->parent_type == CT_FUNC_DEF) ||
        (pc->parent_type == CT_ENUM) ||
-       ((pc->flags & PCF_IN_ENUM) != 0) ||
+       ((pc->flags & (PCF_IN_ENUM | PCF_IN_STRUCT)) != 0) ||
        chunk_is_str(pc, ":", 1) ||
        chunk_is_str(pc, "+", 1) ||
        chunk_is_str(pc, "-", 1))
@@ -128,6 +128,7 @@ static bool pawn_continued(chunk_t *pc, int br_level)
    return(false);
 }
 
+
 /**
  * Does a scan of level 0 BEFORE stuff in combine.cpp is called.
  * At this point, VSemis have been added only in VBraces.
@@ -163,6 +164,7 @@ void pawn_prescan(void)
    }
 }
 
+
 /**
  * Functions prototypes and definitions can only appear in level 0.
  *
@@ -206,7 +208,8 @@ static chunk_t *pawn_process_line(chunk_t *start)
    {
       if ((pc->level == 0) &&
           ((pc->type == CT_FUNCTION) ||
-           (pc->type == CT_WORD)))
+           (pc->type == CT_WORD) ||
+           (pc->type == CT_OPERATOR_VAL)))
       {
          fcn = pc;
       }
@@ -237,6 +240,7 @@ static chunk_t *pawn_process_line(chunk_t *start)
    return(start);
 }
 
+
 /**
  * follows a variable definition at level 0 until the end.
  * Adds a semicolon at the end, if needed.
@@ -263,6 +267,7 @@ static chunk_t *pawn_process_variable(chunk_t *start)
    return(pc);
 }
 
+
 void pawn_add_virtual_semicolons(void)
 {
    chunk_t *prev;
@@ -292,7 +297,7 @@ void pawn_add_virtual_semicolons(void)
 
          /* we just hit a newline and we have a previous token */
          if (((prev->flags & PCF_IN_PREPROC) == 0) &&
-             ((prev->flags & PCF_IN_ENUM) == 0) &&
+             ((prev->flags & (PCF_IN_ENUM | PCF_IN_STRUCT)) == 0) &&
              (prev->type != CT_VSEMICOLON) &&
              (prev->type != CT_SEMICOLON) &&
              !pawn_continued(prev, prev->brace_level))
@@ -304,6 +309,7 @@ void pawn_add_virtual_semicolons(void)
    }
 }
 
+
 /**
  * We are on a level 0 function proto of def
  */
@@ -342,6 +348,7 @@ static chunk_t *pawn_mark_function0(chunk_t *start, chunk_t *fcn)
    return(pawn_process_func_def(fcn));
 }
 
+
 static chunk_t *pawn_process_func_def(chunk_t *pc)
 {
    /* We are on a function definition */
@@ -455,6 +462,7 @@ static chunk_t *pawn_process_func_def(chunk_t *pc)
    return(last);
 }
 
+
 /**
  * We are in a virtual brace and hit a newline.
  * If this should end the vbrace, then insert a VSEMICOLON and return that.
