@@ -4,8 +4,6 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: newlines.cpp 1493 2009-02-27 03:45:07Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -20,6 +18,7 @@
 static void newlines_double_space_struct_enum_union(chunk_t *open_brace);
 static bool one_liner_nl_ok(chunk_t *pc);
 static void nl_handle_define(chunk_t *pc);
+static void newline_iarf_pair(chunk_t *before, chunk_t *after, argval_t av);
 
 /**
  * Check to see if we are allowed to increase the newline count.
@@ -41,6 +40,7 @@ static bool can_increase_nl(chunk_t *nl)
    return(true);
 }
 
+
 /**
  * Double the newline, if allowed.
  */
@@ -60,6 +60,7 @@ static void double_newline(chunk_t *nl)
    nl->nl_count = 2;
 }
 
+
 /*
  * Basic approach:
  * 1. Find next open brace
@@ -72,7 +73,6 @@ static void double_newline(chunk_t *nl)
  */
 
 //#define DEBUG_NEWLINES
-
 static void setup_newline_add(const chunk_t *prev, chunk_t *nl, const chunk_t *next)
 {
    memset(nl, 0, sizeof(*nl));
@@ -96,20 +96,17 @@ static void setup_newline_add(const chunk_t *prev, chunk_t *nl, const chunk_t *n
    }
 }
 
-/**
- * 2 parts:
- *  - if/switch/while/for after braces
- *  - else before/after braces
- *  - do after braces
- *  - do/while before braces
- */
 
+/**
+ * Add a newline before the chunk if there isn't already a newline present.
+ * Virtual braces are skipped, as they do not contribute to the output.
+ */
 chunk_t *newline_add_before2(chunk_t *pc, const char *fcn, int line)
 {
    chunk_t nl;
    chunk_t *prev;
 
-   prev = chunk_get_prev(pc);
+   prev = chunk_get_prev_nvb(pc);
    if (chunk_is_newline(prev))
    {
       /* Already has a newline before this chunk */
@@ -124,12 +121,17 @@ chunk_t *newline_add_before2(chunk_t *pc, const char *fcn, int line)
    return(chunk_add_before(&nl, pc));
 }
 
+
+/**
+ * Add a newline after the chunk if there isn't already a newline present.
+ * Virtual braces are skipped, as they do not contribute to the output.
+ */
 chunk_t *newline_add_after2(chunk_t *pc, const char *fcn, int line)
 {
    chunk_t nl;
    chunk_t *next;
 
-   next = chunk_get_next(pc);
+   next = chunk_get_next_nvb(pc);
    if (chunk_is_newline(next))
    {
       /* Already has a newline after this chunk */
@@ -144,7 +146,8 @@ chunk_t *newline_add_after2(chunk_t *pc, const char *fcn, int line)
    return(chunk_add_after(&nl, pc));
 }
 
-#define newline_min_after(ref, cnt)     newline_min_after2(ref, cnt, __func__, __LINE__)
+
+#define newline_min_after(ref, cnt)    newline_min_after2(ref, cnt, __func__, __LINE__)
 
 static void newline_min_after2(chunk_t *ref, INT32 count,
                                const char *func, int line)
@@ -182,6 +185,7 @@ static void newline_min_after2(chunk_t *ref, INT32 count,
    }
 }
 
+
 /**
  * Add a newline between two tokens.
  * If there is already a newline between then, nothing is done.
@@ -248,6 +252,7 @@ chunk_t *newline_add_between2(chunk_t *start, chunk_t *end,
    return(newline_add_before(end));
 }
 
+
 /**
  * Removes any CT_NEWLINE or CT_NL_CONT between start and end.
  * Start must be before end on the chunk list.
@@ -263,7 +268,7 @@ void newline_del_between2(chunk_t *start, chunk_t *end,
 {
    chunk_t *next;
    chunk_t *prev;
-   chunk_t *pc    = start;
+   chunk_t *pc = start;
 
    LOG_FMT(LNEWLINE, "%s: '%.*s' line %d:%d and '%.*s' line %d:%d : caller=%s:%d preproc=%d/%d\n",
            __func__, start->len, start->str, start->orig_line, start->orig_col,
@@ -316,11 +321,15 @@ void newline_del_between2(chunk_t *start, chunk_t *end,
    }
 }
 
+
 /**
  * Add or remove a newline between the closing paren and opening brace.
  * Also uncuddles anything on the closing brace. (may get fixed later)
  *
  * "if (...) { \n" or "if (...) \n { \n"
+ *
+ * For virtual braces, we can only add a newline after the vbrace open.
+ * If we do so, also add a newline after the vbrace close.
  */
 static void newlines_if_for_while_switch(chunk_t *start, argval_t nl_opt)
 {
@@ -341,7 +350,9 @@ static void newlines_if_for_while_switch(chunk_t *start, argval_t nl_opt)
       close_paren = chunk_get_next_type(pc, CT_SPAREN_CLOSE, pc->level);
       brace_open  = chunk_get_next_ncnl(close_paren);
 
-      if ((brace_open != NULL) && (brace_open->type == CT_BRACE_OPEN) &&
+      if ((brace_open != NULL) &&
+          ((brace_open->type == CT_BRACE_OPEN) ||
+           (brace_open->type == CT_VBRACE_OPEN)) &&
           one_liner_nl_ok(brace_open))
       {
          if (cpd.settings[UO_nl_multi_line_cond].b)
@@ -356,24 +367,35 @@ static void newlines_if_for_while_switch(chunk_t *start, argval_t nl_opt)
             }
          }
 
-         if (nl_opt & AV_REMOVE)
+         if (brace_open->type == CT_VBRACE_OPEN)
          {
-            newline_del_between(close_paren, brace_open);
+            /* Can only add - we don't want to create a one-line here */
+            if (nl_opt & AV_ADD)
+            {
+               newline_iarf_pair(close_paren, chunk_get_next_ncnl(brace_open), nl_opt);
+               pc = chunk_get_next_type(brace_open, CT_VBRACE_CLOSE, brace_open->level);
+               if (!chunk_is_newline(chunk_get_prev_nc(pc)) &&
+                   !chunk_is_newline(chunk_get_next_nc(pc)))
+               {
+                  newline_add_after(pc);
+               }
+           }
          }
-         if (nl_opt & AV_ADD)
+         else
          {
-            newline_add_between(close_paren, brace_open);
+            newline_iarf_pair(close_paren, brace_open, nl_opt);
+
+            newline_add_between(brace_open, chunk_get_next_ncnl(brace_open));
+
+            /* Make sure nothing is cuddled with the closing brace */
+            pc = chunk_get_next_type(brace_open, CT_BRACE_CLOSE, brace_open->level);
+            newline_add_between(pc, chunk_get_next_nblank(pc));
          }
-
-         newline_add_between(brace_open, chunk_get_next_ncnl(brace_open));
-
-         /* Make sure nothing is cuddled with the closing brace */
-         pc = chunk_get_next_type(brace_open, CT_BRACE_CLOSE, brace_open->level);
-         newline_add_between(pc, chunk_get_next_nblank(pc));
       }
    }
 }
 
+
 /**
  * Add or remove extra newline before the chunk.
  * Adds before comments
@@ -430,7 +452,7 @@ static void newlines_if_for_while_switch_pre_blank_lines(chunk_t *start, argval_
             return;
          }
       }
-      else if (chunk_is_opening_brace(pc) || pc->level < level)
+      else if (chunk_is_opening_brace(pc) || (pc->level < level))
       {
          return;
       }
@@ -472,12 +494,13 @@ static void newlines_if_for_while_switch_pre_blank_lines(chunk_t *start, argval_
    }
 }
 
+
 static chunk_t *get_closing_brace(chunk_t *start)
 {
    chunk_t *pc;
    int     level = start->level;
 
-   for (pc = start; (pc = chunk_get_next(pc)) != NULL;)
+   for (pc = start; (pc = chunk_get_next(pc)) != NULL; )
    {
       if (((pc->type == CT_BRACE_CLOSE) || (pc->type == CT_VBRACE_CLOSE)) && (pc->level == level))
       {
@@ -493,6 +516,7 @@ static chunk_t *get_closing_brace(chunk_t *start)
    return(NULL);
 }
 
+
 /**
  * remove any consecutive newlines following this chunk
  * skip vbraces
@@ -518,6 +542,7 @@ static void remove_next_newlines(chunk_t *start)
    }
 }
 
+
 /**
  * Add or remove extra newline after end of the block started in chunk.
  * Doesn't do anything if close brace after it
@@ -645,10 +670,10 @@ static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval
             {
                prev = chunk_get_prev_nnl(next);
                pc   = chunk_get_next_nl(next);
-               LOG_FMT(LSYS, "  -- pc1=%.*s [%s]\n", pc->len, pc->str, get_token_name(pc->type));
+               //LOG_FMT(LSYS, "  -- pc1=%.*s [%s]\n", pc->len, pc->str, get_token_name(pc->type));
 
-               pc   = chunk_get_next(pc);
-               LOG_FMT(LSYS, "  -- pc2=%.*s [%s]\n", pc->len, pc->str, get_token_name(pc->type));
+               pc = chunk_get_next(pc);
+               //LOG_FMT(LSYS, "  -- pc2=%.*s [%s]\n", pc->len, pc->str, get_token_name(pc->type));
                if ((pc != NULL) && (pc->type == CT_PREPROC) &&
                    (pc->parent_type == CT_PP_ENDIF) &&
                    cpd.settings[UO_nl_squeeze_ifdef].b)
@@ -667,10 +692,12 @@ static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval
    }
 }
 
+
 /**
  * Adds or removes a newline between the keyword and the open brace.
  * If there is something after the '{' on the same line, then
  * the newline is removed unconditionally.
+ * If there is a '=' between the keyword and '{', do nothing.
  *
  * "struct [name] {" or "struct [name] \n {"
  */
@@ -687,14 +714,16 @@ static void newlines_struct_enum_union(chunk_t *start, argval_t nl_opt, bool lea
    }
 
    /* step past any junk between the keyword and the open brace
-    * Quit if we hit a semicolon, which is not expected.
+    * Quit if we hit a semicolon or '=', which are not expected.
     */
    int level = start->level;
    pc = start;
    while (((pc = chunk_get_next_ncnl(pc)) != NULL) && (pc->level >= level))
    {
       if ((pc->level == level) &&
-          ((pc->type == CT_BRACE_OPEN) || chunk_is_semicolon(pc)))
+          ((pc->type == CT_BRACE_OPEN) ||
+           chunk_is_semicolon(pc) ||
+           (pc->type == CT_ASSIGN)))
       {
          break;
       }
@@ -717,17 +746,11 @@ static void newlines_struct_enum_union(chunk_t *start, argval_t nl_opt, bool lea
          nl_opt = AV_IGNORE;
       }
 
-      if ((nl_opt & AV_REMOVE) != 0)
-      {
-         newline_del_between(start, pc);
-      }
-      if ((nl_opt & AV_ADD) != 0)
-      {
-         newline_add_between(start, pc);
-      }
+      newline_iarf_pair(start, pc, nl_opt);
    }
 }
 
+
 /**
  * Cuddles or un-cuddles a chunk with a previous close brace
  *
@@ -749,18 +772,11 @@ static void newlines_cuddle_uncuddle(chunk_t *start, argval_t nl_opt)
    br_close = chunk_get_prev_ncnl(start);
    if ((br_close != NULL) && (br_close->type == CT_BRACE_CLOSE))
    {
-      /* remove before add - not exclusive! */
-      if ((nl_opt & AV_REMOVE) != 0)
-      {
-         newline_del_between(br_close, start);
-      }
-      if ((nl_opt & AV_ADD) != 0)
-      {
-         newline_add_between(br_close, start);
-      }
+      newline_iarf_pair(br_close, start, nl_opt);
    }
 }
 
+
 /**
  * Adds/removes a newline between else and '{'.
  * "else {" or "else \n {"
@@ -768,6 +784,7 @@ static void newlines_cuddle_uncuddle(chunk_t *start, argval_t nl_opt)
 static void newlines_do_else(chunk_t *start, argval_t nl_opt)
 {
    chunk_t *next;
+   chunk_t *tmp;
 
    if ((nl_opt == AV_IGNORE) ||
        (((start->flags & PCF_IN_PREPROC) != 0) &&
@@ -777,28 +794,44 @@ static void newlines_do_else(chunk_t *start, argval_t nl_opt)
    }
 
    next = chunk_get_next_ncnl(start);
-   if ((next != NULL) && (next->type == CT_BRACE_OPEN))
+   if ((next != NULL) &&
+       ((next->type == CT_BRACE_OPEN) ||
+        (next->type == CT_VBRACE_OPEN)))
    {
       if (!one_liner_nl_ok(next))
       {
          return;
       }
-      if ((nl_opt & AV_ADD) != 0)
+      if (next->type == CT_VBRACE_OPEN)
       {
-         newline_add_between(start, next);
-         chunk_t *tmp = chunk_get_next(next);
-         if ((tmp != NULL) && !chunk_is_newline(tmp))
+         /* Can only add - we don't want to create a one-line here */
+         if (nl_opt & AV_ADD)
          {
-            newline_add_between(next, tmp);
+            newline_iarf_pair(start, chunk_get_next_ncnl(next), nl_opt);
+            tmp = chunk_get_next_type(next, CT_VBRACE_CLOSE, next->level);
+            if (!chunk_is_newline(chunk_get_next_nc(tmp)) &&
+                !chunk_is_newline(chunk_get_prev_nc(tmp)))
+            {
+               newline_add_after(tmp);
+            }
          }
       }
-      else if ((nl_opt & AV_REMOVE) != 0)
+      else
       {
-         newline_del_between(start, next);
+         newline_iarf_pair(start, next, nl_opt);
+         if ((nl_opt & AV_ADD) != 0)
+         {
+            tmp = chunk_get_next_nc(next);
+            if ((tmp != NULL) && !chunk_is_newline(tmp))
+            {
+               newline_add_between(next, tmp);
+            }
+         }
       }
    }
 }
 
+
 /**
  * We are at the open brace for a function body.
  * put a newline after the block of variable definitions
@@ -841,6 +874,7 @@ static void newline_fnc_var_def(chunk_t *br_open, int nl_count)
    }
 }
 
+
 /**
  * Handles the brace_on_func_line setting and decides if the closing brace
  * of a pair should be right after a newline.
@@ -915,14 +949,8 @@ static void newlines_brace_pair(chunk_t *br_open)
       if (chunk_is_newline(next))
       {
          prev = chunk_get_prev_ncnl(br_open);
-         if ((cpd.settings[UO_nl_assign_brace].a & AV_ADD) != 0)
-         {
-            newline_add_between(prev, br_open);
-         }
-         else if ((cpd.settings[UO_nl_assign_brace].a & AV_REMOVE) != 0)
-         {
-            newline_del_between(prev, br_open);
-         }
+
+         newline_iarf_pair(prev, br_open, cpd.settings[UO_nl_assign_brace].a);
       }
    }
 
@@ -966,14 +994,7 @@ static void newlines_brace_pair(chunk_t *br_open)
          /* Grab the chunk before the open brace */
          prev = chunk_get_prev_ncnl(br_open);
 
-         if (val & AV_ADD)
-         {
-            newline_add_between(prev, br_open);
-         }
-         else if (val & AV_REMOVE)
-         {
-            newline_del_between(prev, br_open);
-         }
+         newline_iarf_pair(prev, br_open, val);
       }
 
       if (cpd.settings[UO_nl_func_var_def_blk].n > 0)
@@ -1019,6 +1040,7 @@ static void newlines_brace_pair(chunk_t *br_open)
    }
 }
 
+
 /**
  * Put a empty line between the 'case' statement and the previous case colon
  * or semicolon.
@@ -1069,6 +1091,7 @@ static void newline_case(chunk_t *start)
    }
 }
 
+
 static void newline_case_colon(chunk_t *start)
 {
    chunk_t *pc = start;
@@ -1085,6 +1108,7 @@ static void newline_case_colon(chunk_t *start)
    }
 }
 
+
 /**
  * Put a empty line after a return statement, unless it is followed by a
  * close brace.
@@ -1121,6 +1145,31 @@ static void newline_return(chunk_t *start)
    }
 }
 
+
+/**
+ * Does the Ignore, Add, Remove, or Force thing between two chunks
+ *
+ * @param before The first chunk
+ * @param after  The second chunk
+ * @param av     The IARF value
+ */
+static void newline_iarf_pair(chunk_t *before, chunk_t *after, argval_t av)
+{
+   if ((before != NULL) && (after != NULL))
+   {
+      if ((av & AV_REMOVE) != 0)
+      {
+         newline_del_between(before, after);
+      }
+
+      if ((av & AV_ADD) != 0)
+      {
+         newline_add_between(before, after);
+      }
+   }
+}
+
+
 /**
  * Does a simple Ignore, Add, Remove, or Force after the given chunk
  *
@@ -1129,27 +1178,10 @@ static void newline_return(chunk_t *start)
  */
 void newline_iarf(chunk_t *pc, argval_t av)
 {
-   chunk_t *next;
-
-   if ((av & AV_REMOVE) != 0)
-   {
-      next = chunk_get_next_nnl(pc);
-      if (next != NULL)
-      {
-         newline_del_between(pc, next);
-      }
-   }
-
-   if ((av & AV_ADD) != 0)
-   {
-      next = chunk_get_next_nnl(pc);
-      if (next != NULL)
-      {
-         newline_add_between(pc, next);
-      }
-   }
+   newline_iarf_pair(pc, chunk_get_next_nnl(pc), av);
 }
 
+
 /**
  * Formats a function declaration
  * Start points to the open paren
@@ -1171,7 +1203,8 @@ static void newline_func_def(chunk_t *start)
 
    /* Handle break newlines type and function */
    prev = chunk_get_prev_ncnl(start);
-   prev = chunk_get_prev_ncnl(prev);
+   /* Don't split up a function variable */
+   prev = (prev->type == CT_PAREN_CLOSE) ? NULL : chunk_get_prev_ncnl(prev);
 
    if (prev != NULL)
    {
@@ -1193,21 +1226,34 @@ static void newline_func_def(chunk_t *start)
       }
 
       argval_t a = (tmp->parent_type == CT_FUNC_PROTO) ?
-          cpd.settings[UO_nl_func_proto_type_name].a :
-          cpd.settings[UO_nl_func_type_name].a;
+                   cpd.settings[UO_nl_func_proto_type_name].a :
+                   cpd.settings[UO_nl_func_type_name].a;
+      if ((tmp->flags & PCF_IN_CLASS) &&
+          (cpd.settings[UO_nl_func_type_name_class].a != AV_IGNORE))
+      {
+         a = cpd.settings[UO_nl_func_type_name_class].a;
+      }
 
       if (a != AV_IGNORE)
       {
+         if ((prev != NULL) && (prev->type == CT_DESTRUCTOR))
+         {
+            prev = chunk_get_prev_ncnl(prev);
+         }
          /* If we are on a '::', step back two tokens
           * TODO: do we also need to check for '.' ?
           */
          while ((prev != NULL) && (prev->type == CT_DC_MEMBER))
          {
             prev = chunk_get_prev_ncnl(prev);
+            prev = skip_template_prev(prev);
             prev = chunk_get_prev_ncnl(prev);
          }
 
-         if (prev != NULL)
+         if ((prev != NULL) &&
+             (prev->type != CT_BRACE_CLOSE) &&
+             (prev->type != CT_BRACE_OPEN) &&
+             (prev->type != CT_SEMICOLON))
          {
             newline_iarf(prev, a);
          }
@@ -1217,12 +1263,15 @@ static void newline_func_def(chunk_t *start)
    pc = chunk_get_next_ncnl(start);
    if (chunk_is_str(pc, ")", 1))
    {
+      if (cpd.settings[UO_nl_func_decl_empty].a != AV_IGNORE)
+      {
+         newline_iarf(start, cpd.settings[UO_nl_func_decl_empty].a);
+      }
       return;
    }
 
-   newline_iarf(start, cpd.settings[UO_nl_func_decl_start].a);
-
    /* Now scan for commas */
+   int comma_count = 0;
    for (pc = chunk_get_next_ncnl(start);
         (pc != NULL) && (pc->level > start->level);
         pc = chunk_get_next_ncnl(pc))
@@ -1230,6 +1279,7 @@ static void newline_func_def(chunk_t *start)
       prev = pc;
       if ((pc->type == CT_COMMA) && (pc->level == (start->level + 1)))
       {
+         comma_count++;
          tmp = chunk_get_next(pc);
          if (chunk_is_comment(tmp))
          {
@@ -1239,30 +1289,48 @@ static void newline_func_def(chunk_t *start)
       }
    }
 
+   argval_t as = cpd.settings[UO_nl_func_decl_start].a;
+   argval_t ae = cpd.settings[UO_nl_func_decl_end].a;
+   if (comma_count == 0)
+   {
+      if (cpd.settings[UO_nl_func_decl_start_single].a != AV_IGNORE)
+      {
+         as = cpd.settings[UO_nl_func_decl_start_single].a;
+      }
+      if (cpd.settings[UO_nl_func_decl_end_single].a != AV_IGNORE)
+      {
+         ae = cpd.settings[UO_nl_func_decl_end_single].a;
+      }
+   }
+   newline_iarf(start, as);
+
    /* and fix up the close paren */
    if ((prev != NULL) && (pc != NULL) && (pc->type == CT_FPAREN_CLOSE))
    {
-      prev = chunk_get_prev(pc);
+      prev = chunk_get_prev_nnl(pc);
       if (prev->type != CT_FPAREN_OPEN)
       {
-         newline_iarf(prev, cpd.settings[UO_nl_func_decl_end].a);
+         newline_iarf(prev, ae);
       }
    }
 }
 
+
 /**
  * Checks to see if it is OK to add a newline around the chunk.
  * Don't want to break one-liners...
  */
 static bool one_liner_nl_ok(chunk_t *pc)
 {
-   LOG_FMT(LNL1LINE, "%s: check [%s] parent=[%s] flg=%x, on line %d, col %d - ",
+   LOG_FMT(LNL1LINE, "%s: check [%s] parent=[%s] flg=%" PRIx64 ", on line %d, col %d - ",
            __func__, get_token_name(pc->type), get_token_name(pc->parent_type),
            pc->flags, pc->orig_line, pc->orig_col);
 
    if ((pc->flags & PCF_ONE_LINER) &&
        ((pc->type == CT_BRACE_OPEN) ||
-        (pc->type == CT_BRACE_CLOSE)))
+        (pc->type == CT_BRACE_CLOSE) ||
+        (pc->type == CT_VBRACE_OPEN) ||
+        (pc->type == CT_VBRACE_CLOSE)))
    {
       if (cpd.settings[UO_nl_class_leave_one_liners].b &&
           (pc->flags & PCF_IN_CLASS))
@@ -1293,7 +1361,8 @@ static bool one_liner_nl_ok(chunk_t *pc)
       }
 
       if (cpd.settings[UO_nl_func_leave_one_liners].b &&
-          (pc->parent_type == CT_FUNC_DEF))
+          ((pc->parent_type == CT_FUNC_DEF) ||
+           (pc->parent_type == CT_FUNC_CLASS)))
       {
          LOG_FMT(LNL1LINE, "false (func def)\n");
          return(false);
@@ -1311,6 +1380,7 @@ static bool one_liner_nl_ok(chunk_t *pc)
    return(true);
 }
 
+
 static void nl_create_one_liner(chunk_t *vbrace_open)
 {
    chunk_t *tmp;
@@ -1334,6 +1404,7 @@ static void nl_create_one_liner(chunk_t *vbrace_open)
    }
 }
 
+
 /**
  * Step through all chunks.
  */
@@ -1378,6 +1449,10 @@ void newlines_cleanup_braces(void)
       {
          newlines_if_for_while_switch(pc, cpd.settings[UO_nl_while_brace].a);
       }
+      else if (pc->type == CT_USING_STMT)
+      {
+         newlines_if_for_while_switch(pc, cpd.settings[UO_nl_using_brace].a);
+      }
       else if (pc->type == CT_SWITCH)
       {
          newlines_if_for_while_switch(pc, cpd.settings[UO_nl_switch_brace].a);
@@ -1392,14 +1467,7 @@ void newlines_cleanup_braces(void)
          next = chunk_get_next_ncnl(pc);
          if ((next != NULL) && (next->type == CT_ELSEIF))
          {
-            if (cpd.settings[UO_nl_else_if].a & AV_REMOVE)
-            {
-               newline_del_between(pc, next);
-            }
-            if (cpd.settings[UO_nl_else_if].a & AV_ADD)
-            {
-               newline_add_between(pc, next);
-            }
+            newline_iarf_pair(pc, next, cpd.settings[UO_nl_else_if].a);
          }
          newlines_do_else(pc, cpd.settings[UO_nl_else_brace].a);
       }
@@ -1422,6 +1490,15 @@ void newlines_cleanup_braces(void)
       }
       else if (pc->type == CT_BRACE_OPEN)
       {
+         if (cpd.settings[UO_nl_brace_brace].a != AV_IGNORE)
+         {
+            next = chunk_get_next_nc(pc, CNAV_PREPROC);
+            if ((next != NULL) && (next->type == CT_BRACE_OPEN))
+            {
+               newline_iarf_pair(pc, next, cpd.settings[UO_nl_brace_brace].a);
+            }
+         }
+
          if (cpd.settings[UO_nl_ds_struct_enum_cmt].b &&
              ((pc->parent_type == CT_ENUM) ||
               (pc->parent_type == CT_STRUCT) ||
@@ -1432,7 +1509,7 @@ void newlines_cleanup_braces(void)
 
          if ((pc->parent_type == CT_CLASS) && (pc->level == pc->brace_level))
          {
-            newlines_do_else(chunk_get_prev(pc),
+            newlines_do_else(chunk_get_prev_nnl(pc),
                              cpd.settings[UO_nl_class_brace].a);
          }
 
@@ -1447,7 +1524,7 @@ void newlines_cleanup_braces(void)
          }
          else if (next->type == CT_BRACE_OPEN)
          {
-            //TODO: do something with two brace opens on the same line? "{ {"
+            // already handled
          }
          else
          {
@@ -1491,6 +1568,15 @@ void newlines_cleanup_braces(void)
       }
       else if (pc->type == CT_BRACE_CLOSE)
       {
+         if (cpd.settings[UO_nl_brace_brace].a != AV_IGNORE)
+         {
+            next = chunk_get_next_nc(pc, CNAV_PREPROC);
+            if ((next != NULL) && (next->type == CT_BRACE_CLOSE))
+            {
+               newline_iarf_pair(pc, next, cpd.settings[UO_nl_brace_brace].a);
+            }
+         }
+
          if (cpd.settings[UO_eat_blanks_before_close_brace].b)
          {
             /* Limit the newlines before the close brace to 1 */
@@ -1507,7 +1593,7 @@ void newlines_cleanup_braces(void)
          {
             if ((pc->flags & PCF_ONE_LINER) == 0)
             {
-               /* Make sure the } is preceeded by two newlines */
+               /* Make sure the } is preceded by two newlines */
                prev = chunk_get_prev(pc);
                if (!chunk_is_newline(prev))
                {
@@ -1539,18 +1625,31 @@ void newlines_cleanup_braces(void)
       }
       else if (pc->type == CT_VBRACE_OPEN)
       {
-         if (cpd.settings[UO_nl_after_vbrace_open].b)
+         if (cpd.settings[UO_nl_after_vbrace_open].b ||
+             cpd.settings[UO_nl_after_vbrace_open_empty].b)
          {
-            next = chunk_get_next(pc);
-            if ((next->type != CT_VBRACE_CLOSE) &&
-                !chunk_is_comment(next) &&
-                !chunk_is_newline(next))
+            next = chunk_get_next(pc, CNAV_PREPROC);
+            bool add_it;
+            if (chunk_is_semicolon(next))
+            {
+               add_it = cpd.settings[UO_nl_after_vbrace_open_empty].b;
+            }
+            else
+            {
+               add_it = (cpd.settings[UO_nl_after_vbrace_open].b &&
+                         (next->type != CT_VBRACE_CLOSE) &&
+                         !chunk_is_comment(next) &&
+                         !chunk_is_newline(next));
+            }
+            if (add_it)
             {
                newline_iarf(pc, AV_ADD);
             }
          }
 
-         if (((pc->parent_type == CT_IF) &&
+         if ((((pc->parent_type == CT_IF) ||
+               (pc->parent_type == CT_ELSEIF) ||
+               (pc->parent_type == CT_ELSE)) &&
               cpd.settings[UO_nl_create_if_one_liner].b) ||
              ((pc->parent_type == CT_FOR) &&
               cpd.settings[UO_nl_create_for_one_liner].b) ||
@@ -1560,6 +1659,16 @@ void newlines_cleanup_braces(void)
             nl_create_one_liner(pc);
          }
       }
+      else if (pc->type == CT_VBRACE_CLOSE)
+      {
+         if (cpd.settings[UO_nl_after_vbrace_close].b)
+         {
+            if (!chunk_is_newline(chunk_get_next_nc(pc)))
+            {
+               newline_add_after(pc);
+            }
+         }
+      }
       else if (pc->type == CT_STRUCT)
       {
          newlines_struct_enum_union(pc, cpd.settings[UO_nl_struct_brace].a, true);
@@ -1635,12 +1744,15 @@ void newlines_cleanup_braces(void)
       {
          if (((pc->parent_type == CT_FUNC_DEF) ||
               (pc->parent_type == CT_FUNC_PROTO) ||
+              (pc->parent_type == CT_FUNC_CLASS) ||
               (pc->parent_type == CT_OPERATOR))
              &&
              ((cpd.settings[UO_nl_func_decl_start].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_decl_args].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_decl_end].a != AV_IGNORE) ||
+              (cpd.settings[UO_nl_func_decl_empty].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_type_name].a != AV_IGNORE) ||
+              (cpd.settings[UO_nl_func_type_name_class].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_scope_name].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_proto_type_name].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_paren].a != AV_IGNORE)))
@@ -1735,6 +1847,7 @@ void newlines_cleanup_braces(void)
    }
 }
 
+
 /**
  * Find the next newline or nl_cont
  */
@@ -1765,6 +1878,7 @@ static void nl_handle_define(chunk_t *pc)
    }
 }
 
+
 void newline_after_multiline_comment(void)
 {
    chunk_t *pc;
@@ -1789,6 +1903,7 @@ void newline_after_multiline_comment(void)
    }
 }
 
+
 /**
  * Handle insertion/removal of blank lines before if/for/while/do
  */
@@ -1830,6 +1945,7 @@ void newlines_insert_blank_lines(void)
    }
 }
 
+
 void newlines_squeeze_ifdef(void)
 {
    chunk_t *pc;
@@ -1888,6 +2004,7 @@ void newlines_squeeze_ifdef(void)
    }
 }
 
+
 void newlines_eat_start_end(void)
 {
    chunk_t *pc;
@@ -1959,18 +2076,21 @@ void newlines_eat_start_end(void)
    }
 }
 
+
 /**
  * Searches for a chunk of type chunk_type and moves them, if needed.
  * Will not move tokens that are on their own line or have other than
  * exactly 1 newline before (UO_pos_comma == TRAIL) or after (UO_pos_comma == LEAD).
+ * We can't remove a newline if it is right before a preprocessor.
  */
 void newlines_chunk_pos(c_token_t chunk_type, tokenpos_e mode)
 {
    chunk_t *pc;
    chunk_t *next;
    chunk_t *prev;
+   int     nl_flag;
 
-   if (mode == TP_IGNORE)
+   if ((mode & (TP_LEAD | TP_TRAIL)) == 0)
    {
       return;
    }
@@ -1982,16 +2102,71 @@ void newlines_chunk_pos(c_token_t chunk_type, tokenpos_e mode)
          prev = chunk_get_prev_nc(pc);
          next = chunk_get_next_nc(pc);
 
-         /* if both are newlines or neither are newlines, skip this chunk */
-         if (chunk_is_newline(prev) == chunk_is_newline(next))
+         nl_flag = ((chunk_is_newline(prev) ? 1 : 0) |
+                    (chunk_is_newline(next) ? 2 : 0));
+
+         if (((nl_flag == 0) && ((mode & (TP_FORCE | TP_BREAK)) == 0)) ||
+             ((nl_flag == 3) && ((mode & TP_FORCE) == 0)))
          {
+            /* No newlines and not adding any or both and not forcing */
             continue;
          }
 
-         /*NOTE: may end up processing a chunk twice if changed */
-         if (mode == TP_TRAIL)
+         if (((mode & TP_LEAD) && (nl_flag == 1)) ||
+             ((mode & TP_TRAIL) && (nl_flag == 2)))
          {
-            if (chunk_is_newline(prev) && (prev->nl_count == 1))
+            /* Already a newline before (lead) or after (trail) */
+            continue;
+         }
+
+         /* If there were no newlines, we need to add one */
+         if (nl_flag == 0)
+         {
+            if (mode & TP_LEAD)
+            {
+               newline_add_before(pc);
+            }
+            else
+            {
+               newline_add_after(pc);
+            }
+            continue;
+         }
+
+         /* If there were both newlines, we need to remove one */
+         if (nl_flag == 3)
+         {
+            if (mode & TP_LEAD)
+            {
+               remove_next_newlines(pc);
+            }
+            else
+            {
+               remove_next_newlines(chunk_get_prev_ncnl(pc));
+            }
+            continue;
+         }
+
+         /* we need to move the newline */
+         if (mode & TP_LEAD)
+         {
+            chunk_t *next2 = chunk_get_next(next);
+            if ((next2 != NULL) &&
+                ((next2->type == CT_PREPROC) ||
+                 ((chunk_type == CT_ASSIGN) &&
+                  (next2->type == CT_BRACE_OPEN))))
+            {
+               continue;
+            }
+            if (next->nl_count == 1)
+            {
+               /* move the CT_BOOL to after the newline */
+               chunk_move_after(pc, next);
+            }
+         }
+         else
+         {
+            if (prev->nl_count == 1)
             {
                /* Back up to the next non-comment item */
                prev = chunk_get_prev_nc(prev);
@@ -2002,18 +2177,11 @@ void newlines_chunk_pos(c_token_t chunk_type, tokenpos_e mode)
                }
             }
          }
-         else  /* (mode == TP_LEAD) */
-         {
-            if (chunk_is_newline(next) && (next->nl_count == 1))
-            {
-               /* move the CT_BOOL to after the newline */
-               chunk_move_after(pc, next);
-            }
-         }
       }
    }
 }
 
+
 /**
  * Searches for CT_CLASS_COLON and moves them, if needed.
  * Also breaks up the args
@@ -2063,7 +2231,7 @@ void newlines_class_colon_pos(void)
             }
          }
 
-         if (mode == TP_TRAIL)
+         if (mode & TP_TRAIL)
          {
             if (chunk_is_newline(prev) && (prev->nl_count == 1) &&
                 chunk_safe_to_del_nl(prev))
@@ -2071,7 +2239,7 @@ void newlines_class_colon_pos(void)
                chunk_swap(pc, prev);
             }
          }
-         else if (mode == TP_LEAD)
+         else if (mode & TP_LEAD)
          {
             if (chunk_is_newline(next) && (next->nl_count == 1) &&
                 chunk_safe_to_del_nl(next))
@@ -2092,11 +2260,11 @@ void newlines_class_colon_pos(void)
          {
             if ((cpd.settings[UO_nl_class_init_args].a & AV_ADD) != 0)
             {
-               if (cpd.settings[UO_pos_class_comma].tp == TP_TRAIL)
+               if (cpd.settings[UO_pos_class_comma].tp & TP_TRAIL)
                {
                   newline_add_after(pc);
                }
-               else if (cpd.settings[UO_pos_class_comma].tp == TP_LEAD)
+               else if (cpd.settings[UO_pos_class_comma].tp & TP_LEAD)
                {
                   newline_add_before(pc);
                   next = chunk_get_next_nc(pc);
@@ -2119,6 +2287,7 @@ void newlines_class_colon_pos(void)
    }
 }
 
+
 /**
  * Scans for newline tokens and limits the nl_count.
  * A newline token has a minimum nl_count of 1.
@@ -2271,7 +2440,8 @@ void do_blank_lines(void)
 
       /* Change blanks between a function comment and body */
       if ((cpd.settings[UO_nl_comment_func_def].n != 0) &&
-          chunk_is_comment(pcmt) &&
+          (pcmt->type == CT_COMMENT_MULTI) &&
+          (pcmt->parent_type == CT_COMMENT_WHOLE) &&
           (next != NULL) && (next->parent_type == CT_FUNC_DEF))
       {
          if (cpd.settings[UO_nl_comment_func_def].n != pc->nl_count)
@@ -2333,6 +2503,7 @@ void do_blank_lines(void)
    }
 }
 
+
 void newlines_cleanup_dup(void)
 {
    chunk_t *pc;
@@ -2354,6 +2525,7 @@ void newlines_cleanup_dup(void)
    }
 }
 
+
 /**
  * Make sure there is a blank line after a commented group of values
  */
