@@ -4,8 +4,6 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: options.h 1495 2009-02-28 17:37:10Z bengardner $
  */
 #ifndef OPTIONS_H_INCLUDED
 #define OPTIONS_H_INCLUDED
@@ -43,12 +41,18 @@ enum lineends_e
    LE_AUTO,    /* keep last */
 };
 
-/** Token position */
+/** Token position - these are bit fields */
 enum tokenpos_e
 {
-   TP_IGNORE,     /* don't change it */
-   TP_LEAD,       /* at the start of a line or leading */
-   TP_TRAIL,      /* at the end of a line or trailing */
+   TP_IGNORE      = 0,     /* don't change it */
+   TP_BREAK       = 1,     /* add a newline before or after the if not present */
+   TP_FORCE       = 2,     /* force a newline on one side and not the other */
+   TP_LEAD        = 4,     /* at the start of a line or leading if wrapped line */
+   TP_LEAD_BREAK  = (TP_LEAD | TP_BREAK),
+   TP_LEAD_FORCE  = (TP_LEAD | TP_FORCE),
+   TP_TRAIL       = 8,     /* at the end of a line or trailing if wrapped line */
+   TP_TRAIL_BREAK = (TP_TRAIL | TP_BREAK),
+   TP_TRAIL_FORCE = (TP_TRAIL | TP_FORCE),
 };
 
 union op_val_t
@@ -94,7 +98,9 @@ enum uncrustify_options
    UO_output_tab_size,          // tab size for output: usually 8
 
    UO_indent_columns,           // ie 3 or 8
+   UO_indent_continue,
    UO_indent_with_tabs,         // 1=only to the 'level' indent, 2=use tabs for indenting
+   UO_indent_cmt_with_tabs,
    //UO_indent_brace_struct,      //TODO: spaces to indent brace after struct/enum/union def
    //UO_indent_paren,             //TODO: indent for open paren on next line (1)
    UO_indent_paren_nl,           // indent-align under paren for open followed by nl
@@ -102,6 +108,7 @@ enum uncrustify_options
    UO_indent_paren_close,        // indent of close paren after a newline
    UO_indent_comma_paren,        // indent of comma if inside a paren
    UO_indent_bool_paren,         // indent of bool if inside a paren
+   UO_indent_first_bool_expr,    // if UO_indent_bool_paren == true, aligns the first expression to the following ones
    UO_pp_indent,                 // indent preproc 1 space per level (add/ignore/remove)
    UO_pp_indent_at_level,        // indent #if, #else, #endif at brace level
    UO_pp_indent_count,
@@ -120,6 +127,8 @@ enum uncrustify_options
    UO_indent_brace,              // spaces to indent '{' from level (usually 0)
    UO_indent_braces,             // whether to indent the braces or not
    UO_indent_braces_no_func,     // whether to not indent the function braces (depends on UO_indent_braces)
+   UO_indent_braces_no_class,    // whether to not indent the class braces (depends on UO_indent_braces)
+   UO_indent_braces_no_struct,   // whether to not indent the struct braces (depends on UO_indent_braces)
    UO_indent_brace_parent,       // indent the braces based on the parent size (if=3, for=4, etc)
    UO_indent_label,              // 0=left >0=col from left, <0=sub from brace indent
    UO_indent_access_spec,        // same as indent_label, but for "private:", "public:"
@@ -136,25 +145,28 @@ enum uncrustify_options
    UO_indent_func_class_param,   // same, but for classes
    UO_indent_func_ctor_var_param,
    UO_indent_template_param,
-   UO_indent_func_param_double,  // double the tab indent for
+   UO_indent_func_param_double,             // double the tab indent for
 
-   UO_indent_func_const,         // indentation for standalone "const" qualifier
-   UO_indent_func_throw,         // indentation for standalone "throw" qualifier
+   UO_indent_func_const,                    // indentation for standalone "const" qualifier
+   UO_indent_func_throw,                    // indentation for standalone "throw" qualifier
 
-   UO_indent_namespace,          // indent stuff inside namespace braces
+   UO_indent_namespace,                     // indent stuff inside namespace braces
+   UO_indent_namespace_level,               // level to indent namespace blocks
+   UO_indent_namespace_limit,               // no indent if namespace is longer than this
    UO_indent_extern,
-   UO_indent_class,              // indent stuff inside class braces
-   UO_indent_class_colon,        // indent stuff after a class colon
+   UO_indent_class,                         // indent stuff inside class braces
+   UO_indent_class_colon,                   // indent stuff after a class colon
 
-   UO_indent_member,             // indent lines broken at a member '.' or '->'
+   UO_indent_member,                        // indent lines broken at a member '.' or '->'
 
-   UO_indent_sing_line_comments, // indent single line ('//') comments on lines before code
+   UO_indent_sing_line_comments,            // indent single line ('//') comments on lines before code
    UO_indent_relative_single_line_comments, // indent single line ('//') comments after code
-   UO_indent_preserve_sql,       // preserve indent of EXEC SQL statement body
+   UO_indent_preserve_sql,                  // preserve indent of EXEC SQL statement body
    UO_indent_align_assign,
 
    UO_indent_else_if,
    UO_indent_var_def_blk,        // indent a variable def block that appears at the top
+   UO_indent_var_def_cont,
 
    /*
     * Misc inter-element spacing
@@ -206,10 +218,15 @@ enum uncrustify_options
    UO_sp_after_comma,           // space after ','
    UO_sp_before_comma,          // space before ','
 
+   UO_sp_before_ellipsis,       // space before '...'
+
    UO_sp_arith,                 // space around + - / * etc
    UO_sp_bool,                  // space around || &&
+   UO_sp_pp_concat,             // space around ##
+   UO_sp_pp_stringify,          // space around #
    UO_sp_compare,               // space around < > ==, etc
    UO_sp_assign,                // space around =, +=, etc
+   UO_sp_assign_default,        // space around '=' in prototype
    UO_sp_before_assign,         // space before =, +=, etc
    UO_sp_after_assign,          // space after =, +=, etc
    UO_sp_enum_assign,           // space around = in enum
@@ -245,6 +262,8 @@ enum uncrustify_options
    UO_sp_before_semi,           // space before all ';'
    UO_sp_before_semi_for,       // space before the two ';' in a for() - non-empty
    UO_sp_before_semi_for_empty, // space before ';' in empty for statement
+   UO_sp_after_semi,
+   UO_sp_after_semi_for,
    UO_sp_after_semi_for_empty,  // space after final ';' in empty for statement
    UO_sp_inside_braces,         // space inside '{' and '}' - "{ 1, 2, 3 }"
    UO_sp_inside_braces_empty,   // space inside '{' and '}' - "{ }"
@@ -287,11 +306,18 @@ enum uncrustify_options
    UO_sp_before_send_oc_colon,
    UO_sp_after_send_oc_colon,
    UO_sp_after_oc_type,
+   UO_sp_after_oc_return_type,
+   UO_sp_after_oc_at_sel,
+   UO_sp_after_oc_at_sel_parens,
+   UO_sp_inside_oc_at_sel_parens,
+   UO_sp_before_oc_block_caret,
+   UO_sp_after_oc_block_caret,
    UO_sp_cond_colon,
    UO_sp_cond_question,
    UO_sp_case_label,
    UO_sp_range,
    UO_sp_cmt_cpp_start,
+   UO_sp_endif_cmt,
 
    /*
     * Line splitting options (for long lines)
@@ -321,6 +347,7 @@ enum uncrustify_options
    UO_align_right_cmt_span,       // align comment that end lines. 0=don't align
    UO_align_right_cmt_mix,        // mix comments after '}' and preproc with others
    UO_align_right_cmt_gap,
+   UO_align_right_cmt_at_col,     // align comment that end lines at or beyond column N; 'pulls in' comments as a bonus side effect
    UO_align_func_params,          // align prototype variable defs on variable
    UO_align_same_func_call_params,
    UO_align_var_def_span,         // align variable defs on variable (span for regular stuff)
@@ -331,119 +358,131 @@ enum uncrustify_options
    UO_align_var_def_star_style,   // see UO_align_typedef_star_style
    UO_align_var_def_amp_style,    // see UO_align_typedef_star_style
    UO_align_var_def_colon,        // align the colon in struct bit fields
+   UO_align_var_def_attribute,
    UO_align_var_struct_span,      // span for struct/union (0=don't align)
    UO_align_var_struct_thresh,    // threshold for struct/union, 0=no limit
    UO_align_var_struct_gap,       // gap for struct/union
-   UO_align_pp_define_span,       // align bodies in #define statments
+   UO_align_pp_define_span,       // align bodies in #define statements
    //UO_align_pp_define_col_min,    //TODO: min column for a #define value
    //UO_align_pp_define_col_max,    //TODO: max column for a #define value
    UO_align_pp_define_gap,        // min space between define label and value "#define a <---> 16"
    //UO_align_enum_col_min,         //TODO: the min column for enum '=' alignment
    //UO_align_enum_col_max,         //TODO: the max column for enum '=' alignment
-   UO_align_struct_init_span,     // align structure initializer values
-   UO_align_func_proto_span,      // align function prototypes
-   UO_align_func_proto_gap,       // align function prototypes
+   UO_align_struct_init_span,      // align structure initializer values
+   UO_align_func_proto_span,       // align function prototypes
+   UO_align_func_proto_gap,        // align function prototypes
    UO_align_on_operator,
-   UO_align_mix_var_proto,        // mix function prototypes and variable decl
-   UO_align_single_line_func,     // mix single line function with prototypes
-   UO_align_single_line_brace,    // align the open brace of single line functions
-   UO_align_single_line_brace_gap,// gap for align_single_line_brace
-   UO_align_oc_msg_spec_span,     // align ObjC msg spec
-   UO_align_number_left,          // left-align numbers (not fully supported, yet)
-   UO_align_typedef_span,         // align single-line typedefs
-   UO_align_typedef_gap,          // minimum spacing
-   UO_align_typedef_func,         // how to align func type with types
-   UO_align_typedef_star_style,   // Start aligning style
-                                  // 0: '*' not part of type
-                                  // 1: '*' part of the type - no space
-                                  // 2: '*' part of type, dangling
-   UO_align_typedef_amp_style,    // align_typedef_star_style for ref '&' stuff
-   //UO_align_struct_array_brace,   // TODO: align array of structure initializers
+   UO_align_mix_var_proto,         // mix function prototypes and variable decl
+   UO_align_single_line_func,      // mix single line function with prototypes
+   UO_align_single_line_brace,     // align the open brace of single line functions
+   UO_align_single_line_brace_gap, // gap for align_single_line_brace
+   UO_align_oc_msg_spec_span,      // align ObjC msg spec
+   UO_align_number_left,           // left-align numbers (not fully supported, yet)
+   UO_align_typedef_span,          // align single-line typedefs
+   UO_align_typedef_gap,           // minimum spacing
+   UO_align_typedef_func,          // how to align func type with types
+   UO_align_typedef_star_style,    // Start aligning style
+                                   // 0: '*' not part of type
+                                   // 1: '*' part of the type - no space
+                                   // 2: '*' part of type, dangling
+   UO_align_typedef_amp_style,     // align_typedef_star_style for ref '&' stuff
+   //UO_align_struct_array_brace,  // TODO: align array of structure initializers
    UO_align_left_shift,
+//    UO_align_oc_msg_colon,
+   UO_align_oc_msg_colon_span,
+   UO_align_oc_decl_colon,
 
 
    /*
     * Newline adding and removing options
     */
 
-   UO_nl_fdef_brace,              // "int foo() {" vs "int foo()\n{"
-   UO_nl_func_paren,              // newline between function and open paren
-   UO_nl_func_decl_start,         // newline after the '(' in a function decl
-   UO_nl_func_decl_args,          // newline after each ',' in a function decl
-   UO_nl_func_decl_end,           // newline before the ')' in a function decl
-   UO_nl_func_type_name,          // newline between return type and func name in def
+   UO_nl_fdef_brace,                 // "int foo() {" vs "int foo()\n{"
+   UO_nl_func_paren,                 // newline between function and open paren
+   UO_nl_func_decl_start,            // newline after the '(' in a function decl
+   UO_nl_func_decl_start_single,
+   UO_nl_func_decl_args,             // newline after each ',' in a function decl
+   UO_nl_func_decl_end,              // newline before the ')' in a function decl
+   UO_nl_func_decl_end_single,
+   UO_nl_func_decl_empty,            // as above, but for empty parens '()'
+   UO_nl_func_type_name,             // newline between return type and func name in def
+   UO_nl_func_type_name_class,       // newline between return type and func name in class
    UO_nl_func_scope_name,
-   UO_nl_func_proto_type_name,    // nl_func_type_name, but for prottypes
-   UO_nl_func_var_def_blk,        // newline after a block of variable defs
-   UO_nl_before_case,             // newline before 'case' statement
+   UO_nl_func_proto_type_name,       // nl_func_type_name, but for prottypes
+   UO_nl_func_var_def_blk,           // newline after a block of variable defs
+   UO_nl_before_case,                // newline before 'case' statement
    UO_nl_before_throw,
-   UO_nl_after_return,            /* newline after return statement */
+   UO_nl_after_return,               /* newline after return statement */
    UO_nl_return_expr,
-   UO_nl_after_case,              /* disallow nested "case 1: a=3;" */
-   UO_nl_after_semicolon,         // disallow multiple statements on a line "a=1;b=4;"
-   UO_nl_after_brace_open,        // force a newline after a brace open
-   UO_nl_after_brace_open_cmt,    // put the newline before the comment
-   UO_nl_after_vbrace_open,       // force a newline after a virtual brace open
-   UO_nl_after_brace_close,       // force a newline after a brace close
-   UO_nl_fcall_brace,             /* newline between function call and open brace */
-   UO_nl_squeeze_ifdef,           /* no blanks after #ifxx, #elxx, or before #endif */
-   UO_nl_enum_brace,              /* nl between enum and brace */
-   UO_nl_struct_brace,            /* nl between struct and brace */
-   UO_nl_union_brace,             /* nl between union and brace */
-   UO_nl_assign_brace,            /* nl between '=' and brace */
-   UO_nl_assign_square,           /* nl between '=' and '[' */
-   UO_nl_after_square_assign,     /* nl after '= [' */
-   UO_nl_class_brace,             /* nl between class name and brace */
-   UO_nl_namespace_brace,         /* nl between namespace name and brace */
+   UO_nl_after_case,                 /* disallow nested "case 1: a=3;" */
+   UO_nl_after_semicolon,            // disallow multiple statements on a line "a=1;b=4;"
+   UO_nl_after_brace_open,           // force a newline after a brace open
+   UO_nl_after_brace_open_cmt,       // put the newline before the comment
+   UO_nl_after_vbrace_open,          // force a newline after a virtual brace open
+   UO_nl_after_vbrace_open_empty,    // force a newline after a virtual brace open
+   UO_nl_after_brace_close,          // force a newline after a brace close
+   UO_nl_after_vbrace_close,         // force a newline after a virtual brace close
+   UO_nl_fcall_brace,                /* newline between function call and open brace */
+   UO_nl_squeeze_ifdef,              /* no blanks after #ifxx, #elxx, or before #endif */
+   UO_nl_enum_brace,                 /* nl between enum and brace */
+   UO_nl_struct_brace,               /* nl between struct and brace */
+   UO_nl_union_brace,                /* nl between union and brace */
+   UO_nl_assign_brace,               /* nl between '=' and brace */
+   UO_nl_assign_square,              /* nl between '=' and '[' */
+   UO_nl_after_square_assign,        /* nl after '= [' */
+   UO_nl_class_brace,                /* nl between class name and brace */
+   UO_nl_namespace_brace,            /* nl between namespace name and brace */
 
-   UO_nl_do_brace,                /* nl between do and { */
-   UO_nl_if_brace,                /* nl between if and { */
-   UO_nl_for_brace,               /* nl between for and { */
+   UO_nl_brace_brace,                /* nl between '{{' or '}}' */
+   UO_nl_do_brace,                   /* nl between do and { */
+   UO_nl_if_brace,                   /* nl between if and { */
+   UO_nl_for_brace,                  /* nl between for and { */
    UO_nl_else_if,
-   UO_nl_else_brace,              /* nl between else and { */
-   UO_nl_finally_brace,           /* nl between finally and { */
-   UO_nl_brace_finally,           /* nl between } and finally */
-   UO_nl_try_brace,               /* nl between try and { */
-   UO_nl_getset_brace,            /* nl between get/set and { */
-   UO_nl_catch_brace,             /* nl between catch and { */
-   UO_nl_brace_catch,             /* nl between } and catch */
-   UO_nl_while_brace,             /* nl between while and { */
-   UO_nl_switch_brace,            /* nl between switch and { */
-   UO_nl_brace_else,              // nl between } and else
-   UO_nl_brace_while,             // nl between } and while of do stmt
+   UO_nl_else_brace,                 /* nl between else and { */
+   UO_nl_finally_brace,              /* nl between finally and { */
+   UO_nl_brace_finally,              /* nl between } and finally */
+   UO_nl_try_brace,                  /* nl between try and { */
+   UO_nl_getset_brace,               /* nl between get/set and { */
+   UO_nl_catch_brace,                /* nl between catch and { */
+   UO_nl_brace_catch,                /* nl between } and catch */
+   UO_nl_while_brace,                /* nl between while and { */
+   UO_nl_using_brace,
+   UO_nl_switch_brace,               /* nl between switch and { */
+   UO_nl_brace_else,                 // nl between } and else
+   UO_nl_brace_while,                // nl between } and while of do stmt
 
-   UO_nl_multi_line_cond,         /* nl between ) and { when cond spans >=2 lines */
-   UO_nl_elseif_brace,            // nl between close paren and open brace in 'else if () {'
+   UO_nl_multi_line_cond,            /* nl between ) and { when cond spans >=2 lines */
+   UO_nl_elseif_brace,               // nl between close paren and open brace in 'else if () {'
 
-   UO_nl_multi_line_define,       // nl after define XXX for multi-line define
+   UO_nl_multi_line_define,          // nl after define XXX for multi-line define
 
-   UO_nl_before_if,               // nl before if
-   UO_nl_after_if,                // nl after if/else
-   UO_nl_before_for,              // nl before for
-   UO_nl_after_for,               // nl after for close
-   UO_nl_before_while,            // nl before while
-   UO_nl_after_while,             // nl after while close
-   UO_nl_before_switch,           // nl before switch
-   UO_nl_after_switch,            // nl after switch close
-   UO_nl_before_do,               // nl before do
-   UO_nl_after_do,                // nl after while of do
-   UO_nl_ds_struct_enum_cmt,      // nl between commented-elements of struct/enum
+   UO_nl_before_if,                  // nl before if
+   UO_nl_after_if,                   // nl after if/else
+   UO_nl_before_for,                 // nl before for
+   UO_nl_after_for,                  // nl after for close
+   UO_nl_before_while,               // nl before while
+   UO_nl_after_while,                // nl after while close
+   UO_nl_before_switch,              // nl before switch
+   UO_nl_after_switch,               // nl after switch close
+   UO_nl_before_do,                  // nl before do
+   UO_nl_after_do,                   // nl after while of do
+   UO_nl_ds_struct_enum_cmt,         // nl between commented-elements of struct/enum
    UO_nl_ds_struct_enum_close_brace, // force nl before } of struct/union/enum
 
-   UO_nl_define_macro,            // alter newlines in #define macros
-   UO_nl_start_of_file,           // alter newlines at the start of file
-   UO_nl_start_of_file_min,       // min number of newlines at the start of the file
-   UO_nl_end_of_file,             // alter newlines at the end of file
-   UO_nl_end_of_file_min,         // min number of newlines at the end of the file
+   UO_nl_define_macro,               // alter newlines in #define macros
+   UO_nl_start_of_file,              // alter newlines at the start of file
+   UO_nl_start_of_file_min,          // min number of newlines at the start of the file
+   UO_nl_end_of_file,                // alter newlines at the end of file
+   UO_nl_end_of_file_min,            // min number of newlines at the end of the file
 
-   UO_nl_class_colon,             // nl before/after class colon (tied to UO_pos_class_colon)
-   UO_nl_class_init_args,         // newline after comma in class init args
-   UO_nl_collapse_empty_body,     // change { \n } into {}
-   UO_nl_class_leave_one_liners,  // leave one-line function bodies in "class xx { here }"
-   UO_nl_assign_leave_one_liners, // leave one-line assign bodies in "foo_t f = { a, b, c };"
-   UO_nl_enum_leave_one_liners,   // leave one-line enum bodies in "enum FOO { BAR = 5 };"
-   UO_nl_getset_leave_one_liners, // leave one-line get/set bodies
-   UO_nl_func_leave_one_liners,   // leave one-line function def bodies
+   UO_nl_class_colon,                // nl before/after class colon (tied to UO_pos_class_colon)
+   UO_nl_class_init_args,            // newline after comma in class init args
+   UO_nl_collapse_empty_body,        // change { \n } into {}
+   UO_nl_class_leave_one_liners,     // leave one-line function bodies in "class xx { here }"
+   UO_nl_assign_leave_one_liners,    // leave one-line assign bodies in "foo_t f = { a, b, c };"
+   UO_nl_enum_leave_one_liners,      // leave one-line enum bodies in "enum FOO { BAR = 5 };"
+   UO_nl_getset_leave_one_liners,    // leave one-line get/set bodies
+   UO_nl_func_leave_one_liners,      // leave one-line function def bodies
    UO_nl_if_leave_one_liners,
 
    UO_nl_template_class,          // newline between '>' and class in "template <x> class"
@@ -455,6 +494,8 @@ enum uncrustify_options
    UO_pos_arith,                  // position of trailing/leading arithmetic ops
    UO_pos_assign,                 // position of trailing/leading =
    UO_pos_bool,                   // position of trailing/leading &&/||
+   UO_pos_compare,                // position of trailing/leading <=/>, etc
+   UO_pos_conditional,            // position of trailing/leading (b ? t : f)
    UO_pos_comma,                  // position of comma in functions
    UO_pos_class_comma,            // position of comma in constructor init list
    UO_pos_class_colon,            // position of trailing/leading class colon
@@ -493,19 +534,24 @@ enum uncrustify_options
    UO_mod_paren_on_return,        // add or remove paren on return
    UO_mod_full_brace_nl,          // max number of newlines to span w/o braces
    UO_mod_full_brace_if,          // add or remove braces on if
+   UO_mod_full_brace_if_chain,
    UO_mod_full_brace_for,         // add or remove braces on for
    UO_mod_full_brace_do,          // add or remove braces on do
    UO_mod_full_brace_while,       // add or remove braces on while
+   UO_mod_full_brace_using,       // add or remove braces on using
    UO_mod_pawn_semicolon,         // add optional semicolons
    UO_mod_full_brace_function,    // add optional braces on Pawn functions
    UO_mod_full_paren_if_bool,
    UO_mod_remove_extra_semicolon, // remove extra semicolons
    UO_mod_add_long_function_closebrace_comment,
    UO_mod_add_long_switch_closebrace_comment,
+   UO_mod_add_long_ifdef_else_comment,
+   UO_mod_add_long_ifdef_endif_comment,
    UO_mod_sort_import,
    UO_mod_sort_using,
    UO_mod_sort_include,
    UO_mod_move_case_break,
+   UO_mod_case_brace,
    UO_mod_remove_empty_return,
 
 
@@ -514,6 +560,7 @@ enum uncrustify_options
     */
 
    UO_cmt_width,                // column to wrap comments
+   UO_cmt_reflow_mode,          // comment reflow style
    UO_cmt_indent_multi,         // change left indent of multiline comments
    UO_cmt_star_cont,            // put a star on subsequent comment lines
    UO_cmt_sp_before_star_cont,  // # of spaces for subsequent comment lines (before possible star)
