@@ -4,10 +4,7 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: output.cpp 1469 2009-01-28 01:33:24Z bengardner $
  */
-
 #include "uncrustify_types.h"
 #include "prototypes.h"
 #include "chunk_list.h"
@@ -22,10 +19,13 @@ struct cmt_reflow
 {
    chunk_t    *pc;
    int        column;      /* Column of the comment start */
-   int        br_column;   /* Brace column (for indenting with tabs) */
+   int        brace_col;   /* Brace column (for indenting with tabs) */
+   int        base_col;    /* Base column (for indenting with tabs) */
    int        word_count;  /* number of words on this line */
    bool       kw_subst;    /* do keyword substitution */
-   const char *cont_text;  /* fixed text to output at the start of the line (3-chars) */
+   int        xtra_indent; /* extra indent of non-first lines (0 or 1) */
+   const char *cont_text;  /* fixed text to output at the start of a line (0 to 3 chars) */
+   bool       reflow;      /* reflow the current line */
 };
 
 
@@ -47,6 +47,7 @@ static void add_char(char ch)
       fputs(cpd.newline, cpd.fout);
       cpd.column      = 1;
       cpd.did_newline = 1;
+      cpd.spaces      = 0;
    }
 
    /* convert a newline into the LF/CRLF/CR sequence */
@@ -55,12 +56,14 @@ static void add_char(char ch)
       fputs(cpd.newline, cpd.fout);
       cpd.column      = 1;
       cpd.did_newline = 1;
+      cpd.spaces      = 0;
    }
    else if (ch == '\r')
    {
       /* do not output '\r' */
       cpd.column      = 1;
       cpd.did_newline = 1;
+      cpd.spaces      = 0;
    }
    else
    {
@@ -74,8 +77,18 @@ static void add_char(char ch)
          }
          return;
       }
+      else if (ch == ' ')
+      {
+         cpd.spaces++;
+         cpd.column++;
+      }
       else
       {
+         while (cpd.spaces > 0)
+         {
+            fputc(' ', cpd.fout);
+            cpd.spaces--;
+         }
          fputc(ch, cpd.fout);
          if (ch == '\t')
          {
@@ -90,6 +103,7 @@ static void add_char(char ch)
    last_char = ch;
 }
 
+
 static void add_text(const char *text)
 {
    char ch;
@@ -101,6 +115,7 @@ static void add_text(const char *text)
    }
 }
 
+
 static void add_text_len(const char *text, int len)
 {
    while (len-- > 0)
@@ -110,6 +125,7 @@ static void add_text_len(const char *text, int len)
    }
 }
 
+
 /**
  * Count the number of characters to the end of the next chunk of text.
  * If it exceeds the limit, return true.
@@ -161,25 +177,36 @@ static void output_to_column(int column, bool allow_tabs)
    }
 }
 
-/**
- * Output to the column using indent_with_tabs as the rule.
- *
- * @param column    the column that we should end up in
- * @param brace_col the level that indent_with_tabs=1 should tab to
- */
-static void output_indent(int column, int brace_col)
-{
-   cpd.did_newline = 0;
-   if ((cpd.column == 1) && (cpd.settings[UO_indent_with_tabs].n != 0))
-   {
-      if (cpd.settings[UO_indent_with_tabs].n == 2)
-      {
-         brace_col = column;
-      }
 
+/**
+ * Output a comment to the column using indent_with_tabs and
+ * indent_cmt_with_tabs as the rules.
+ * base_col is the indent of the first line of the comment.
+ * On the first line, column == base_col.
+ * On subsequnet lines, column >= base_col.
+ *
+ * @param brace_col the brace-level indent of the comment
+ * @param base_col  the indent of the start of the comment (multiline)
+ * @param column    the column that we should end up in
+ */
+static void cmt_output_indent(int brace_col, int base_col, int column)
+{
+   int iwt;
+   int tab_col;
+
+   iwt = cpd.settings[UO_indent_cmt_with_tabs].b ? 2 :
+         (cpd.settings[UO_indent_with_tabs].n ? 1 : 0);
+
+   tab_col = (iwt == 0) ? 0 : ((iwt == 1) ? brace_col : base_col);
+
+   //LOG_FMT(LSYS, "%s(brace=%d base=%d col=%d iwt=%d) tab=%d cur=%d\n",
+   //        __func__, brace_col, base_col, column, iwt, tab_col, cpd.column);
+
+   cpd.did_newline = 0;
+   if ((iwt == 2) || ((cpd.column == 1) && (iwt == 1)))
+   {
       /* tab out as far as possible and then use spaces */
-      int nc;
-      while ((nc = next_tab_column(cpd.column)) <= brace_col)
+      while (next_tab_column(cpd.column) <= tab_col)
       {
          add_text("\t");
       }
@@ -192,6 +219,7 @@ static void output_indent(int column, int brace_col)
    }
 }
 
+
 void output_parsed(FILE *pfile)
 {
    chunk_t *pc;
@@ -205,7 +233,7 @@ void output_parsed(FILE *pfile)
    fprintf(pfile, "Line      Tag          Parent     Columns  Br/Lvl/pp Flag Nl  Text");
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
    {
-      fprintf(pfile, "\n%3d> %13.13s[%13.13s][%2d/%2d/%2d][%d/%d/%d][%8x][%d-%d]",
+      fprintf(pfile, "\n%3d> %13.13s[%13.13s][%2d/%2d/%2d][%d/%d/%d][%10" PRIx64 "][%d-%d]",
               pc->orig_line, get_token_name(pc->type),
               get_token_name(pc->parent_type),
               pc->column, pc->orig_col, pc->orig_col_end,
@@ -232,6 +260,7 @@ void output_parsed(FILE *pfile)
    fflush(pfile);
 }
 
+
 void output_options(FILE *pfile)
 {
    int idx;
@@ -259,6 +288,7 @@ void output_options(FILE *pfile)
    }
 }
 
+
 /**
  * This renders the chunk list to a file.
  */
@@ -272,7 +302,14 @@ void output_text(FILE *pfile)
 
    cpd.fout = pfile;
 
-   cpd.column = 1;
+   cpd.did_newline = 1;
+   cpd.column      = 1;
+
+   if (cpd.bom != NULL)
+   {
+      add_text_len(cpd.bom->str, cpd.bom->len);
+   }
+
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
    {
       if (pc->type == CT_NEWLINE)
@@ -287,6 +324,34 @@ void output_text(FILE *pfile)
       }
       else if (pc->type == CT_NL_CONT)
       {
+         /* FIXME: this really shouldn't be done here! */
+         if ((pc->flags & PCF_WAS_ALIGNED) == 0)
+         {
+            if (cpd.settings[UO_sp_before_nl_cont].a & AV_REMOVE)
+            {
+               pc->column = cpd.column + (cpd.settings[UO_sp_before_nl_cont].a == AV_FORCE);
+            }
+            else
+            {
+               /* Try to keep the same relative spacing */
+               prev = chunk_get_prev(pc);
+               while ((prev != NULL) && (prev->orig_col == 0) && (prev->nl_count == 0))
+               {
+                  prev = chunk_get_prev(prev);
+               }
+
+               if ((prev != NULL) && (prev->nl_count == 0))
+               {
+                  int orig_sp = (pc->orig_col - prev->orig_col_end);
+                  pc->column = cpd.column + orig_sp;
+                  if ((cpd.settings[UO_sp_before_nl_cont].a != AV_IGNORE) &&
+                      (pc->column < (cpd.column + 1)))
+                  {
+                     pc->column = cpd.column + 1;
+                  }
+               }
+            }
+         }
          output_to_column(pc->column, (cpd.settings[UO_indent_with_tabs].n == 2));
          add_char('\\');
          add_char('\n');
@@ -381,10 +446,9 @@ void output_text(FILE *pfile)
                prev       = chunk_get_prev(pc);
                allow_tabs = (cpd.settings[UO_align_with_tabs].b &&
                              ((pc->flags & PCF_WAS_ALIGNED) != 0) &&
-                             (((pc->column - 1) % cpd.settings[UO_output_tab_size].n) == 0) &&
                              ((prev->column + prev->len + 1) != pc->column));
             }
-            LOG_FMT(LOUTIND, " %d -", pc->column);
+            LOG_FMT(LOUTIND, " %d(%d) -", pc->column, allow_tabs);
          }
 
          output_to_column(pc->column, allow_tabs);
@@ -395,12 +459,62 @@ void output_text(FILE *pfile)
 }
 
 /**
- * Given a multi-line comemnt block that starts in column X, figure out how
- * much subsequent lines should be indented.
+ * Checks for and updates the lead chars.
  *
- * The answer is either 0 or 1.
+ * @param line the comment line
+ * @return 0=not present, >0=number of chars that are part of the lead
+ */
+static int cmt_parse_lead(const char *line, int is_last)
+{
+   int len = 0;
+
+   while ((len < 32) && (line[len] != 0))
+   {
+      if ((len > 0) && (line[len] == '/'))
+      {
+         /* ignore combined comments */
+         int tmp = len + 1;
+         while (unc_isspace(line[tmp]))
+         {
+            tmp++;
+         }
+         if (line[tmp] == '/')
+         {
+            return 1;
+         }
+         break;
+      }
+      else if (strchr("*|\\#+", line[len]) == NULL)
+      {
+         break;
+      }
+      len++;
+   }
+
+   if (len > 30)
+   {
+      return 1;
+   }
+
+   if ((len > 0) && ((line[len] == 0) || unc_isspace(line[len])))
+   {
+      return len;
+   }
+   if (is_last && (len > 0))
+   {
+      return len;
+   }
+   return 0;
+}
+
+/**
+ * Scans a multiline comment to determine the following:
+ *  - the extra indent of the non-first line (0 or 1)
+ *  - the continuation text ('' or '* ')
  *
  * The decision is based on:
+ *  - cmt_indent_multi
+ *  - cmt_star_cont
  *  - the first line length
  *  - the second line leader length
  *  - the last line length
@@ -418,16 +532,18 @@ void output_text(FILE *pfile)
  * @param start_col Starting column
  * @return 0 or 1
  */
-static int calculate_comment_body_indent(const char *str, int len, int start_col)
+static void calculate_comment_body_indent(cmt_reflow &cmt, const char *str, int len)
 {
    int idx       = 0;
    int first_len = 0;
    int last_len  = 0;
    int width     = 0;
 
+   cmt.xtra_indent = 0;
+
    if (!cpd.settings[UO_cmt_indent_multi].b)
    {
-      return(0);
+      return;
    }
 
    if (cpd.settings[UO_cmt_multi_check_last].b)
@@ -471,7 +587,7 @@ static int calculate_comment_body_indent(const char *str, int len, int start_col
 
    /* Scan the second line */
    width = 0;
-   for ( /* nada */; idx < len; idx++)
+   for (/* nada */; idx < len - 1; idx++)
    {
       if ((str[idx] == ' ') || (str[idx] == '\t'))
       {
@@ -498,6 +614,7 @@ static int calculate_comment_body_indent(const char *str, int len, int start_col
       }
       else
       {
+         width = 0;
          break;
       }
    }
@@ -507,12 +624,13 @@ static int calculate_comment_body_indent(const char *str, int len, int start_col
    /*TODO: make the first_len minimum (4) configurable? */
    if ((first_len == last_len) && ((first_len > 4) || (first_len == width)))
    {
-      return(0);
+      return;
    }
 
-   return((width == 2) ? 0 : 1);
+   cmt.xtra_indent = ((width == 2) ? 0 : 1);
 }
 
+
 static chunk_t *get_next_function(chunk_t *pc)
 {
    while ((pc = chunk_get_next(pc)) != NULL)
@@ -527,6 +645,7 @@ static chunk_t *get_next_function(chunk_t *pc)
    return(NULL);
 }
 
+
 static chunk_t *get_next_class(chunk_t *pc)
 {
    while ((pc = chunk_get_next(pc)) != NULL)
@@ -539,11 +658,12 @@ static chunk_t *get_next_class(chunk_t *pc)
    return(NULL);
 }
 
+
 /**
  * Adds the javadoc-style @param and @return stuff, based on the params and
  * return value for pc.
  * If the arg list is '()' or '(void)', then no @params are added.
- * Likewise, if the reutrn value is 'void', then no @return is added.
+ * Likewise, if the return value is 'void', then no @return is added.
  */
 static void add_comment_javaparam(chunk_t *pc, cmt_reflow& cmt)
 {
@@ -625,6 +745,7 @@ static void add_comment_javaparam(chunk_t *pc, cmt_reflow& cmt)
    }
 }
 
+
 /**
  * text starts with '$('. see if this matches a keyword and add text based
  * on that keyword.
@@ -686,6 +807,7 @@ static int add_comment_kw(const char *text, int len, cmt_reflow& cmt)
    return(0);
 }
 
+
 /**
  * Outputs a comment. The initial opening '//' may be included in the text.
  * Subsequent openings (if combining comments), should not be included.
@@ -719,15 +841,30 @@ static void add_comment_text(const char *text, int len,
       }
 
       /* Split the comment */
-      if ((text[idx] == '\n') ||
-          ((text[idx] == ' ') &&
-           (cpd.settings[UO_cmt_width].n > 0) &&
-           ((cpd.column > cpd.settings[UO_cmt_width].n) ||
-            next_word_exceeds_limit(text + idx))))
+      if (text[idx] == '\n')
       {
          in_word = false;
          add_char('\n');
-         output_indent(cmt.column, cmt.br_column);
+         cmt_output_indent(cmt.brace_col, cmt.base_col, cmt.column);
+         if (cmt.xtra_indent)
+         {
+            add_char(' ');
+         }
+         add_text(cmt.cont_text);
+      }
+      else if (cmt.reflow &&
+               (text[idx] == ' ') &&
+               (cpd.settings[UO_cmt_width].n > 0) &&
+               ((cpd.column > cpd.settings[UO_cmt_width].n) ||
+                next_word_exceeds_limit(text + idx)))
+      {
+         in_word = false;
+         add_char('\n');
+         cmt_output_indent(cmt.brace_col, cmt.base_col, cmt.column);
+         if (cmt.xtra_indent)
+         {
+            add_char(' ');
+         }
          add_text(cmt.cont_text);
       }
       else
@@ -752,21 +889,38 @@ static void add_comment_text(const char *text, int len,
    }
 }
 
+
 static void output_cmt_start(cmt_reflow& cmt, chunk_t *pc)
 {
-   cmt.word_count = 0;
-   cmt.pc         = pc;
-   cmt.column     = pc->column;
-   cmt.br_column  = pc->column_indent;
+   cmt.pc          = pc;
+   cmt.column      = pc->column;
+   cmt.brace_col   = pc->column_indent;
+   cmt.base_col    = pc->column_indent;
+   cmt.word_count  = 0;
+   cmt.kw_subst    = false;
+   cmt.xtra_indent = 0;
+   cmt.cont_text   = "";
+   cmt.reflow      = false;
+
+   if (cmt.brace_col == 0)
+   {
+      cmt.brace_col = 1 + (pc->brace_level * cpd.settings[UO_output_tab_size].n);
+   }
+
+   //LOG_FMT(LSYS, "%s: line %d, brace=%d base=%d col=%d orig=%d aligned=%x\n",
+   //        __func__, pc->orig_line, cmt.brace_col, cmt.base_col, cmt.column, pc->orig_col,
+   //        pc->flags & (PCF_WAS_ALIGNED | PCF_RIGHT_COMMENT));
 
    if ((pc->parent_type == CT_COMMENT_START) ||
        (pc->parent_type == CT_COMMENT_WHOLE))
    {
       if (!cpd.settings[UO_indent_col1_comment].b &&
-          (pc->orig_col == 1))
+          (pc->orig_col == 1) &&
+          !(pc->flags & PCF_INSERTED))
       {
          cmt.column    = 1;
-         cmt.br_column = 1;
+         cmt.base_col  = 1;
+         cmt.brace_col = 1;
       }
    }
    else if (pc->parent_type == CT_COMMENT_END)
@@ -783,12 +937,28 @@ static void output_cmt_start(cmt_reflow& cmt, chunk_t *pc)
       }
    }
 
+   /* tab aligning code */
+   if (cpd.settings[UO_indent_cmt_with_tabs].b &&
+       ((pc->parent_type == CT_COMMENT_END) ||
+        (pc->parent_type == CT_COMMENT_WHOLE)))
+   {
+      cmt.column = next_tab_column(cmt.column - 1);
+      //LOG_FMT(LSYS, "%s: line %d, orig:%d new:%d\n",
+      //        __func__, pc->orig_line, pc->column, cmt.column);
+      pc->column = cmt.column;
+   }
+   cmt.base_col = cmt.column;
+
+   //LOG_FMT(LSYS, "%s: -- brace=%d base=%d col=%d\n",
+   //        __func__, cmt.brace_col, cmt.base_col, cmt.column);
+
    /* Bump out to the column */
-   output_indent(cmt.column, cmt.br_column);
+   cmt_output_indent(cmt.brace_col, cmt.base_col, cmt.column);
 
    cmt.kw_subst = (pc->flags & PCF_INSERTED) != 0;
 }
 
+
 /**
  * Checks to see if the current comment can be combined with the next comment.
  * The two can be combined if:
@@ -813,8 +983,8 @@ static bool can_combine_comment(chunk_t *pc, cmt_reflow& cmt)
       if ((next != NULL) &&
           (next->type == pc->type) &&
           (((next->column == 1) && (pc->column == 1)) ||
-           ((next->column == cmt.br_column) && (pc->column == cmt.br_column)) ||
-           ((next->column > cmt.br_column) && (pc->parent_type == CT_COMMENT_END))))
+           ((next->column == cmt.base_col) && (pc->column == cmt.base_col)) ||
+           ((next->column > cmt.base_col) && (pc->parent_type == CT_COMMENT_END))))
       {
          return(true);
       }
@@ -822,6 +992,7 @@ static bool can_combine_comment(chunk_t *pc, cmt_reflow& cmt)
    return(false);
 }
 
+
 /**
  * Outputs the C comment at pc.
  * C comment combining is done here
@@ -833,6 +1004,7 @@ static chunk_t *output_comment_c(chunk_t *first)
    cmt_reflow cmt;
 
    output_cmt_start(cmt, first);
+   cmt.reflow = (cpd.settings[UO_cmt_reflow_mode].n != 1);
 
    cmt.cont_text = cpd.settings[UO_cmt_star_cont].b ? " *" : "  ";
 
@@ -867,6 +1039,7 @@ static chunk_t *output_comment_c(chunk_t *first)
    return(pc);
 }
 
+
 /**
  * Outputs the CPP comment at pc.
  * CPP comment combining is done here
@@ -878,6 +1051,7 @@ static chunk_t *output_comment_cpp(chunk_t *first)
    cmt_reflow cmt;
 
    output_cmt_start(cmt, first);
+   cmt.reflow = (cpd.settings[UO_cmt_reflow_mode].n != 1);
 
    /* CPP comments can't be grouped unless they are converted to C comments */
    if (!cpd.settings[UO_cmt_cpp_to_c].b)
@@ -893,7 +1067,7 @@ static chunk_t *output_comment_cpp(chunk_t *first)
          add_comment_text(first->str, 2, cmt, false);
 
          const char *tmp = first->str + 2;
-         int        len = first->len - 2;
+         int        len  = first->len - 2;
 
          if (cpd.settings[UO_sp_cmt_cpp_start].a & AV_REMOVE)
          {
@@ -967,6 +1141,40 @@ static chunk_t *output_comment_cpp(chunk_t *first)
    return(pc);
 }
 
+
+static void cmt_trim_whitespace(int& line_len, char *line, bool in_preproc)
+{
+   /* Remove trailing whitespace on the line */
+   while ((line_len > 0) &&
+          ((line[line_len - 1] == ' ') ||
+           (line[line_len - 1] == '\t')))
+   {
+      line_len--;
+   }
+
+   /* If in a preproc, shift any bs-nl back to the comment text */
+   if (in_preproc && (line_len > 1) && (line[line_len - 1] == '\\'))
+   {
+      bool do_space = false;
+
+      /* If there was any space before the backslash, change it to 1 space */
+      line_len--;
+      while ((line_len > 0) &&
+             ((line[line_len - 1] == ' ') ||
+              (line[line_len - 1] == '\t')))
+      {
+         do_space = true;
+         line_len--;
+      }
+      if (do_space)
+      {
+         line[line_len++] = ' ';
+      }
+      line[line_len++] = '\\';
+   }
+}
+
+
 /**
  * A multiline comment -- woopeee!
  * The only trick here is that we have to trim out whitespace characters
@@ -974,47 +1182,42 @@ static chunk_t *output_comment_cpp(chunk_t *first)
  */
 static void output_comment_multi(chunk_t *pc)
 {
-   int        cmt_col = pc->column;
+   int        cmt_col;
    const char *cmt_str;
    int        remaining;
    char       ch;
    chunk_t    *prev;
-   char       line[1024];
+   char       *line;
    int        line_len;
    int        line_count = 0;
-   int        ccol;
+   int        ccol; /* the col of subsequent comment lines */
    int        col_diff = 0;
-   int        xtra     = 1;
-   char       lead[5];
+   char       lead[80];
    bool       nl_end = false;
-
    cmt_reflow cmt;
 
-   output_cmt_start(cmt, pc);
-   cmt.cont_text = !cpd.settings[UO_cmt_indent_multi].b ? "" :
-                   cpd.settings[UO_cmt_star_cont].b ? " * " : "   ";
+   //LOG_FMT(LSYS, "%s: line %d\n", __func__, pc->orig_line);
 
    prev = chunk_get_prev(pc);
-   if ((prev != NULL) && (prev->type != CT_NEWLINE))
-   {
-      cmt_col = pc->orig_col;
-   }
-   else
-   {
-      col_diff = pc->orig_col - pc->column;
-   }
+   output_cmt_start(cmt, pc);
+   cmt.reflow = (cpd.settings[UO_cmt_reflow_mode].n != 1);
 
-   xtra = calculate_comment_body_indent(pc->str, pc->len, pc->column);
+   cmt_col = cmt.base_col;
+   col_diff = pc->orig_col - cmt.base_col;
 
-   //LOG_FMT(LSYS, "Indenting1 line %d to col %d (orig=%d) col_diff=%d xtra=%d\n",
-   //        pc->orig_line, cmt_col, pc->orig_col, col_diff, xtra);
+   calculate_comment_body_indent(cmt, pc->str, pc->len);
 
-   ccol          = pc->column;
-   remaining     = pc->len;
-   cmt_str       = pc->str;
-   line_len      = 0;
-   cmt.column    = ccol;
-   cmt.br_column = ccol;
+   cmt.cont_text = !cpd.settings[UO_cmt_indent_multi].b ? "" :
+                   (cpd.settings[UO_cmt_star_cont].b ? "* " : "  ");
+
+   //LOG_FMT(LSYS, "Indenting1 line %d to col %d (orig=%d) col_diff=%d xtra=%d cont='%s'\n",
+   //        pc->orig_line, cmt_col, pc->orig_col, col_diff, cmt.xtra_indent, cmt.cont_text);
+
+   ccol      = pc->column;
+   remaining = pc->len;
+   cmt_str   = pc->str;
+   line_len  = 0;
+   line      = new char[remaining + 1024 + 1]; /* + 1 for '\0' */
    while (remaining > 0)
    {
       ch = *cmt_str;
@@ -1052,6 +1255,107 @@ static void output_comment_multi(chunk_t *pc)
          }
       }
 
+      /*
+       * Now see if we need/must fold the next line with the current to enable
+       * full reflow
+       */
+      if ((cpd.settings[UO_cmt_reflow_mode].n == 2) &&
+          (ch == '\n') &&
+          (remaining > 0))
+      {
+         int  nxt_len            = 0;
+         int  next_nonempty_line = -1;
+         int  prev_nonempty_line = -1;
+         int  nwidx          = line_len;
+         bool star_is_bullet = false;
+
+         /* strip trailing whitespace from the line collected so far */
+         line[nwidx] = 0; // sentinel
+         while (nwidx > 0)
+         {
+            nwidx--;
+            if ((prev_nonempty_line < 0) &&
+                !unc_isspace(line[nwidx]) &&
+                (line[nwidx] != '*') && // block comment: skip '*' at end of line
+                ((pc->flags & PCF_IN_PREPROC)
+                 ? (line[nwidx] != '\\') ||
+                 ((line[nwidx + 1] != 'r') &&
+                  (line[nwidx + 1] != '\n'))
+                 : true))
+            {
+               prev_nonempty_line = nwidx; // last nonwhitespace char in the previous line
+            }
+         }
+
+         for (nxt_len = 0;
+              (nxt_len <= remaining) &&
+              (cmt_str[nxt_len] != 'r') &&
+              (cmt_str[nxt_len] != '\n');
+              nxt_len++)
+         {
+            if ((next_nonempty_line < 0) &&
+                !unc_isspace(cmt_str[nxt_len]) &&
+                (cmt_str[nxt_len] != '*') &&
+                ((nxt_len == remaining) ||
+                 ((pc->flags & PCF_IN_PREPROC)
+                  ? (cmt_str[nxt_len] != '\\') ||
+                  ((cmt_str[nxt_len + 1] != 'r') &&
+                   (cmt_str[nxt_len + 1] != '\n'))
+                  : true)))
+            {
+               next_nonempty_line = nxt_len; // first nonwhitespace char in the next line
+            }
+         }
+
+         /*
+          * see if we should fold up; usually that'd be a YES, but there are a few
+          * situations where folding/reflowing by merging lines is frowned upon:
+          *
+          * - ASCII art in the comments (most often, these are drawings done in +-\/|.,*)
+          *
+          * - Doxygen/JavaDoc/etc. parameters: these often start with \ or @, at least
+          *   something clearly non-alphanumeric (you see where we're going with this?)
+          *
+          * - bullet lists that are closely spaced: bullets are always non-alphanumeric
+          *   characters, such as '-' or '+' (or, oh horor, '*' - that's bloody ambiguous
+          *   to parse :-( ... with or without '*' comment start prefix, that's the
+          *   question, then.)
+          *
+          * - semi-HTML formatted code, e.g. <pre>...</pre> comment sections (NDoc, etc.)
+          *
+          * - New lines which form a new paragraph without there having been added an
+          *   extra empty line between the last sentence and the new one.
+          *   A bit like this, really; so it is opportune to check if the last line ended
+          *   in a terminal (that would be the set '.:;!?') and the new line starts with
+          *   a capital.
+          *   Though new lines starting with comment delimiters, such as '(', should be
+          *   pulled up.
+          *
+          * So it bores down to this: the only folding (& reflowing) that's going to happen
+          * is when the next line starts with an alphanumeric character AND the last
+          * line didn't end with an non-alphanumeric character, except: ',' AND the next
+          * line didn't start with a '*' all of a sudden while the previous one didn't
+          * (the ambiguous '*'-for-bullet case!)
+          */
+         if ((prev_nonempty_line >= 0) && (next_nonempty_line >= 0) &&
+             (((unc_isalnum(line[prev_nonempty_line]) ||
+                strchr(",)]", line[prev_nonempty_line])) &&
+               (unc_isalnum(cmt_str[next_nonempty_line]) ||
+                strchr("([", cmt_str[next_nonempty_line]))) ||
+              (('.' == line[prev_nonempty_line]) &&    // dot followed by non-capital is NOT a new sentence start
+               unc_isupper(cmt_str[next_nonempty_line]))) &&
+             !star_is_bullet)
+         {
+            // rewind the line to the last non-alpha:
+            line_len = prev_nonempty_line + 1;
+            // roll the current line forward to the first non-alpha:
+            cmt_str   += next_nonempty_line;
+            remaining -= next_nonempty_line;
+            // override the NL and make it a single whitespace:
+            ch = ' ';
+         }
+      }
+
       line[line_len++] = ch;
 
       /* If we just hit an end of line OR we just hit end-of-comment... */
@@ -1062,34 +1366,17 @@ static void output_comment_multi(chunk_t *pc)
          /* strip trailing tabs and spaces before the newline */
          if (ch == '\n')
          {
-            line_len--;
-            while ((line_len > 0) &&
-                   ((line[line_len - 1] == ' ') ||
-                    (line[line_len - 1] == '\t')))
-            {
-               line_len--;
-            }
-            if ((line[line_len - 1] == '\\') && (line[line_len - 2] != '*'))
-            {
-               /* Kill off the backslash-newline */
-               line_len--;
-               while ((line_len > 0) &&
-                      ((line[line_len - 1] == ' ') ||
-                       (line[line_len - 1] == '\t')))
-               {
-                  line_len--;
-               }
-               line[line_len++] = ' ';
-               line[line_len++] = '\\';
-            }
             nl_end = true;
+            line_len--;
+            cmt_trim_whitespace(line_len, line, pc->flags & PCF_IN_PREPROC);
          }
          line[line_len] = 0;
 
+         //LOG_FMT(LSYS, "[%3d]%s\n", ccol, line);
+
          if (line_count == 1)
          {
             /* this is the first line - add unchanged */
-            output_indent(cmt_col, cmt_col);
             add_comment_text(line, line_len, cmt, false);
             if (nl_end)
             {
@@ -1099,9 +1386,7 @@ static void output_comment_multi(chunk_t *pc)
          else
          {
             /* This is not the first line, so we need to indent to the
-             * correct column. Each line is indented a minimum of three spaces.
-             * Those three spaces are filled with whatever lead char should be
-             * there.
+             * correct column. Each line is indented 0 or more spaces.
              */
             ccol -= col_diff;
             if (ccol < (cmt_col + 3))
@@ -1111,12 +1396,16 @@ static void output_comment_multi(chunk_t *pc)
 
             if (line_len == 0)
             {
-               /* Emtpy line - just a '\n' */
+               /* Empty line - just a '\n' */
                if (cpd.settings[UO_cmt_star_cont].b)
                {
-                  output_indent(cmt_col + xtra + cpd.settings[UO_cmt_sp_before_star_cont].n,
-                                cmt_col);
-                  add_char('*');
+                  cmt.column = cmt_col + cpd.settings[UO_cmt_sp_before_star_cont].n;
+                  cmt_output_indent(cmt.brace_col, cmt.base_col, cmt.column);
+                  if (cmt.xtra_indent)
+                  {
+                     add_char(' ');
+                  }
+                  add_text(cmt.cont_text);
                }
                add_char('\n');
             }
@@ -1129,37 +1418,48 @@ static void output_comment_multi(chunk_t *pc)
                    (line[0] != '*') && (line[0] != '|') && (line[0] != '#') &&
                    ((line[0] != '\\') || unc_isalpha(line[1])) && (line[0] != '+'))
                {
-                  int start_col = cmt_col + xtra + cpd.settings[UO_cmt_sp_before_star_cont].n;
+                  int start_col = cmt_col + cpd.settings[UO_cmt_sp_before_star_cont].n;
 
                   if (cpd.settings[UO_cmt_star_cont].b)
                   {
-                     output_indent(start_col, cmt_col);
-                     add_text("* ");
+                     cmt.column = start_col;
+                     cmt_output_indent(cmt.brace_col, cmt.base_col, cmt.column);
+                     if (cmt.xtra_indent)
+                     {
+                        add_char(' ');
+                     }
+                     add_text(cmt.cont_text);
                      output_to_column(ccol + cpd.settings[UO_cmt_sp_after_star_cont].n,
                                       false);
                   }
                   else
                   {
-                     output_indent(ccol, cmt_col);
+                     cmt.column = ccol;
+                     cmt_output_indent(cmt.brace_col, cmt.base_col, cmt.column);
                   }
                }
                else
                {
-                  output_indent(cmt_col + xtra + cpd.settings[UO_cmt_sp_before_star_cont].n,
-                                cmt_col);
+                  cmt.column = cmt_col + cpd.settings[UO_cmt_sp_before_star_cont].n;
+                  cmt_output_indent(cmt.brace_col, cmt.base_col, cmt.column);
+                  if (cmt.xtra_indent)
+                  {
+                     add_char(' ');
+                  }
 
-                  int idx  = 0;
-                  int sidx = 0;
-                  if (xtra > 0)
+                  int idx;
+
+                  idx = cmt_parse_lead(line, (remaining == 0));
+                  if (idx > 0)
                   {
-                     lead[idx++] = ' ';
+                     memcpy(lead, line, idx);
+                     lead[idx] = 0;
+                     cmt.cont_text = lead;
                   }
-                  while ((idx < 3) && !unc_isspace(line[sidx]))
+                  else
                   {
-                     lead[idx++] = line[sidx++];
+                     add_text(cmt.cont_text);
                   }
-                  lead[idx]     = 0;
-                  cmt.cont_text = lead;
                }
 
                add_comment_text(line, line_len, cmt, false);
@@ -1173,8 +1473,10 @@ static void output_comment_multi(chunk_t *pc)
          ccol     = 1;
       }
    }
+   delete[] line;
 }
 
+
 /**
  * Output a multiline comment without any reformatting other than shifting
  * it left or right to get the column right.
@@ -1191,7 +1493,10 @@ static void output_comment_multi_simple(chunk_t *pc)
    int        line_count = 0;
    int        ccol;
    int        col_diff = 0;
-   bool       nl_end = false;
+   bool       nl_end   = false;
+   cmt_reflow cmt;
+
+   output_cmt_start(cmt, pc);
 
    if (chunk_is_newline(chunk_get_prev(pc)))
    {
@@ -1206,9 +1511,6 @@ static void output_comment_multi_simple(chunk_t *pc)
       col_diff = 0;
    }
 
-   //LOG_FMT(LSYS, "Indenting1 line %d to col %d (orig=%d) col_diff=%d xtra=%d\n",
-   //        pc->orig_line, cmt_col, pc->orig_col, col_diff, xtra);
-
    ccol      = pc->column;
    remaining = pc->len;
    cmt_str   = pc->str;
@@ -1262,27 +1564,10 @@ static void output_comment_multi_simple(chunk_t *pc)
          if (ch == '\n')
          {
             line_len--;
-            while ((line_len > 0) &&
-                   ((line[line_len - 1] == ' ') ||
-                    (line[line_len - 1] == '\t')))
-            {
-               line_len--;
-            }
-            /* REVISIT: the check for '*' seem unnecessary */
-            if ((line[line_len - 1] == '\\') && (line[line_len - 2] != '*'))
-            {
-               /* Kill off the backslash-newline */
-               line_len--;
-               while ((line_len > 0) &&
-                      ((line[line_len - 1] == ' ') ||
-                       (line[line_len - 1] == '\t')))
-               {
-                  line_len--;
-               }
-               line[line_len++] = ' ';
-               line[line_len++] = '\\';
-            }
             nl_end = true;
+
+            /* Say we aren't in a preproc to prevent changing any bs-nl */
+            cmt_trim_whitespace(line_len, line, false);
          }
          line[line_len] = 0;
 
@@ -1293,7 +1578,8 @@ static void output_comment_multi_simple(chunk_t *pc)
 
          if (line_len > 0)
          {
-            output_indent(ccol, cmt_col);
+            cmt.column = ccol;
+            cmt_output_indent(cmt.brace_col, cmt.base_col, cmt.column);
             add_text_len(line, line_len);
          }
          if (nl_end)
