@@ -4,8 +4,6 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: parens.cpp 1320 2008-08-14 13:36:25Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -32,13 +30,15 @@ void do_parens(void)
       while ((pc = chunk_get_next_ncnl(pc)) != NULL)
       {
          if ((pc->type != CT_SPAREN_OPEN) ||
-             ((pc->parent_type != CT_IF) && (pc->parent_type != CT_SWITCH)))
+             ((pc->parent_type != CT_IF) &&
+              (pc->parent_type != CT_ELSEIF) &&
+              (pc->parent_type != CT_SWITCH)))
          {
             continue;
          }
 
          /* Grab the close sparen */
-         pclose = chunk_get_next_type(pc, CT_SPAREN_CLOSE, pc->level);
+         pclose = chunk_get_next_type(pc, CT_SPAREN_CLOSE, pc->level, CNAV_PREPROC);
          if (pclose != NULL)
          {
             check_bool_parens(pc, pclose, 0);
@@ -48,6 +48,7 @@ void do_parens(void)
    }
 }
 
+
 /**
  * Add an open paren after first and add a close paren before the last
  */
@@ -82,7 +83,7 @@ static void add_parens_between(chunk_t *first, chunk_t *last)
 
    chunk_add_before(&pc, first_n);
 
-   last_p         = chunk_get_prev_ncnl(last);
+   last_p         = chunk_get_prev_ncnl(last, CNAV_PREPROC);
    pc.type        = CT_PAREN_CLOSE;
    pc.str         = ")";
    pc.flags       = last_p->flags & PCF_COPY_FLAGS;
@@ -101,6 +102,7 @@ static void add_parens_between(chunk_t *first, chunk_t *last)
    last_p->level++;
 }
 
+
 /**
  * Scans between two parens and adds additional parens if needed.
  * This function is recursive. If it hits another open paren, it'll call itself
@@ -135,9 +137,12 @@ static void check_bool_parens(chunk_t *popen, chunk_t *pclose, int nest)
    pc = popen;
    while (((pc = chunk_get_next_ncnl(pc)) != NULL) && (pc != pclose))
    {
-      if (pc->type == CT_BOOL)
+      if ((pc->type == CT_BOOL) ||
+          (pc->type == CT_QUESTION) ||
+          (pc->type == CT_COND_COLON))
       {
-         LOG_FMT(LPARADD2, " -- bool [%.*s] at line %d col %d, level %d\n",
+         LOG_FMT(LPARADD2, " -- %s [%.*s] at line %d col %d, level %d\n",
+                 get_token_name(pc->type),
                  pc->len, pc->str, pc->orig_line, pc->orig_col, pc->level);
          if (hit_compare)
          {
