@@ -4,8 +4,6 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: space.cpp 1490 2009-02-27 03:23:28Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -65,10 +63,13 @@ struct no_space_table_s no_space_table[] =
    { CT_PAREN_CLOSE,    CT_FUNC_CALL     },
    { CT_PAREN_CLOSE,    CT_ADDR          },
    { CT_PAREN_CLOSE,    CT_FPAREN_OPEN   },
+   { CT_OC_SEL_NAME,    CT_OC_SEL_NAME   },
 };
 
-#define log_rule(rule) \
-   do { if (log_sev_on(LSPACE)) { log_rule2(__LINE__, (rule), first, second, complete); } } while (0)
+#define log_rule(rule)                                        \
+   do { if (log_sev_on(LSPACE)) {                             \
+      log_rule2(__LINE__, (rule), first, second, complete); } \
+   } while (0)
 
 static void log_rule2(int line, const char *rule, chunk_t *first, chunk_t *second, bool complete)
 {
@@ -82,6 +83,7 @@ static void log_rule2(int line, const char *rule, chunk_t *first, chunk_t *secon
            complete ? "\n" : "");
 }
 
+
 /**
  * Decides how to change inter-chunk spacing.
  * Note that the order of the if statements is VERY important.
@@ -90,12 +92,28 @@ static void log_rule2(int line, const char *rule, chunk_t *first, chunk_t *secon
  * @param second  The second chunk
  * @return        AV_IGNORE, AV_ADD, AV_REMOVE or AV_FORCE
  */
-argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
+argval_t do_space(chunk_t *first, chunk_t *second, bool complete = true)
 {
    int      idx;
    argval_t arg;
    chunk_t  *next;
 
+   if ((first->type == CT_IGNORED) || (second->type == CT_IGNORED))
+   {
+      log_rule("IGNORED");
+      return(AV_REMOVE);
+   }
+   if ((first->type == CT_PP) || (second->type == CT_PP))
+   {
+      log_rule("sp_pp_concat");
+      return(cpd.settings[UO_sp_pp_concat].a);
+   }
+   if (first->type == CT_POUND)
+   {
+      log_rule("sp_pp_stringify");
+      return(cpd.settings[UO_sp_pp_stringify].a);
+   }
+
    if ((first->type == CT_SPACE) || (second->type == CT_SPACE))
    {
       log_rule("REMOVE");
@@ -108,12 +126,12 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       log_rule("REMOVE");
       return(AV_REMOVE);
    }
-   if (first->type == CT_VBRACE_OPEN)
+   if ((first->type == CT_VBRACE_OPEN) && (second->type != CT_NL_CONT))
    {
       log_rule("ADD");
       return(AV_ADD);
    }
-   if (first->type == CT_VBRACE_CLOSE)
+   if ((first->type == CT_VBRACE_CLOSE) && (second->type != CT_NL_CONT))
    {
       log_rule("REMOVE");
       return(AV_REMOVE);
@@ -227,6 +245,31 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(arg);
    }
 
+   /* "for (;;)" vs "for (;; )" and "for (a;b;c)" vs "for (a; b; c)" */
+   if (first->type == CT_SEMICOLON)
+   {
+      if (first->parent_type == CT_FOR)
+      {
+         if ((cpd.settings[UO_sp_after_semi_for_empty].a != AV_IGNORE) &&
+             (second->type == CT_SPAREN_CLOSE))
+         {
+            log_rule("sp_after_semi_for_empty");
+            return(cpd.settings[UO_sp_after_semi_for_empty].a);
+         }
+         if (cpd.settings[UO_sp_after_semi_for].a != AV_IGNORE)
+         {
+            log_rule("sp_after_semi_for");
+            return(cpd.settings[UO_sp_after_semi_for].a);
+         }
+      }
+      else if (!chunk_is_comment(second))
+      {
+         log_rule("sp_after_semi");
+         return(cpd.settings[UO_sp_after_semi].a);
+      }
+      /* Let the comment spacing rules handle this */
+   }
+
    if (((first->type == CT_NEG) || (first->type == CT_POS) || (first->type == CT_ARITH)) &&
        ((second->type == CT_NEG) || (second->type == CT_POS) || (second->type == CT_ARITH)))
    {
@@ -267,7 +310,8 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_after_dc].a);
    }
    if ((second->type == CT_DC_MEMBER) &&
-       ((first->type == CT_WORD) || (first->type == CT_TYPE)))
+       ((first->type == CT_WORD) || (first->type == CT_TYPE) ||
+        CharTable::IsKw1(first->str[0])))
    {
       log_rule("sp_before_dc");
       return(cpd.settings[UO_sp_before_dc].a);
@@ -285,25 +329,31 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_before_comma].a);
    }
 
-   if (first->type == CT_TAG_COLON)
+   if (second->type == CT_ELLIPSIS)
    {
-      if (second->type == CT_ELLIPSIS)
+      /* non-punc followed by a ellipsis */
+      if (((first->flags & PCF_PUNCTUATOR) == 0) &&
+          (cpd.settings[UO_sp_before_ellipsis].a != AV_IGNORE))
+      {
+         log_rule("sp_before_ellipsis");
+         return(cpd.settings[UO_sp_before_ellipsis].a);
+      }
+
+      if (first->type == CT_TAG_COLON)
       {
          log_rule("FORCE");
          return(AV_FORCE);
       }
-      log_rule("sp_after_tag");
-      return(cpd.settings[UO_sp_after_tag].a);
    }
    if ((first->type == CT_ELLIPSIS) && CharTable::IsKw1(second->str[0]))
    {
       log_rule("FORCE");
       return(AV_FORCE);
    }
-   if (second->type == CT_ELLIPSIS)
+   if (first->type == CT_TAG_COLON)
    {
-      log_rule("REMOVE");
-      return(AV_REMOVE);
+      log_rule("sp_after_tag");
+      return(cpd.settings[UO_sp_after_tag].a);
    }
    if (second->type == CT_TAG_COLON)
    {
@@ -340,6 +390,17 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_assign].a);
    }
 
+   if (second->type == CT_OC_BLOCK_CARET)
+   {
+      log_rule("sp_before_oc_block_caret");
+      return(cpd.settings[UO_sp_before_oc_block_caret].a);
+   }
+   if (first->type == CT_OC_BLOCK_CARET)
+   {
+      log_rule("sp_after_oc_block_caret");
+      return(cpd.settings[UO_sp_after_oc_block_caret].a);
+   }
+
    if (second->type == CT_ASSIGN)
    {
       if (second->flags & PCF_IN_ENUM)
@@ -352,6 +413,12 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
          log_rule("sp_enum_assign");
          return(cpd.settings[UO_sp_enum_assign].a);
       }
+      if ((cpd.settings[UO_sp_assign_default].a != AV_IGNORE) &&
+          (second->parent_type == CT_FUNC_PROTO))
+      {
+         log_rule("sp_assign_default");
+         return(cpd.settings[UO_sp_assign_default].a);
+      }
       if (cpd.settings[UO_sp_before_assign].a != AV_IGNORE)
       {
          log_rule("sp_before_assign");
@@ -373,6 +440,12 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
          log_rule("sp_enum_assign");
          return(cpd.settings[UO_sp_enum_assign].a);
       }
+      if ((cpd.settings[UO_sp_assign_default].a != AV_IGNORE) &&
+          (first->parent_type == CT_FUNC_PROTO))
+      {
+         log_rule("sp_assign_default");
+         return(cpd.settings[UO_sp_assign_default].a);
+      }
       if (cpd.settings[UO_sp_after_assign].a != AV_IGNORE)
       {
          log_rule("sp_after_assign");
@@ -383,7 +456,7 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
    }
 
    /* "a [x]" vs "a[x]" */
-   if (second->type == CT_SQUARE_OPEN)
+   if ((second->type == CT_SQUARE_OPEN) && (second->parent_type != CT_OC_MSG))
    {
       log_rule("sp_before_square");
       return(cpd.settings[UO_sp_before_square].a);
@@ -540,6 +613,14 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
        ((second->type == CT_PAREN_OPEN) ||
         (second->type == CT_FPAREN_OPEN)))
    {
+      /* "(int)a" vs "(int) a" or "cast(int)a" vs "cast(int) a" */
+      if ((first->parent_type == CT_C_CAST) ||
+          (first->parent_type == CT_D_CAST))
+      {
+         log_rule("sp_after_cast");
+         return(cpd.settings[UO_sp_after_cast].a);
+      }
+
       /* Must be an indirect/chained function call? */
       log_rule("REMOVE");
       return(AV_REMOVE);  /* TODO: make this configurable? */
@@ -557,7 +638,7 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       log_rule("sp_func_class_paren");
       return(cpd.settings[UO_sp_func_class_paren].a);
    }
-   if (first->type == CT_CLASS)
+   if ((first->type == CT_CLASS) && (first->parent_type != CT_OC_MSG))
    {
       log_rule("FORCE");
       return(AV_FORCE);
@@ -758,12 +839,47 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_inside_fparen].a);
    }
 
-   if ((first->type == CT_PAREN_CLOSE) &&
-       ((first->parent_type == CT_OC_MSG_SPEC) ||
-        (first->parent_type == CT_OC_MSG_DECL)))
+   if (first->type == CT_PAREN_CLOSE)
    {
-      log_rule("sp_after_oc_type");
-      return(cpd.settings[UO_sp_after_oc_type].a);
+      if (first->parent_type == CT_OC_RTYPE)
+      {
+         log_rule("sp_after_oc_return_type");
+         return(cpd.settings[UO_sp_after_oc_return_type].a);
+      }
+      else if ((first->parent_type == CT_OC_MSG_SPEC) ||
+               (first->parent_type == CT_OC_MSG_DECL))
+      {
+         log_rule("sp_after_oc_type");
+         return(cpd.settings[UO_sp_after_oc_type].a);
+      }
+      else if ((first->parent_type == CT_OC_SEL) &&
+               (second->type != CT_SQUARE_CLOSE))
+      {
+         log_rule("sp_after_oc_at_sel_parens");
+         return(cpd.settings[UO_sp_after_oc_at_sel_parens].a);
+      }
+   }
+
+   if (cpd.settings[UO_sp_inside_oc_at_sel_parens].a != AV_IGNORE)
+   {
+      if (((first->type == CT_PAREN_OPEN) &&
+           ((first->parent_type == CT_OC_SEL) ||
+            (first->parent_type == CT_OC_PROTOCOL)))
+          ||
+          ((second->type == CT_PAREN_CLOSE) &&
+           ((second->parent_type == CT_OC_SEL) ||
+            (second->parent_type == CT_OC_PROTOCOL))))
+      {
+         log_rule("sp_inside_oc_at_sel_parens");
+         return(cpd.settings[UO_sp_inside_oc_at_sel_parens].a);
+      }
+   }
+
+   if ((second->type == CT_PAREN_OPEN) &&
+       ((first->type == CT_OC_SEL) || (first->type == CT_OC_PROTOCOL)))
+   {
+      log_rule("sp_after_oc_at_sel");
+      return(cpd.settings[UO_sp_after_oc_at_sel].a);
    }
 
    /* C cast:   "(int)"      vs "( int )"
@@ -808,19 +924,6 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_square_fparen].a);
    }
 
-   /* "for (;;)" vs "for (;; )" */
-   if ((second->type == CT_SPAREN_CLOSE) &&
-       (first->type == CT_SEMICOLON) &&
-       (second->parent_type == CT_FOR))
-   {
-      arg = cpd.settings[UO_sp_after_semi_for_empty].a;
-      if (arg != AV_IGNORE)
-      {
-         log_rule("sp_after_semi_for_empty");
-         return(arg);
-      }
-   }
-
    /* "if(...)" vs "if( ... )" */
    if ((second->type == CT_SPAREN_CLOSE) &&
        (cpd.settings[UO_sp_inside_sparen_close].a != AV_IGNORE))
@@ -991,7 +1094,11 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
          log_rule("sp_brace_finally");
          return(cpd.settings[UO_sp_brace_finally].a);
       }
-      else if (first->parent_type == CT_ENUM)
+   }
+
+   if (first->type == CT_BRACE_OPEN)
+   {
+      if (first->parent_type == CT_ENUM)
       {
          log_rule("sp_inside_braces_enum");
          return(cpd.settings[UO_sp_inside_braces_enum].a);
@@ -1130,7 +1237,9 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
    }
    if (second->type == CT_OC_COLON)
    {
-      if (first->parent_type == CT_OC_MSG)
+      if ((first->parent_type == CT_OC_MSG) &&
+          ((first->type == CT_OC_MSG_FUNC) ||
+           (first->type == CT_OC_MSG_NAME)))
       {
          log_rule("sp_before_send_oc_colon");
          return(cpd.settings[UO_sp_before_send_oc_colon].a);
@@ -1148,6 +1257,17 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(AV_FORCE);
    }
 
+   if ((second->type == CT_COMMENT) &&
+       ((first->type == CT_PP_ELSE) || (first->type == CT_PP_ENDIF)))
+   {
+      if (cpd.settings[UO_sp_endif_cmt].a != AV_IGNORE)
+      {
+         second->type = CT_COMMENT_ENDIF;
+         log_rule("sp_endif_cmt");
+         return(cpd.settings[UO_sp_endif_cmt].a);
+      }
+   }
+
    if (chunk_is_comment(second))
    {
       log_rule("IGNORE");
@@ -1176,6 +1296,7 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
    return(AV_ADD);
 }
 
+
 /**
  * Marches through the whole file and checks to see how many spaces should be
  * between two chunks
@@ -1312,13 +1433,17 @@ void space_text(void)
              chunk_is_newline(chunk_get_next(next)) &&
              (column < (int)next->orig_col))
          {
-            if (cpd.settings[UO_indent_relative_single_line_comments].b)
+            if ((cpd.settings[UO_sp_endif_cmt].a == AV_IGNORE) ||
+                ((pc->type != CT_PP_ELSE) && (pc->type != CT_PP_ENDIF)))
             {
-               column = pc->column + (next->orig_col - pc->orig_col_end);
-            }
-            else
-            {
-               column = next->orig_col;
+               if (cpd.settings[UO_indent_relative_single_line_comments].b)
+               {
+                  column = pc->column + (next->orig_col - pc->orig_col_end);
+               }
+               else
+               {
+                  column = next->orig_col;
+               }
             }
          }
          next->column = column;
@@ -1334,6 +1459,7 @@ void space_text(void)
    }
 }
 
+
 /**
  * Marches through the whole file and adds spaces around nested parens
  */
@@ -1394,6 +1520,28 @@ void space_text_balance_nested_parens(void)
    }
 }
 
+
+/**
+ * Determines if a space is required between two chunks
+ */
+bool space_needed(chunk_t *first, chunk_t *second)
+{
+   switch (do_space(first, second))
+   {
+   case AV_ADD:
+   case AV_FORCE:
+      return(true);
+
+   case AV_REMOVE:
+      return(false);
+
+   case AV_IGNORE:
+   default:
+      return(second->orig_col > (first->orig_col + first->len));
+   }
+}
+
+
 /**
  * Calculates the column difference between two chunks.
  * The rules are bent a bit here, as AV_IGNORE and AV_ADD become AV_FORCE.
@@ -1422,7 +1570,7 @@ int space_col_align(chunk_t *first, chunk_t *second)
       break;
 
    case AV_IGNORE:
-      if (second->orig_col> (first->orig_col + first->len))
+      if (second->orig_col > (first->orig_col + first->len))
       {
          coldiff++;
       }
@@ -1431,6 +1579,7 @@ int space_col_align(chunk_t *first, chunk_t *second)
    return(coldiff);
 }
 
+
 void space_add_after(chunk_t *pc, int count)
 {
    if (count <= 0)
