@@ -6,10 +6,7 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: tokenize.cpp 1475 2009-02-07 21:23:02Z bengardner $
  */
-
 #include "uncrustify_types.h"
 #include "char_table.h"
 #include "prototypes.h"
@@ -23,7 +20,118 @@
 
 static bool parse_string(chunk_t *pc, int quote_idx, bool allow_escape);
 
-#include "d.tokenize.cpp"
+/**
+ * Parses all legal D string constants.
+ *
+ * Quoted strings:
+ *   r"Wysiwyg"      # WYSIWYG string
+ *   x"hexstring"    # Hexadecimal array
+ *   `Wysiwyg`       # WYSIWYG string
+ *   'char'          # single character
+ *   "reg_string"    # regular string
+ *
+ * Non-quoted strings:
+ * \x12              # 1-byte hex constant
+ * \u1234            # 2-byte hex constant
+ * \U12345678        # 4-byte hex constant
+ * \123              # octal constant
+ * \&amp;            # named entity
+ * \n                # single character
+ *
+ * @param pc   The structure to update, str is an input.
+ * @return     Whether a string was parsed
+ */
+static bool d_parse_string(chunk_t *pc)
+{
+   if (pc->str[0] == '"')
+   {
+      return(parse_string(pc, 0, true));
+   }
+   else if ((pc->str[0] == '\'') ||
+            (pc->str[0] == '`'))
+   {
+      return(parse_string(pc, 0, true));
+   }
+   else if (pc->str[0] == '\\')
+   {
+      pc->len = 0;
+      while (pc->str[pc->len] == '\\')
+      {
+         pc->len++;
+         /* Check for end of file */
+         switch (pc->str[pc->len])
+         {
+         case 'x':
+            /* \x HexDigit HexDigit */
+            pc->len += 3;
+            break;
+
+         case 'u':
+            /* \u HexDigit HexDigit HexDigit HexDigit */
+            pc->len += 5;
+            break;
+
+         case 'U':
+            /* \U HexDigit (x8) */
+            pc->len += 9;
+            break;
+
+         case '0':
+         case '1':
+         case '2':
+         case '3':
+         case '4':
+         case '5':
+         case '6':
+         case '7':
+            /* handle up to 3 octal digits */
+            pc->len++;
+            if ((pc->str[pc->len] >= '0') && (pc->str[pc->len] <= '7'))
+            {
+               pc->len++;
+               if ((pc->str[pc->len] >= '0') && (pc->str[pc->len] <= '7'))
+               {
+                  pc->len++;
+               }
+            }
+            break;
+
+         case '&':
+            /* \& NamedCharacterEntity ; */
+            pc->len++;
+            while (isalpha(pc->str[pc->len]))
+            {
+               pc->len++;
+            }
+            if (pc->str[pc->len] == ';')
+            {
+               pc->len++;
+            }
+            break;
+
+         default:
+            /* Everything else is a single character */
+            pc->len++;
+            break;
+         }
+      }
+
+      if (pc->len > 1)
+      {
+         pc->type    = CT_STRING;
+         cpd.column += pc->len;
+         return(true);
+      }
+   }
+   else if (pc->str[1] == '"')
+   {
+      if ((pc->str[0] == 'r') || (pc->str[0] == 'x'))
+      {
+         return(parse_string(pc, 1, false));
+      }
+   }
+   return(false);
+}
 
 /**
  * A string-in-string search.  Like strstr() with a haystack length.
@@ -176,10 +284,17 @@ static bool parse_comment(chunk_t *pc)
             cpd.column += 2;
 
             /* If there is another C comment right after this one, combine them */
-            if ((pc->str[len] != '/') || (pc->str[len + 1] != '*'))
+            int tmp_len = len;
+            while ((pc->str[tmp_len] == ' ') ||
+                   (pc->str[tmp_len] == '\t'))
+            {
+               tmp_len++;
+            }
+            if ((pc->str[tmp_len] != '/') || (pc->str[tmp_len + 1] != '*'))
             {
                break;
             }
+            pc->len = tmp_len;
          }
 
          if ((pc->str[len] == '\n') || (pc->str[len] == '\r'))
@@ -230,6 +345,7 @@ static bool parse_comment(chunk_t *pc)
    return(true);
 }
 
+
 /**
  * Count the number of characters in the number.
  * The next bit of text starts with a number (0-9 or '.'), so it is a number.
@@ -404,6 +520,7 @@ static bool parse_number(chunk_t *pc)
    return(true);
 }
 
+
 /**
  * Count the number of characters in a quoted string.
  * The next bit of text starts with a quote char " or ' or <.
@@ -426,9 +543,18 @@ static bool parse_string(chunk_t *pc, int quote_idx, bool allow_escape)
    len++;
 
    cpd.column += len;
-   for ( /* nada */; pc->str[len] != 0; len++)
+   for (/* nada */; pc->str[len] != 0; len++)
    {
       cpd.column++;
+
+      if ((pc->str[len] == '\n') ||
+          ((pc->str[len] == '\r') && (pc->str[len + 1] != '\n')))
+      {
+         cpd.line_number++;
+         cpd.column = 1;
+         pc->nl_count++;
+         pc->type = CT_STRING_MULTI;
+      }
       if (!escaped)
       {
          if (pc->str[len] == escape_char)
@@ -440,13 +566,6 @@ static bool parse_string(chunk_t *pc, int quote_idx, bool allow_escape)
          {
             escaped = allow_escape;
          }
-         else if (pc->str[len] == '\n')
-         {
-            cpd.line_number++;
-            cpd.column = 1;
-            pc->nl_count++;
-            pc->type = CT_STRING_MULTI;
-         }
          else if (pc->str[len] == end_ch)
          {
             len++;
@@ -472,6 +591,7 @@ static bool parse_string(chunk_t *pc, int quote_idx, bool allow_escape)
    return(true);
 }
 
+
 /**
  * Literal string, ends with single "
  * Two "" don't end the string.
@@ -506,6 +626,7 @@ static bool parse_cs_string(chunk_t *pc)
    return(true);
 }
 
+
 /**
  * Count the number of characters in a word.
  * The first character is already valid for a keyword
@@ -515,7 +636,7 @@ static bool parse_cs_string(chunk_t *pc)
  */
 bool parse_word(chunk_t *pc, bool skipcheck)
 {
-   int len = 1;
+   int               len = 1;
    const chunk_tag_t *tag;
 
    while ((pc->str[len] < 127) && CharTable::IsKw2(pc->str[len]))
@@ -554,6 +675,7 @@ bool parse_word(chunk_t *pc, bool skipcheck)
    return(true);
 }
 
+
 /**
  * Count the number of whitespace characters.
  *
@@ -623,6 +745,32 @@ static bool parse_whitespace(chunk_t *pc)
    return(len != 0);
 }
 
+
+/**
+ * Count the number of non-ascii characters at the start of a file.
+ *
+ * @param pc   The structure to update, str is an input.
+ * @return     Whether a BOM was parsed
+ */
+static bool parse_bom(chunk_t *pc)
+{
+   int len = 0;
+
+   while ((pc->str[len] != 0) &&
+          ((pc->str[len] <= 0) || (pc->str[len] >= 127)))
+   {
+      len++;
+   }
+
+   if (len > 0)
+   {
+      pc->type = CT_SOF;
+      pc->len  = len;
+   }
+   return(len != 0);
+}
+
+
 /**
  * Called when we hit a backslash.
  * If there is nothing but whitespace until the newline, then this is a
@@ -657,6 +805,7 @@ static bool parse_bs_newline(chunk_t *pc)
    return(false);
 }
 
+
 /**
  * Parses any number of tab or space chars followed by a newline.
  * Does not change pc->len if a newline isn't found.
@@ -691,6 +840,7 @@ static bool parse_newline(chunk_t *pc)
    return(false);
 }
 
+
 static bool parse_ignored(chunk_t *pc)
 {
    int nl_count = 0;
@@ -731,6 +881,7 @@ static bool parse_ignored(chunk_t *pc)
    return(false);
 }
 
+
 /**
  * Skips the next bit of whatever and returns the type of block.
  *
@@ -770,6 +921,14 @@ static bool parse_next(chunk_t *pc)
       }
    }
 
+   if (chunk_get_head() == NULL)
+   {
+      if (parse_bom(pc))
+      {
+         return(true);
+      }
+   }
+
    /**
     *  Parse whitespace
     */
@@ -901,12 +1060,13 @@ static bool parse_next(chunk_t *pc)
       /* Not D stuff */
 
       /* Check for L'a', L"abc", 'a', "abc", <abc> strings */
-      if (((*pc->str == 'L') && ((pc->str[1] == '"') || (pc->str[1] == '\''))) ||
+      if ((((*pc->str == 'L') || (*pc->str == 'S')) &&
+           ((pc->str[1] == '"') || (pc->str[1] == '\''))) ||
           (*pc->str == '"') ||
           (*pc->str == '\'') ||
           ((*pc->str == '<') && (cpd.in_preproc == CT_PP_INCLUDE)))
       {
-         parse_string(pc, (*pc->str == 'L') ? 1 : 0, true);
+         parse_string(pc, unc_isalpha(*pc->str) ? 1 : 0, true);
          return(true);
       }
 
@@ -954,6 +1114,7 @@ static bool parse_next(chunk_t *pc)
    return(true);
 }
 
+
 /**
  * This function parses or tokenizes the whole buffer into a list.
  * It has to do some tricks to parse preprocessors.
@@ -1002,6 +1163,12 @@ void tokenize(const char *data, int data_len, chunk_t *ref)
          continue;
       }
 
+      if (chunk.type == CT_SOF)
+      {
+         cpd.bom = chunk_dup(&chunk);
+         continue;
+      }
+
       if (chunk.type == CT_NEWLINE)
       {
          last_was_tab    = chunk.after_tab;
@@ -1126,6 +1293,7 @@ void tokenize(const char *data, int data_len, chunk_t *ref)
    }
 }
 
+
 /**
  * A simplistic fixed-sized needle in the fixed-size haystack string search.
  */
