@@ -7,10 +7,7 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: tokenize_cleanup.cpp 1480 2009-02-21 03:07:51Z bengardner $
  */
-
 #include "uncrustify_types.h"
 #include "prototypes.h"
 #include "chunk_list.h"
@@ -42,7 +39,7 @@ void tokenize_cleanup(void)
          pc->len  = 2;
          chunk_del(next);
          pc->orig_col_end += 1;
-         next = chunk_get_next_ncnl(pc);
+         next              = chunk_get_next_ncnl(pc);
       }
 
       if ((pc->type == CT_DOT) && ((cpd.lang_flags & LANG_ALLC) != 0))
@@ -122,7 +119,7 @@ void tokenize_cleanup(void)
       }
 
       /**
-       * Change CT_STAR to CT_PTR_TYPE if preceeded by CT_TYPE,
+       * Change CT_STAR to CT_PTR_TYPE if preceded by CT_TYPE,
        * CT_QUALIFIER, or CT_PTR_TYPE.
        */
       if ((next->type == CT_STAR) &&
@@ -145,14 +142,7 @@ void tokenize_cleanup(void)
        */
       if ((pc->type == CT_ANGLE_OPEN) && (pc->parent_type != CT_TYPE_CAST))
       {
-         if (pc->flags & PCF_IN_PREPROC)
-         {
-            pc->type = CT_COMPARE;
-         }
-         else
-         {
-            check_template(pc);
-         }
+         check_template(pc);
       }
       if ((pc->type == CT_ANGLE_CLOSE) && (pc->parent_type != CT_TEMPLATE))
       {
@@ -213,76 +203,87 @@ void tokenize_cleanup(void)
          }
       }
 
-      /* Change item after operator (>=, ==, etc) to a CT_FUNCTION */
+      /* Change item after operator (>=, ==, etc) to a CT_OPERATOR_VAL
+       * Usually the next item is part of the operator.
+       * In a few cases the next few tokens are part of it:
+       *  operator +       - common case
+       *  operator ()
+       *  operator []      - already converted to TSQUARE
+       *  operator new []
+       *  operator delete []
+       *  operator const char *
+       *  operator std::allocator<U>
+       *
+       * In all cases except the last, this will put the entire operator value
+       * in one chunk.
+       */
       if (pc->type == CT_OPERATOR)
       {
-         LOG_FMT(LOPERATOR, "%s: %d:%d operator", __func__, pc->orig_line, pc->orig_col);
-
          /* Handle special case of () operator -- [] already handled */
          if (next->type == CT_PAREN_OPEN)
          {
             tmp = chunk_get_next(next);
             if ((tmp != NULL) && (tmp->type == CT_PAREN_CLOSE))
             {
-               next->str         = "()";
-               next->len         = 2;
-               next->type        = CT_FUNCTION;
-               next->parent_type = CT_OPERATOR;
+               next->str  = "()";
+               next->len  = 2;
+               next->type = CT_OPERATOR_VAL;
                chunk_del(tmp);
                next->orig_col_end += 1;
-               LOG_FMT(LOPERATOR, " ()");
             }
          }
+         else if (next->flags & PCF_PUNCTUATOR)
+         {
+            next->type = CT_OPERATOR_VAL;
+         }
          else
          {
-            /* Scan until we hit a '(' or ';' */
-            tmp = next;
-            while ((tmp != NULL) &&
-                   (tmp->type != CT_PAREN_OPEN) &&
-                   (tmp->type != CT_SEMICOLON))
-            {
-               LOG_FMT(LOPERATOR, " %.*s", tmp->len, tmp->str);
-               tmp = chunk_get_next_ncnl(tmp);
-            }
+            next->type = CT_TYPE;
 
-            if ((tmp != NULL) && (tmp->type == CT_PAREN_OPEN))
+            /* Replace next with a collection of all tokens that are part of
+             * the type.
+             */
+            char opbuf[256];
+            int  len;
+
+            len = snprintf(opbuf, sizeof(opbuf), "%.*s",
+                           next->len, next->str);
+
+            tmp2 = next;
+            while ((tmp = chunk_get_next(tmp2)) != NULL)
             {
-               /* Mark chunks between 'operator' and '('.
-                * If 'next' is a WORD, then the last 'type' present
-                * is the function name. Otherwise, the item after the
-                * 'operator' is the function name.
-                */
-               tmp2 = next;
-               if ((next->flags & PCF_PUNCTUATOR) == 0)
+               if ((tmp->type != CT_WORD) &&
+                   (tmp->type != CT_TYPE) &&
+                   (tmp->type != CT_QUALIFIER) &&
+                   (tmp->type != CT_STAR) &&
+                   (tmp->type != CT_AMP) &&
+                   (tmp->type != CT_TSQUARE))
                {
-                  tmp = chunk_get_next_ncnl(next);
-                  while ((tmp != NULL) && (tmp->type != CT_PAREN_OPEN))
-                  {
-                     tmp->parent_type = CT_OPERATOR;
-                     make_type(tmp);
-                     if (tmp->type == CT_TYPE)
-                     {
-                        tmp2 = tmp;
-                     }
-                     tmp = chunk_get_next_ncnl(tmp);
-                  }
-                  if (tmp2->type != CT_TYPE)
-                  {
-                     tmp2 = next;
-                  }
+                  break;
                }
-
-               LOG_FMT(LOPERATOR, " [%.*s]", tmp2->len, tmp2->str);
-
-               tmp2->type        = CT_FUNCTION;
-               tmp2->parent_type = CT_OPERATOR;
+               len += snprintf(opbuf + len, sizeof(opbuf) - len, "%s%.*s",
+                               space_needed(tmp2, tmp) ? " " : "",
+                               tmp->len, tmp->str);
+               tmp2 = tmp;
             }
-            LOG_FMT(LOPERATOR, "\n");
-         }
-         if (chunk_is_addr(prev))
-         {
-            prev->type = CT_BYREF;
+
+            while ((tmp2 = chunk_get_next(next)) != tmp)
+            {
+               chunk_del(tmp2);
+            }
+
+            next->str    = strdup(opbuf);
+            next->len    = len;
+            next->flags |= PCF_OWN_STR;
+            next->type   = CT_OPERATOR_VAL;
+
+            next->orig_col_end = next->orig_col + next->len;
          }
+         next->parent_type = CT_OPERATOR;
+
+         LOG_FMT(LOPERATOR, "%s: %d:%d operator '%.*s'\n",
+                 __func__, pc->orig_line, pc->orig_col,
+                 next->len, next->str);
       }
 
       /* Change private, public, protected into either a qualifier or label */
@@ -348,9 +349,14 @@ void tokenize_cleanup(void)
       }
 
       /* Detect Objective C class name */
-      if ((pc->type == CT_OC_IMPL) || (pc->type == CT_OC_INTF))
+      if ((pc->type == CT_OC_IMPL) ||
+          (pc->type == CT_OC_INTF) ||
+          (pc->type == CT_OC_PROTOCOL))
       {
-         next->type        = CT_OC_CLASS;
+         if (next->type != CT_PAREN_OPEN)
+         {
+            next->type = CT_OC_CLASS;
+         }
          next->parent_type = pc->type;
 
          tmp = chunk_get_next_ncnl(next);
@@ -366,6 +372,40 @@ void tokenize_cleanup(void)
          }
       }
 
+      /* Detect Objective-C categories and class extensions */
+      /* @interface ClassName (CategoryName) */
+      /* @implementation ClassName (CategoryName) */
+      /* @interface ClassName () */
+      /* @implementation ClassName () */
+      if (((pc->parent_type == CT_OC_IMPL) ||
+           (pc->parent_type == CT_OC_INTF) ||
+           (pc->type == CT_OC_CLASS)) &&
+          (next->type == CT_PAREN_OPEN))
+      {
+         next->parent_type = pc->parent_type;
+
+         tmp = chunk_get_next(next);
+         if ((tmp != NULL) && (tmp->next != NULL))
+         {
+            if (tmp->type == CT_PAREN_CLOSE)
+            {
+               //tmp->type        = CT_OC_CLASS_EXT;
+               tmp->parent_type = pc->parent_type;
+            }
+            else
+            {
+               tmp->type        = CT_OC_CATEGORY;
+               tmp->parent_type = pc->parent_type;
+            }
+         }
+
+         tmp = chunk_get_next_type(pc, CT_PAREN_CLOSE, pc->level);
+         if (tmp != NULL)
+         {
+            tmp->parent_type = pc->parent_type;
+         }
+      }
+
       /**
        * Objective C @dynamic and @synthesize
        *  @dynamic xxx, yyy;
@@ -414,6 +454,95 @@ void tokenize_cleanup(void)
          }
       }
 
+      /* Detect Objective C @selector
+       *  @selector(msgNameWithNoArg)
+       *  @selector(msgNameWith1Arg:)
+       *  @selector(msgNameWith2Args:arg2Name:)
+       */
+      if ((pc->type == CT_OC_SEL) && (next->type == CT_PAREN_OPEN))
+      {
+         next->parent_type = pc->type;
+
+         tmp = chunk_get_next(next);
+         if (tmp != NULL)
+         {
+            tmp->type        = CT_OC_SEL_NAME;
+            tmp->parent_type = pc->type;
+
+            while ((tmp = chunk_get_next_ncnl(tmp)) != NULL)
+            {
+               if (tmp->type == CT_PAREN_CLOSE)
+               {
+                  tmp->parent_type = CT_OC_SEL;
+                  break;
+               }
+               tmp->type        = CT_OC_SEL_NAME;
+               tmp->parent_type = pc->type;
+            }
+         }
+      }
+
+      /* Mark Objective-C blocks (aka lambdas or closures)
+       *  The syntax and usage is exactly like C function pointers with two exceptions:
+       *  Instead of an asterisk they have a caret as pointer symbol.
+       *  In method declarations which take a block as parameter, there can be anonymous blocks, e.g.: (^)
+       *  1. block literal: ^{ ... };
+       *  2. block declaration: return_t (^name) (int arg1, int arg2, ...) NB: return_t is optional and name can be optional if found as param in a method declaration.
+       *  3. block expression: ^ return_t (int arg) { ... }; NB: return_t is optional
+       *
+       *  See http://developer.apple.com/mac/library/documentation/Cocoa/Conceptual/Blocks for more info...
+       */
+      if ((cpd.lang_flags & LANG_OC) &&
+          (pc->type == CT_ARITH) &&
+          (chunk_is_str(pc, "^", 1)) &&
+          (prev->type != CT_NUMBER) &&
+          (prev->type != CT_NUMBER_FP))
+      {
+         /* mark objc blocks caret so that we can process it later*/
+         pc->type = CT_OC_BLOCK_CARET;
+
+         if (prev->type == CT_PAREN_OPEN)
+         {
+            /* block declaration */
+            pc->parent_type = CT_OC_BLOCK_TYPE;
+         }
+         else if ((next->type == CT_PAREN_OPEN) ||
+                  (next->type == CT_BRACE_OPEN))
+         {
+            /* block expression without return type */
+            /* block literal */
+            pc->parent_type = CT_OC_BLOCK_EXPR;
+         }
+         else
+         {
+            /* block expression with return type (seldomly used) */
+            if (prev->type == CT_ASSIGN)
+            {
+               /* shortcut to spare the peeking below
+                * the XOR operator wouldn't be used directly
+                * after an assign all by itself */
+               pc->parent_type = CT_OC_BLOCK_EXPR;
+            }
+            else
+            {
+               /* this ones tricky because we don't know how many
+                * stars the return type has - if there even is one */
+               tmp = pc;
+               while ((tmp = chunk_get_next(tmp)) != NULL)
+               {
+                  /* we just peek ahead and see if the line contains
+                   * an open brace somewhere.
+                   * FIXME: this check needs to be more thorough. */
+                  if (tmp->type == CT_BRACE_OPEN)
+                  {
+                     pc->parent_type = CT_OC_BLOCK_EXPR;
+                     break;
+                  }
+               }
+            }
+         }
+      }
+
       /* Handle special preprocessor junk */
       if (pc->type == CT_PREPROC)
       {
@@ -474,7 +603,7 @@ void tokenize_cleanup(void)
       {
          pc->len++;
          pc->type = CT_ARITH;
-         tmp = chunk_get_next_ncnl(next);
+         tmp      = chunk_get_next_ncnl(next);
          chunk_del(next);
          next = tmp;
       }
@@ -492,6 +621,11 @@ void tokenize_cleanup(void)
          pc->type = CT_QUALIFIER;
       }
 
+      if ((pc->type == CT_USING) && (next->type == CT_PAREN_OPEN))
+      {
+         pc->type = CT_USING_STMT;
+      }
+
       /* TODO: determine other stuff here */
 
       prev = pc;
@@ -500,6 +634,7 @@ void tokenize_cleanup(void)
    }
 }
 
+
 /**
  * If there is nothing but CT_WORD and CT_MEMBER, then it's probably a
  * template thingy.  Otherwise, it's likely a comparison.
@@ -514,7 +649,7 @@ static void check_template(chunk_t *start)
 
    LOG_FMT(LTEMPL, "%s: Line %d, col %d:", __func__, start->orig_line, start->orig_col);
 
-   prev = chunk_get_prev_ncnl(start);
+   prev = chunk_get_prev_ncnl(start, CNAV_PREPROC);
    if (prev == NULL)
    {
       return;
@@ -526,7 +661,9 @@ static void check_template(chunk_t *start)
 
       /* We have: "template< ... >", which is a template declaration */
       int level = 1;
-      for (pc = chunk_get_next_ncnl(start); pc != NULL; pc = chunk_get_next_ncnl(pc))
+      for (pc = chunk_get_next_ncnl(start, CNAV_PREPROC);
+           pc != NULL;
+           pc = chunk_get_next_ncnl(pc, CNAV_PREPROC))
       {
          LOG_FMT(LTEMPL, " [%s,%d]", get_token_name(pc->type), level);
 
@@ -555,7 +692,10 @@ static void check_template(chunk_t *start)
        */
 
       /* A template requires a word/type right before the open angle */
-      if ((prev->type != CT_WORD) && (prev->type != CT_TYPE) && (prev->parent_type != CT_OPERATOR))
+      if ((prev->type != CT_WORD) &&
+          (prev->type != CT_TYPE) &&
+          (prev->type != CT_OPERATOR_VAL) &&
+          (prev->parent_type != CT_OPERATOR))
       {
          LOG_FMT(LTEMPL, " - after %s + ( - Not a template\n", get_token_name(prev->type));
          start->type = CT_COMPARE;
@@ -566,7 +706,7 @@ static void check_template(chunk_t *start)
 
       /* Scan back and make sure we aren't inside square parens */
       pc = start;
-      while ((pc = chunk_get_prev_ncnl(pc)) != NULL)
+      while ((pc = chunk_get_prev_ncnl(pc, CNAV_PREPROC)) != NULL)
       {
          if ((pc->type == CT_SEMICOLON) ||
              (pc->type == CT_BRACE_OPEN) ||
@@ -596,7 +736,9 @@ static void check_template(chunk_t *start)
       int       num_tokens = 1;
 
       tokens[0] = CT_ANGLE_OPEN;
-      for (pc = chunk_get_next_ncnl(start); pc != NULL; pc = chunk_get_next_ncnl(pc))
+      for (pc = chunk_get_next_ncnl(start, CNAV_PREPROC);
+           pc != NULL;
+           pc = chunk_get_next_ncnl(pc, CNAV_PREPROC))
       {
          LOG_FMT(LTEMPL, " [%s,%d]", get_token_name(pc->type), num_tokens);
 
@@ -651,9 +793,8 @@ static void check_template(chunk_t *start)
 
    if ((end != NULL) && (end->type == CT_ANGLE_CLOSE))
    {
-      pc = chunk_get_next_ncnl(end);
-      if ((pc != NULL) &&
-          (pc->type != CT_NUMBER))
+      pc = chunk_get_next_ncnl(end, CNAV_PREPROC);
+      if ((pc != NULL) && (pc->type != CT_NUMBER))
       {
          LOG_FMT(LTEMPL, " - Template Detected\n");
 
@@ -662,7 +803,7 @@ static void check_template(chunk_t *start)
          pc = start;
          while (pc != end)
          {
-            next = chunk_get_next_ncnl(pc);
+            next       = chunk_get_next_ncnl(pc, CNAV_PREPROC);
             pc->flags |= PCF_IN_TEMPLATE;
             if (next->type != CT_PAREN_OPEN)
             {
