@@ -4,8 +4,6 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: uncrustify.cpp 1495 2009-02-28 17:37:10Z bengardner $
  */
 #define DEFINE_PCF_NAMES
 #define DEFINE_CHAR_TABLE
@@ -26,12 +24,13 @@
 #include <cstdlib>
 #include <cstring>
 #include <cerrno>
+#include <fcntl.h>
 #include "unc_ctype.h"
 #ifdef HAVE_SYS_STAT_H
- #include <sys/stat.h>
+#include <sys/stat.h>
 #endif
 #ifdef HAVE_STRINGS_H
- #include <strings.h> /* strcasecmp() */
+#include <strings.h>  /* strcasecmp() */
 #endif
 
 /* Global data */
@@ -89,7 +88,8 @@ const char *path_basename(const char *path)
    while ((ch = *path) != 0)
    {
       path++;
-      if (ch == PATH_SEP)
+      /* Check both slash types to support windows */
+      if ((ch == '/') || (ch == '\\'))
       {
          last_path = path;
       }
@@ -97,6 +97,7 @@ const char *path_basename(const char *path)
    return(last_path);
 }
 
+
 /**
  * Returns the length of the directory part of the filename.
  */
@@ -106,9 +107,10 @@ int path_dirname_len(const char *filename)
    {
       return(0);
    }
-   return(path_basename(filename) - filename);
+   return((int)(path_basename(filename) - filename));
 }
 
+
 static void usage_exit(const char *msg, const char *argv0, int code)
 {
    if (msg != NULL)
@@ -192,16 +194,19 @@ static void usage_exit(const char *msg, const char *argv0, int code)
    exit(code);
 }
 
+
 static void version_exit(void)
 {
    printf("uncrustify %s\n", UNCRUSTIFY_VERSION);
    exit(0);
 }
 
+
 static void redir_stdout(const char *output_file)
 {
    /* Reopen stdout */
    FILE *my_stdout = stdout;
+
    if (output_file != NULL)
    {
       my_stdout = freopen(output_file, "wb", stdout);
@@ -215,6 +220,7 @@ static void redir_stdout(const char *output_file)
    }
 }
 
+
 int main(int argc, char *argv[])
 {
    char       *data        = NULL;
@@ -228,7 +234,7 @@ int main(int argc, char *argv[])
    int        idx;
    const char *p_arg;
 
-   /* If ran without options... check keywork sort and show the usage info */
+   /* If ran without options... check keyword sort and show the usage info */
    if (argc == 1)
    {
       keywords_are_sorted();
@@ -258,7 +264,7 @@ int main(int argc, char *argv[])
 
 #ifdef WIN32
    /* tell windoze not to change what I write to stdout */
-   _setmode(_fileno(stdout), _O_BINARY);
+   (void)_setmode(_fileno(stdout), _O_BINARY);
 #endif
 
    /* Init logging */
@@ -520,13 +526,13 @@ int main(int argc, char *argv[])
    {
       if (source_file != NULL)
       {
-         usage_exit("Cannot specify both the single file option and a mulit-file option.",
+         usage_exit("Cannot specify both the single file option and a multi-file option.",
                     argv[0], 67);
       }
 
       if (output_file != NULL)
       {
-         usage_exit("Cannot specify -o with a mulit-file option.",
+         usage_exit("Cannot specify -o with a multi-file option.",
                     argv[0], 68);
       }
    }
@@ -565,7 +571,7 @@ int main(int argc, char *argv[])
    else if (source_file != NULL)
    {
       /* Doing a single file */
-      do_source_file(source_file, output_file, parsed_file, no_backup, false);
+      do_source_file(source_file, output_file, parsed_file, no_backup, keep_mtime);
    }
    else
    {
@@ -601,6 +607,7 @@ int main(int argc, char *argv[])
    return((cpd.error_count != 0) ? 1 : 0);
 }
 
+
 static void process_source_list(const char *source_list,
                                 const char *prefix, const char *suffix,
                                 bool no_backup, bool keep_mtime)
@@ -641,8 +648,10 @@ static void process_source_list(const char *source_list,
                         NULL, no_backup, keep_mtime);
       }
    }
+   fclose(p_file);
 }
 
+
 static char *read_stdin(int& out_len)
 {
    char *data;
@@ -686,10 +695,11 @@ static char *read_stdin(int& out_len)
    return(data);
 }
 
+
 static void make_folders(const char *filename)
 {
-   int idx;
-   int last_idx = 0;
+   int  idx;
+   int  last_idx = 0;
    char outname[1024];
 
    snprintf(outname, sizeof(outname), "%s", filename);
@@ -721,6 +731,7 @@ static void make_folders(const char *filename)
    }
 }
 
+
 /**
  * Loads a file into memory
  */
@@ -767,7 +778,7 @@ static int load_mem_file(const char *filename, file_mem& fm)
    else
    {
       fm.data[fm.length] = 0;
-      retval = 0;
+      retval             = 0;
    }
    fclose(p_file);
 
@@ -780,6 +791,7 @@ static int load_mem_file(const char *filename, file_mem& fm)
    return(retval);
 }
 
+
 /**
  * Try to load the file from the config folder first and then by name
  */
@@ -804,6 +816,7 @@ static int load_mem_file_config(const char *filename, file_mem& fm)
    return(retval);
 }
 
+
 static int load_header_files()
 {
    int retval = 0;
@@ -835,6 +848,7 @@ static int load_header_files()
    return(retval);
 }
 
+
 static const char *make_output_filename(char *buf, int buf_size,
                                         const char *filename,
                                         const char *prefix,
@@ -854,6 +868,79 @@ static const char *make_output_filename(char *buf, int buf_size,
 }
 
 
+/**
+ * Reinvent the wheel with a file comparision function...
+ */
+static bool file_content_matches(const char *filename1, const char *filename2)
+{
+   struct stat st1, st2;
+   int         fd1, fd2;
+   UINT8       buf1[1024], buf2[1024];
+   int         len1 = 0, len2 = 0;
+   int         minlen;
+
+   /* Check the sizes first */
+   if ((stat(filename1, &st1) != 0) ||
+       (stat(filename2, &st2) != 0) ||
+       (st1.st_size != st2.st_size))
+   {
+      return(false);
+   }
+
+   if ((fd1 = open(filename1, O_RDONLY)) < 0)
+   {
+      return(false);
+   }
+   if ((fd2 = open(filename2, O_RDONLY)) < 0)
+   {
+      close(fd1);
+      return(false);
+   }
+
+   while ((len1 >= 0) && (len2 >= 0))
+   {
+      if (len1 == 0)
+      {
+         len1 = read(fd1, buf1, sizeof(buf1));
+      }
+      if (len2 == 0)
+      {
+         len2 = read(fd2, buf2, sizeof(buf2));
+      }
+      if ((len1 <= 0) || (len2 <= 0))
+      {
+         break;
+      }
+      minlen = (len1 < len2) ? len1 : len2;
+      if (memcmp(buf1, buf2, minlen) != 0)
+      {
+         break;
+      }
+      len1 -= minlen;
+      len2 -= minlen;
+   }
+
+   close(fd1);
+   close(fd2);
+
+   return((len1 == 0) && (len2 == 0));
+}
+
+
+const char *fix_filename(const char *filename)
+{
+   char *tmp_file;
+
+   /* Create 'outfile.uncrustify' */
+   tmp_file = new char[strlen(filename) + 16 + 1]; /* + 1 for '\0' */
+   if (tmp_file != NULL)
+   {
+      sprintf(tmp_file, "%s.uncrustify", filename);
+   }
+   return(tmp_file);
+}
+
+
 /**
  * Does a source file.
  *
@@ -866,13 +953,14 @@ static const char *make_output_filename(char *buf, int buf_size,
 static void do_source_file(const char *filename_in,
                            const char *filename_out,
                            const char *parsed_file,
-                           bool no_backup,
-                           bool keep_mtime)
+                           bool       no_backup,
+                           bool       keep_mtime)
 {
-   FILE     *pfout;
-   bool     did_open    = false;
-   bool     need_backup = false;
-   file_mem fm;
+   FILE       *pfout;
+   bool       did_open    = false;
+   bool       need_backup = false;
+   file_mem   fm;
+   const char *filename_tmp = NULL;
 
    /* Do some simple language detection based on the filename extension */
    if (cpd.lang_flags == 0)
@@ -897,27 +985,47 @@ static void do_source_file(const char *filename_in,
    }
    else
    {
-      if (!no_backup && (strcmp(filename_in, filename_out) == 0))
+      /* If the out file is the same as the in file, then use a temp file */
+      filename_tmp = filename_out;
+      if (strcmp(filename_in, filename_out) == 0)
       {
-         if (backup_copy_file(filename_in, fm.data, fm.length) != SUCCESS)
+         /* Create 'outfile.uncrustify' */
+         filename_tmp = fix_filename(filename_out);
+         if (filename_tmp == NULL)
          {
-            LOG_FMT(LERR, "%s: Failed to create backup file for %s\n",
-                    __func__, filename_in);
-            free(fm.data);
+            LOG_FMT(LERR, "%s: Out of memory\n", __func__);
             cpd.error_count++;
+            free(fm.data);
             return;
          }
-         need_backup = true;
-      }
-      make_folders(filename_out);
 
-      pfout = fopen(filename_out, "wb");
+         if (!no_backup)
+         {
+            if (backup_copy_file(filename_in, fm.data, fm.length) != SUCCESS)
+            {
+               LOG_FMT(LERR, "%s: Failed to create backup file for %s\n",
+                       __func__, filename_in);
+               free(fm.data);
+               cpd.error_count++;
+               return;
+            }
+            need_backup = true;
+         }
+      }
+      make_folders(filename_tmp);
+
+      pfout = fopen(filename_tmp, "wb");
       if (pfout == NULL)
       {
          LOG_FMT(LERR, "%s: Unable to create %s: %s (%d)\n",
-                 __func__, filename_out, strerror(errno), errno);
+                 __func__, filename_tmp, strerror(errno), errno);
          cpd.error_count++;
          free(fm.data);
+         if ((filename_tmp != NULL) && (filename_tmp != filename_out))
+         {
+            delete [] (char *)filename_tmp;
+            filename_tmp = NULL;
+         }
          return;
       }
       did_open = true;
@@ -938,6 +1046,35 @@ static void do_source_file(const char *filename_in,
          backup_create_md5_file(filename_in);
       }
 
+      if ((filename_tmp != NULL) && (filename_tmp != filename_out))
+      {
+         /* We need to compare and then do a rename */
+         if (file_content_matches(filename_tmp, filename_out))
+         {
+            /* No change - remove tmp file */
+            (void)unlink(filename_tmp);
+         }
+         else
+         {
+#ifdef WIN32
+            /* windows can't rename a file if the target exists, so delete it
+             * first. This may cause data loss if the tmp file gets deleted
+             * or can't be renamed.
+             */
+            (void)unlink(filename_out);
+#endif
+            /* Change - rename filename_tmp to filename_out */
+            if (rename(filename_tmp, filename_out) != 0)
+            {
+               LOG_FMT(LERR, "%s: Unable to rename '%s' to '%s'\n",
+                       __func__, filename_tmp, filename_out);
+               cpd.error_count++;
+            }
+         }
+         delete [] (char *)filename_tmp;
+         filename_tmp = NULL;
+      }
+
 #ifdef HAVE_UTIME_H
       if (keep_mtime)
       {
@@ -949,6 +1086,7 @@ static void do_source_file(const char *filename_in,
    }
 }
 
+
 static void add_file_header()
 {
    if (!chunk_is_comment(chunk_get_head()))
@@ -958,6 +1096,7 @@ static void add_file_header()
    }
 }
 
+
 static void add_file_footer()
 {
    chunk_t *pc = chunk_get_tail();
@@ -980,6 +1119,7 @@ static void add_file_footer()
    }
 }
 
+
 static void add_func_header(c_token_t type, file_mem& fm)
 {
    chunk_t *pc;
@@ -995,19 +1135,27 @@ static void add_func_header(c_token_t type, file_mem& fm)
       }
 
       do_insert = false;
+
       /* On a function proto or def. Back up to a close brace or semicolon on
        * the same level
        */
       ref = pc;
       while ((ref = chunk_get_prev(ref)) != NULL)
       {
-         /* Bail if we change level */
-         if (ref->level != pc->level)
+         /* Bail if we change level or find an access specifier colon */
+         if ((ref->level != pc->level) || (ref->type == CT_PRIVATE_COLON))
          {
             do_insert = true;
             break;
          }
 
+         /* If we hit an angle close, back up to the angle open */
+         if (ref->type == CT_ANGLE_CLOSE)
+         {
+            ref = chunk_get_prev_type(ref, CT_ANGLE_OPEN, ref->level, CNAV_PREPROC);
+            continue;
+         }
+
          /* Bail if we hit a preprocessor and cmt_insert_before_preproc is false */
          if (ref->flags & PCF_IN_PREPROC)
          {
@@ -1051,6 +1199,7 @@ static void add_func_header(c_token_t type, file_mem& fm)
    }
 }
 
+
 static void uncrustify_start(const char *data, int data_len)
 {
    /**
@@ -1064,7 +1213,7 @@ static void uncrustify_start(const char *data, int data_len)
       add_file_header();
    }
 
-   /* Add the file header */
+   /* Add the file footer */
    if (cpd.file_ftr.data != NULL)
    {
       add_file_footer();
@@ -1098,12 +1247,15 @@ static void uncrustify_start(const char *data, int data_len)
     */
    fix_symbols();
 
+   mark_comments();
+
    /**
     * Look at all colons ':' and mark labels, :? sequences, etc.
     */
    combine_labels();
 }
 
+
 static void uncrustify_file(const char *data, int data_len, FILE *pfout,
                             const char *parsed_file)
 {
@@ -1127,6 +1279,11 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
          add_func_header(CT_CLASS, cpd.class_hdr);
       }
 
+      /**
+       * Change virtual braces into real braces...
+       */
+      do_braces();
+
       /* Scrub extra semicolons */
       if (cpd.settings[UO_mod_remove_extra_semicolon].b)
       {
@@ -1139,11 +1296,6 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
          remove_extra_returns();
       }
 
-      /**
-       * Change virtual braces into real braces...
-       */
-      do_braces();
-
       /**
        * Add parens
        */
@@ -1163,6 +1315,15 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
       {
          newlines_chunk_pos(CT_BOOL, cpd.settings[UO_pos_bool].tp);
       }
+      if (cpd.settings[UO_pos_compare].tp != TP_IGNORE)
+      {
+         newlines_chunk_pos(CT_COMPARE, cpd.settings[UO_pos_compare].tp);
+      }
+      if (cpd.settings[UO_pos_conditional].tp != TP_IGNORE)
+      {
+         newlines_chunk_pos(CT_COND_COLON, cpd.settings[UO_pos_conditional].tp);
+         newlines_chunk_pos(CT_QUESTION, cpd.settings[UO_pos_conditional].tp);
+      }
       if (cpd.settings[UO_pos_comma].tp != TP_IGNORE)
       {
          newlines_chunk_pos(CT_COMMA, cpd.settings[UO_pos_comma].tp);
@@ -1234,6 +1395,13 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
          add_long_closebrace_comment();
       }
 
+      /* Insert trailing comments after certain preprocessor conditional blocks */
+      if ((cpd.settings[UO_mod_add_long_ifdef_else_comment].n > 0) ||
+          (cpd.settings[UO_mod_add_long_ifdef_endif_comment].n > 0))
+      {
+         add_long_preprocessor_conditional_block_comment();
+      }
+
       /**
        * Aligning everything else and reindent
        */
@@ -1280,21 +1448,47 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
          output_parsed(p_file);
          fclose(p_file);
       }
+      else
+      {
+         LOG_FMT(LERR, "%s: Failed to open '%s' for write: %s (%d)\n",
+                 __func__, parsed_file, strerror(errno), errno);
+      }
    }
 
    uncrustify_end();
 }
 
+
 static void uncrustify_end()
 {
    /* Free all the memory */
    chunk_t *pc;
+
    while ((pc = chunk_get_head()) != NULL)
    {
       chunk_del(pc);
    }
+
+   if (cpd.bom != NULL)
+   {
+      chunk_del(cpd.bom);
+      cpd.bom = NULL;
+   }
+
+   /* Clean up some state variables */
+   cpd.unc_off     = false;
+   cpd.al_cnt      = 0;
+   cpd.did_newline = true;
+   cpd.frame_count = 0;
+   cpd.pp_level    = 0;
+   cpd.changes     = 0;
+   cpd.in_preproc  = CT_NONE;
+   cpd.consumed    = false;
+   memset(cpd.le_counts, 0, sizeof(cpd.le_counts));
+   cpd.preproc_ncnl_count = 0;
 }
 
+
 const char *get_token_name(c_token_t token)
 {
    if ((token >= 0) && (token < (int)ARRAY_SIZE(token_names)) &&
@@ -1305,9 +1499,10 @@ const char *get_token_name(c_token_t token)
    return("???");
 }
 
+
 /**
  * Grab the token id for the text.
- * returns CT_NONE on failure t match
+ * returns CT_NONE on failure to match
  */
 c_token_t find_token_name(const char *text)
 {
@@ -1326,6 +1521,7 @@ c_token_t find_token_name(const char *text)
    return(CT_NONE);
 }
 
+
 static bool ends_with(const char *filename, const char *tag)
 {
    int len1 = strlen(filename);
@@ -1338,6 +1534,7 @@ static bool ends_with(const char *filename, const char *tag)
    return(false);
 }
 
+
 struct file_lang
 {
    const char *ext;
@@ -1385,6 +1582,7 @@ const char *get_file_extension(int& idx)
    return(val);
 }
 
+
 /**
  * Find the language for the file extension
  * Default to C
@@ -1406,6 +1604,7 @@ static int language_from_filename(const char *filename)
    return(LANG_C);
 }
 
+
 /**
  * Find the language for the file extension
  *
@@ -1426,6 +1625,7 @@ static int language_from_tag(const char *tag)
    return(0);
 }
 
+
 /**
  * Gets the tag text for a language
  *
@@ -1456,19 +1656,20 @@ static const char *language_to_string(int lang)
    return("???");
 }
 
-void log_pcf_flags(log_sev_t sev, UINT32 flags)
+
+void log_pcf_flags(log_sev_t sev, UINT64 flags)
 {
    if (!log_sev_on(sev))
    {
       return;
    }
 
-   log_fmt(sev, "[0x%X:", flags);
+   log_fmt(sev, "[0x%" PRIx64 ":", flags);
 
    const char *tolog = NULL;
    for (int i = 0; i < (int)ARRAY_SIZE(pcf_names); i++)
    {
-      if ((flags & (1 << i)) != 0)
+      if ((flags & (1ULL << i)) != 0)
       {
          if (tolog != NULL)
          {
