@@ -1,8 +1,6 @@
 /**
  * @file universalindentgui.cpp
  * Exports the config file for UniversalIndentGUI
- *
- * $Id: universalindentgui.cpp 1386 2008-10-18 00:24:11Z bengardner $
  */
 #include "prototypes.h"
 #include "uncrustify_version.h"
@@ -229,7 +227,8 @@ void print_universal_indent_cfg(FILE *pfile)
                // EditorType=multiple
                // Choices="pos_bool=ignore|pos_bool=lead|pos_bool=trail"
                fprintf(pfile, "EditorType=multiple\n");
-               fprintf(pfile, "Choices=\"%s=ignore|%s=lead|%s=trail\"\n",
+               fprintf(pfile, "Choices=\"%s=ignore|%s=lead|%s=lead_break|%s=lead_force|%s=trail|%s=trail_break|%s=trail_force\"\n",
+                       option->name, option->name, option->name, option->name,
                        option->name, option->name, option->name);
                fprintf(pfile, "ChoicesReadable=\"Ignore %s|Lead %s|Trail %s\"\n",
                        optionNameReadable, optionNameReadable, optionNameReadable);
@@ -237,16 +236,16 @@ void print_universal_indent_cfg(FILE *pfile)
                break;
 
             case AT_STRING:
-            {
-               fprintf(pfile, "CallName=%s=\n", option->name);
-               fprintf(pfile, "EditorType=string\n");
-               std::string val_string;
-               const char  *val_str;
-               val_string = op_val_to_string(option->type, cpd.settings[option->id]);
-               val_str    = val_string.c_str();
-               fprintf(pfile, "ValueDefault=%s\n", val_str);
-            }
-            break;
+               {
+                  fprintf(pfile, "CallName=%s=\n", option->name);
+                  fprintf(pfile, "EditorType=string\n");
+                  std::string val_string;
+                  const char  *val_str;
+                  val_string = op_val_to_string(option->type, cpd.settings[option->id]);
+                  val_str    = val_string.c_str();
+                  fprintf(pfile, "ValueDefault=%s\n", val_str);
+               }
+               break;
 
             default:
                break;
