@@ -4,8 +4,6 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: align.cpp 365 2006-07-27 02:40:27Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -22,6 +20,7 @@ static_inline bool is_past_width(chunk_t *pc)
    return((pc->column + pc->len) > cpd.settings[UO_code_width].n);
 }
 
+
 /**
  * Split right after the chunk
  */
@@ -38,6 +37,7 @@ static void split_before_chunk(chunk_t *pc)
    }
 }
 
+
 /**
  * Step forward until a token goes beyond the limit and then call split_line()
  * to split the line at or before that point.
@@ -60,6 +60,7 @@ void do_code_width(void)
    }
 }
 
+
 struct cw_entry
 {
    chunk_t *pc;
@@ -99,6 +100,7 @@ static int get_split_pri(c_token_t tok)
    return(0);
 }
 
+
 /**
  * Checks to see if pc is a better spot to split.
  * This should only be called going BACKWARDS (ie prev)
@@ -151,6 +153,7 @@ static void try_split_here(cw_entry& ent, chunk_t *pc)
    }
 }
 
+
 /**
  * Scan backwards to find the most appropriate spot to split the line
  * and insert a newline.
@@ -187,7 +190,7 @@ static void split_line(chunk_t *start)
       return;
    }
 
-   /* Check to see if we are in a for statment */
+   /* Check to see if we are in a for statement */
    if ((start->flags & PCF_IN_FOR) != 0)
    {
       LOG_FMT(LSPLIT, " ** FOR SPLIT **\n");
@@ -254,11 +257,16 @@ static void split_line(chunk_t *start)
 
    /* Break before the token instead of after it according to the pos_xxx rules */
    if ((chunk_is_token(ent.pc, CT_ARITH) &&
-        (cpd.settings[UO_pos_arith].tp == TP_LEAD)) ||
+        (cpd.settings[UO_pos_arith].tp & TP_LEAD)) ||
        (chunk_is_token(ent.pc, CT_ASSIGN) &&
-        (cpd.settings[UO_pos_assign].tp == TP_LEAD)) ||
+        (cpd.settings[UO_pos_assign].tp & TP_LEAD)) ||
+       (chunk_is_token(ent.pc, CT_COMPARE) &&
+        (cpd.settings[UO_pos_compare].tp & TP_LEAD)) ||
+       ((chunk_is_token(ent.pc, CT_COND_COLON) ||
+         chunk_is_token(ent.pc, CT_QUESTION)) &&
+        (cpd.settings[UO_pos_conditional].tp & TP_LEAD)) ||
        (chunk_is_token(ent.pc, CT_BOOL) &&
-        (cpd.settings[UO_pos_bool].tp == TP_LEAD)))
+        (cpd.settings[UO_pos_bool].tp & TP_LEAD)))
    {
       pc = ent.pc;
    }
@@ -285,8 +293,9 @@ static void split_line(chunk_t *start)
    }
 }
 
+
 /**
- * A for statment is too long.
+ * A for statement is too long.
  * Step backwards and forwards to find the semicolons
  * Try splitting at the semicolons first.
  * If that doesn't work, then look for a comma at paren level.
@@ -300,7 +309,7 @@ static void split_for_stmt(chunk_t *start)
    chunk_t *st[2];
    chunk_t *pc;
    chunk_t *open_paren = NULL;
-   int     nl_cnt = 0;
+   int     nl_cnt      = 0;
 
    LOG_FMT(LSPLIT, "%s: starting on %.*s, line %d\n",
            __func__, start->len, start->str, start->orig_line);
@@ -394,6 +403,7 @@ static void split_for_stmt(chunk_t *start)
    /* Oh, well. We tried. */
 }
 
+
 /**
  * Splits the parameters at every comma that is at the fparen level.
  *
@@ -429,6 +439,7 @@ static void split_fcn_params_full(chunk_t *start)
    }
 }
 
+
 /**
  * Figures out where to split a function def/proto/call
  *
