@@ -4,6 +4,10 @@ class Foo : public Bar
 
 };
 
+#define CTOR(i, _) : T(X()), \
+    y() \
+{ }
+
 class Foo2 :
 public Bar
 {
