@@ -22,3 +22,12 @@ struct foo {
    char *b
 };
 
+void f()
+{
+   for (;;)
+   {
+      nothing();
+   }
+   foobar();
+}
+;
