@@ -56,3 +56,23 @@ int foo2(void)
 {
    return -1;
 }
+
+void foo(void)
+{
+   if (!value
+#ifdef OPTION
+       || (value == SOMECONST)
+#endif /* comment */
+       )
+   {
+   }
+}
+
+void foo3(void)
+{
+   if ((*p == '-') && (p[1] != ']') ?
+       (*text <= *++p) && (*text >= last) : (*text == *p))
+   {
+      matched = TRUE;
+   }
+}
