@@ -1,5 +1,8 @@
 #include <list>
 #include <map>
+#include <vector>
+
+#define MACRO(T)    f<T>()
 
 class MyClass
 {
@@ -147,3 +150,11 @@ template<class T> class Y {
    template<class V> void f(V);
 };
 
+void                        (*foobar)(void) = NULL;
+std::vector<void (*)(void)> functions;
+
+#define MACRO(a)    a
+template<typename = int> class X;
+MACRO(void f(X<>& x));
+void g(X<>& x);
+
