@@ -5,8 +5,6 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: windows_compat.h 1004 2007-11-08 03:34:27Z bengardner $
  */
 #ifndef WINDOWS_COMPAT_H_INCLUDED
 #define WINDOWS_COMPAT_H_INCLUDED
@@ -24,6 +22,11 @@ typedef int                INT32;
 typedef unsigned char      UINT8;
 typedef unsigned short     UINT16;
 typedef unsigned int       UINT32;
+typedef unsigned long long UINT64;
+
+#ifndef PRIx64
+#define PRIx64             "llx"
+#endif
 
 /* eliminate GNU's attribute */
 #define __attribute__(x)
